﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;           // Debug.WriteLine出力用
using System.Runtime.InteropServices;
using System.IO;
using System.Reflection;

namespace RTedge
{
    public partial class edge_API
    {
        [DllImport("Kernel32.dll")]
        static extern bool SetDllDirectory(string lpPathName);

        public edge_FW EGFW = null;
        /// <summary>
        /// setDllLocation 現在のdllの位置を作業ディレクトリとしてセットする
        /// </summary>
        /// <returns></returns>
        static bool setDllLocation()
        {
            string dllDir;
            string filePath;
            dllDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);        // step1: 現在のdllの位置
            filePath = dllDir + "\\eghgapiWin.dll";
            if (Directory.Exists(dllDir) && File.Exists(filePath))
            {
                SetDllDirectory(dllDir);
                return true;
            }
            dllDir = Environment.GetEnvironmentVariable("RTEDGE"); ;                    // step2: 環境変数RTEDGE位置
            filePath = dllDir + "eghgapiWin.dll";
            if (Directory.Exists(dllDir))
            {
                SetDllDirectory(dllDir);
                return true;
            }
            dllDir = Environment.GetEnvironmentVariable("RTEDGE"); ;                    // step3: 環境変数RTEDGE + binの位置
            filePath = dllDir + "bin\\eghgapiWin.dll";
            if (Directory.Exists(dllDir))
            {
                SetDllDirectory(dllDir);
                return true;
            }

            return false;
        }
        /// <summary>
        /// Constructors
        /// </summary>
        public edge_API()
        {
            setDllLocation();
        }
        /// <summary>
        /// EgInit 引数無 デフォルト値での実行
        /// </summary>
        /// <returns></returns>
        public int EgInit()
        {
            int ret = -1;
            try
            {
                EDGECONFIG config = new EDGECONFIG();   //C側のInitを呼び出す前処理(引数が無いのでデフォルト指定)
                config = EDGE_CONFIG_DEFAULT;
                return EgInit(config);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// EgInit Config指定
        /// </summary>
        /// <param name="config"></param>
        /// <returns></returns>
        public int EgInit(edge_API.EDGECONFIG config)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                EGFW = new edge_FW();
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(edge_API._EDGECONFIG)));
                }
                //ログ出力
                EGFW.PrintLog("DBG", "Start Win Initialize");

                edge_API._EDGECONFIG _config = new edge_API._EDGECONFIG();
                fnCopyConfig(config, ref _config);
                if (config.mInstMessFunc != null)
                {
                    //各コールバック関数追加(内部処理)
                    EGFW.mInstMessFunc = new InstanceMessageHandler(config.mInstMessFunc);
                    EGFW.mCallbackMessFuncEx = new edge_API.CallbackDelegateEx(EGFW.UserMessageWrap);
                }
                _config.mCallbackMessFuncEx = EGFW.mCallbackMessFuncEx;

                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(_config, pPtr, false);

                //初期化
                ret = _C_EgInit(pPtr);
                if(ret == 0)
                {
                    ret = EGFW.Initialize(_config);
                    if(ret != 0)
                        EGFW.PrintLog("ERR", "Win EGFW Initilize ret = {0}", ret.ToString());

                }
                else
                    EGFW.PrintLog("ERR", "Win C_EgInit ret = {0}", ret.ToString());

                //ログ出力
                EGFW.PrintLog("DBG", "End Win Initialize");
                return ret;
            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog("N/A","FER", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
                //ログ出力
                edge_FW._PrintSysLog("N/A","DBG", "End Win Initialize");
            }
        }
        /// <summary>
        /// EgFinalize CのFinalize処理を呼び出す
        /// </summary>
        /// <returns></returns>
        public int EgFinalize()
        {
            int ret = -1;
            try
            {
                edge_FW._PrintSysLog("N/A", "DBG", "Start Win EgFinalize");
                if (EGFW != null)
                    EGFW.Dispose();
                ret = _C_EgFinalize();
                //最後にキャッシュで開いているメールボックス全て閉じる エラーは無視
                _EgMailboxCloseAll();
                edge_FW._PrintSysLog("N/A", "DBG", "End Win EgFinalize ret = {0}", ret.ToString());
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                edge_FW._PrintSysLog("N/A", "FER", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        ///
        /// ローカル使用 サービス側で指定されたConfigをCに送るための形に変更
        ///
        protected void fnCopyConfig(edge_API.EDGECONFIG config, ref edge_API._EDGECONFIG _config)
        {
            try
            {
                _config.bDebug = config.bDebug;
                _config.bReadXmlF = config.bReadXmlF;
                _config.shPriority = config.shPriority;
                _config.Path = config.Path;
            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog("N/A", "FER", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// 二重起動禁止処理
        /// </summary>
        /// <returns></returns>
        public static bool fnBootCheck()
        {
            //二重起動禁止処理
            if (System.Diagnostics.Process.GetProcessesByName(
                System.Diagnostics.Process.GetCurrentProcess().ProcessName).Length > 1)
            {

                //すでに起動していると判断して終了
                return (false);
            }
            return (true);
        }


        //-------------------------------------------------------------------------------
        //
        // RTCD
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// RTCD初期化
        /// </summary>
        /// <param name="NumMax"></param>
        /// <returns></returns>
        public int EgSetRTCD(string stName)
        {
            int ret = -1;
            try
            {
                //呼び出し
                ret = _C_EgSetRTCD(stName);

                if (ret != 0)
                {
                    string mess = "EgSetRTCD Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// EgInit Config指定
        /// </summary>
        /// <param name="config"></param>
        /// <returns></returns>
        public int _EgSysDefSetting(edge_API.EDGESYSTEMCONFIG sysconfig)
        {
            int ret = -1;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                EGFW = new edge_FW();
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(edge_API.EDGESYSTEMCONFIG)));
                }
                //ログ出力
                EGFW.PrintLog("DBG", "Start System Settings");
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(sysconfig, pPtr, false);
                int size = Marshal.SizeOf(typeof(edge_API.EDGESYSTEMCONFIG));
                //初期化
                ret = _C__EgSysDefSetting(pPtr);
                if (ret != 0)
                  EGFW.PrintLog("ERR", "Win EgSysDefSetting ret = {0}", ret.ToString());

                //ログ出力
                EGFW.PrintLog("DBG", "End System Settings");
                return ret;
            }
            catch (Exception ex)
            {
                edge_FW._PrintSysLog("N/A", "FER", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
                //ログ出力
                edge_FW._PrintSysLog("N/A", "DBG", "End System Settings");
            }
        }
        //-------------------------------------------------------------------------------
        //
        // EgTag
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// Tag初期化（ネームスペース生成）
        /// </summary>
        /// <param name="NumMax"></param>
        /// <returns></returns>
        public int EgTagInit(uint NumMax)
        {
            int ret = -1;
            try
            {
                //サイズチェック
                if (NumMax == 0 ) //最大値チェックは行わない メモリ容量に合わせる
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                //呼び出し
                ret = _C_EgTagInit(NumMax);

                if (ret != 0)
                {
                    string mess = "EgTagInit Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// Tagの追加
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Type"></param>
        /// <param name="Size"></param>
        /// <param name="stSource"></param>
        /// <returns></returns>
        public int EgTagCreate(string stName, ushort Type, ushort Size, string stSource, string stComment)
        {   
            int ret = -1;
            try
            {
                //長さチェック
                if (stName.Length > EGDEFINE.EG_MAX_TAGNAME || stSource.Length > EGDEFINE.EG_MAX_TAGADDRESS)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                //呼び出し
                ret = _C_EgTagCreate(stName, Type, Size, stSource, stComment);

                if (ret != 0)
                {
                    string mess = "EgTagCreate Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// Tagの追加(サイズ指定なし版)
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Type"></param>
        /// <param name="stSource"></param>
        /// <returns></returns>
        public int EgTagCreateEx(string stName, ushort Type, string stSource, string stComment)
        {
            int ret = -1;
            try
            {
                //長さチェック
                if (stName.Length > EGDEFINE.EG_MAX_TAGNAME || stSource.Length > EGDEFINE.EG_MAX_TAGADDRESS)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                //呼び出し
                ret = _C_EgTagCreateEx(stName, Type, stSource, stComment);

                if (ret != 0)
                {
                    string mess = "EgTagCreate Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// Tagの追加(タグ名重複対応版)
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="Type"></param>
        /// <param name="stSource"></param>
        /// <returns></returns>
        public int EgTagCreateSp(string stName, ushort Type, string stSource, string stComment, ref object regName, ref ushort isLink)
        {
            int ret = -1;
            IntPtr regNamePtr = IntPtr.Zero;
            try
            {
                //長さチェック
                if (stName.Length > EGDEFINE.EG_MAX_TAGNAME || stSource.Length > EGDEFINE.EG_MAX_TAGADDRESS)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                int dataSize = EGDEFINE.EG_MAX_TAGNAME + 1;
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.String;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    regNamePtr = Marshal.AllocCoTaskMem(dataSize);
                }

                //Intptrが確保できている場合実行
                if (regNamePtr != IntPtr.Zero)
                {
                    //呼び出し
                    ret = _C_EgTagCreateSp(stName, Type, stSource, stComment, regNamePtr, (uint)dataSize, ref isLink);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(regNamePtr, bytes, 0, dataSize);
                        //タグ登録名を取得
                        regName = egCommonFunc.GetTagAnyData(bytes, usType);
                    }
                    else
                    {
                        string mess = "EgTagCreate Error " + ret.ToString();
                        Debug.WriteLine(mess);
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (regNamePtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(regNamePtr);
            }
        }

        /// <summary>
        /// 書き込みデータをバイト配列に変換する
        /// </summary>
        /// <param name="value">データ</param>
        /// <param name="outByte">バイト配列</param>
        private void fnConvertOutByteData(object value, out byte[] outByte)
        {
            if (value is string)                        // 文字列
            {
                // outByteは文字列+NULLをセット
                outByte = System.Text.Encoding.ASCII.GetBytes((string)value);
                Array.Resize(ref outByte, outByte.Length + 1);
                outByte[outByte.Length - 1] = (byte)'\0';        // NULLを付加
            }
            else if (value is byte[])                   // バイト配列
            {
                outByte = (byte[])value;
            }
            else                                        // 上記以外
            {
                // 上記以外はbyte配列に変換
                outByte = BitConverter.GetBytes((dynamic)(value));
            }
        }

        /// <summary>
        /// タグの書き込み
        /// </summary>
        /// <param name="name"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgTagWrite(string name, object value)
        {
            int ret = -1;
            IntPtr writeBuffPtr = IntPtr.Zero;
            try
            {
                byte[] outByte;

                // バイト配列に変換する
                fnConvertOutByteData(value, out outByte);

                // バイトサイズの算出
                int size = Marshal.SizeOf(typeof(byte)) * outByte.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //書き込むbuff作成
                    writeBuffPtr = Marshal.AllocCoTaskMem(size);
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.Copy(outByte, 0, writeBuffPtr, size);

                ret = _C_EgTagWrite(name, writeBuffPtr, (ushort)size);
                if (ret != 0)
                {
                    string mess = "EgTagWrite Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (writeBuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(writeBuffPtr);
            }
        }

        /// <summary>
        /// タグの書き込み(INDEX指定)
        /// </summary>
        /// <param name="index"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgTagWriteByIndex(EGOBJINDEX index, object value)
        {
            int ret = -1;
            IntPtr writeBuffPtr = IntPtr.Zero;
            IntPtr indexPtr = IntPtr.Zero;
            try
            {
                byte[] outByte;

                // バイト配列に変換する
                fnConvertOutByteData(value, out outByte);

                // バイトサイズの算出
                int size = Marshal.SizeOf(typeof(byte)) * outByte.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //書き込むbuff作成
                    writeBuffPtr = Marshal.AllocCoTaskMem(size);
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.Copy(outByte, 0, writeBuffPtr, size);
                Marshal.StructureToPtr(index, indexPtr, false);

                //EgTagWriteByIndex
                ret = _C_EgTagWriteByIndex(indexPtr, writeBuffPtr, (ushort)size);
                if (ret != 0)
                {
                    string mess = "EgTagWriteByIndex Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (writeBuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(writeBuffPtr);
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
            }
        }

        /// <summary>
        /// タグの書き込み(Entry指定)
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgTagWriteByEntry(IntPtr entry, object value)
        {
            int ret = -1;
            IntPtr writeBuffPtr = IntPtr.Zero;
            try
            {
                byte[] outByte;

                // バイト配列に変換する
                fnConvertOutByteData(value, out outByte);

                // バイトサイズの算出
                int size = Marshal.SizeOf(typeof(byte)) * outByte.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //書き込むbuff作成
                    writeBuffPtr = Marshal.AllocCoTaskMem(size);
                    //indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.Copy(outByte, 0, writeBuffPtr, size);
                //Marshal.StructureToPtr(index, indexPtr, false);

                //EgTagWriteByEntry
                ret = _C_EgTagWriteByEntry(entry, writeBuffPtr, (ushort)size);
                if (ret != 0)
                {
                    string mess = "EgTagWriteByEntry Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (writeBuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(writeBuffPtr);
                //if (indexPtr != IntPtr.Zero)
                //    Marshal.FreeCoTaskMem(indexPtr);
            }
        }

        /// <summary>
        /// タグの値読込
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="value"></param>
        /// <param name="byteF"></param>
        /// <returns></returns>
        public int EgTagRead(string stName, ref object value, bool byteF = false)
        {
            int ret = -1;
            IntPtr ReadBuffPtr = IntPtr.Zero;
            try
            {
                object rtnObj = null;
                //タグのサイズ取得
                ret = EgTagGetProperty(stName, "Size", ref rtnObj);
                if (ret != 0)
                {
                    string mess = "EgTagRead Error edgeTagGetSize = " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                ushort usSize = (ushort)rtnObj;

                //タグのタイプ取得
                ret = EgTagGetProperty(stName, "Type", ref rtnObj);
                if (ret != 0)
                {
                    string mess = "EgTagRead Error edgeTagGetType = " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                ushort usType = (ushort)rtnObj;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    ReadBuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                //EgTagRead
                ret = _C_EgTagRead(stName, ReadBuffPtr, usSize);
                if (ret == 0)
                {
                    byte[] bytes = new byte[usSize];
                    Marshal.Copy(ReadBuffPtr, bytes, 0, usSize);
                    //byteフラグがある場合はbyte配列のまま
                    if (byteF)
                        value = bytes;
                    else
                        value = egCommonFunc.GetTagAnyData(bytes, (EGDEFINE.egTagDataType)usType);  //各タイプに基づいてデータを変換
                }
                else
                {
                    string mess = "EgTagRead Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (ReadBuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(ReadBuffPtr);
            }
        }

        /// <summary>
        /// タグの値読込(INDEX指定)
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="value"></param>
        /// <param name="byteF"></param>
        /// <returns></returns>
        public int EgTagReadByIndex(EGOBJINDEX index, ref object value, bool byteF = false)
        {
            int ret = -1;
            IntPtr ReadBuffPtr = IntPtr.Zero;
            IntPtr indexPtr = IntPtr.Zero;
            try
            {
                object rtnObj = null;
                //タグのサイズ取得
                ret = EgTagGetPropertyByIndex(index, "Size", ref rtnObj);
                if (ret != 0)
                {
                    string mess = "EgTagRead Error edgeTagGetSize = " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                ushort usSize = (ushort)rtnObj;

                //タグのタイプ取得
                ret = EgTagGetPropertyByIndex(index, "Type", ref rtnObj);
                if (ret != 0)
                {
                    string mess = "EgTagRead Error edgeTagGetType = " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                ushort usType = (ushort)rtnObj;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // メモリ確保
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                    ReadBuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(index, indexPtr, false);

                //EgTagReadByIndex
                ret = _C_EgTagReadByIndex(indexPtr, ReadBuffPtr, usSize);
                if (ret == 0)
                {
                    byte[] bytes = new byte[usSize];
                    Marshal.Copy(ReadBuffPtr, bytes, 0, usSize);
                    //byteフラグがある場合はbyte配列のまま
                    if (byteF)
                        value = bytes;
                    else
                        value = egCommonFunc.GetTagAnyData(bytes, (EGDEFINE.egTagDataType)usType);  //各タイプに基づいてデータを変換
                }
                else
                {
                    string mess = "EgTagReadByIndex Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (ReadBuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(ReadBuffPtr);
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
            }
        }

        /// <summary>
        /// タグの値読込(Entry指定)
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="value"></param>
        /// <param name="byteF"></param>
        /// <returns></returns>
        public int EgTagReadByEntry(IntPtr entry, ref object value, bool byteF = false)
        {
            int ret = -1;
            IntPtr ReadBuffPtr = IntPtr.Zero;
            try
            {
                object rtnObj = null;
                //タグのサイズ取得
                ret = EgTagGetPropertyByEntry(entry, "Size", ref rtnObj);
                if (ret != 0)
                {
                    string mess = "EgTagRead Error edgeTagGetSize = " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                ushort usSize = (ushort)rtnObj;

                //タグのタイプ取得
                ret = EgTagGetPropertyByEntry(entry, "Type", ref rtnObj);
                if (ret != 0)
                {
                    string mess = "EgTagRead Error edgeTagGetType = " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                ushort usType = (ushort)rtnObj;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // メモリ確保
                    ReadBuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                //EgTagReadByEntry
                ret = _C_EgTagReadByEntry(entry, ReadBuffPtr, usSize);
                if (ret == 0)
                {
                    byte[] bytes = new byte[usSize];
                    Marshal.Copy(ReadBuffPtr, bytes, 0, usSize);
                    //byteフラグがある場合はbyte配列のまま
                    if (byteF)
                        value = bytes;
                    else
                        value = egCommonFunc.GetTagAnyData(bytes, (EGDEFINE.egTagDataType)usType);  //各タイプに基づいてデータを変換
                }
                else
                {
                    string mess = "EgTagReadByIndex Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (ReadBuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(ReadBuffPtr);
            }
        }

        /* ﾌﾟﾛﾊﾟﾃｨに対応する型を返す */
        /* @memo ﾀﾌﾟﾙが使えないため、out引数での出力(ﾀﾌﾟﾙは.NET FW 4.7～) */
        protected static void _EgTagGetProperty_GetType(string stPropName, out Type clr_type, out int dataSize)
        {
            /* @memo C側はﾌﾟﾛﾊﾟﾃｨ名をﾊｯｼｭ値で比較しているため、大文字小文字の厳密な一致が必要 */
            /*       ここでも、それに合わせてﾌﾟﾛﾊﾟﾃｨ名をﾊﾞｲﾅﾘ比較とする */
            dataSize = -1;
            switch (stPropName)
            {
                case "Type":
                case "Size":
                    clr_type = typeof(UInt16); break;
                case "Comment":
                    clr_type = typeof(string); dataSize = EGDEFINE.EG_MAX_TAGCOMMENT + 1; break;
                case "Address":
                case "Source":
                    clr_type = typeof(string); dataSize = EGDEFINE.EG_MAX_TAGADDRESS + 1; break;
                case "Value":
                    goto default; /* 未対応(ﾀｸﾞ毎に型が違う) */
                case "Entry":
                    clr_type = typeof(IntPtr); break;
                case "Index":   
                    clr_type = typeof(IntPtr); break; /* @memo EGOBJINDEXのｱﾄﾞﾚｽだけ取れてもなぁ… */
                case "Hash":
                    clr_type = typeof(HASHCODE); break;
                case "Name":
                    clr_type = typeof(string); dataSize = EGDEFINE.EG_MAX_TAGNAME + 1; break;
                case "Count":
                    clr_type = typeof(byte); break;
                case "RecordCount":
                    clr_type = typeof(UInt16); break;
                case "Remain":
                    clr_type = typeof(UInt16); break;
                default:
                    clr_type = null; dataSize = 0; break; /* 未対応 */
            }
            if (dataSize < 0) dataSize = Marshal.SizeOf(clr_type);
        }

        /// <summary>
        /// EgTagGetProperty
        /// </summary>
        /// <param name="stTagName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgTagGetProperty(string stTagName, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                Type clr_type;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    _EgTagGetProperty_GetType(stPropName, out clr_type, out dataSize);
                    if (dataSize > 0) propptr = Marshal.AllocCoTaskMem(dataSize);
                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Tagの情報取得
                    ret = _C_EgTagGetProperty(stTagName, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        //if (structF == false)
                        /* @memo どうせobject型で返す為、PtrToStructure戻り値の型をｷｬｽﾄする必要はない */
                        value = (clr_type == typeof(string))
                            ? Marshal.PtrToStringAnsi(propptr)
                            : Marshal.PtrToStructure(propptr, clr_type);
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }
        /// <summary>
        /// EgTagGetPropertyByIndex
        /// </summary>
        /// <param name="index"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgTagGetPropertyByIndex(EGOBJINDEX index, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr propPtr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                Type clr_type;
                GCHandle gch = GCHandle.Alloc(index, GCHandleType.Pinned);

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    _EgTagGetProperty_GetType(stPropName, out clr_type, out dataSize);
                    if (dataSize > 0) propPtr = Marshal.AllocCoTaskMem(dataSize);
                }
                //Intptrが確保できている場合実行
                if (propPtr != IntPtr.Zero)
                {
                    try
                    {
                        //Tagの情報取得
                        ret = _C_EgTagGetPropertyByIndex(gch.AddrOfPinnedObject(), stPropName, propPtr, (uint)dataSize);
                        if (ret == 0)
                        {
                            /* @memo どうせobject型で返す為、PtrToStructure戻り値の型をｷｬｽﾄする必要はない */
                            value = (clr_type == typeof(string))
                                ? Marshal.PtrToStringAnsi(propPtr)
                                : Marshal.PtrToStructure(propPtr, clr_type);
                        }
                    }
                    finally
                    {
                        gch.Free();
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (propPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propPtr);
            }
        }
        /// <summary>
        /// EgTagGetPropertyByEntry
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgTagGetPropertyByEntry(IntPtr entry, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr propPtr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                Type clr_type;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    _EgTagGetProperty_GetType(stPropName, out clr_type, out dataSize);
                    if (dataSize > 0) propPtr = Marshal.AllocCoTaskMem(dataSize);
                }
                //Intptrが確保できている場合実行
                if (propPtr != IntPtr.Zero)
                {
                    //Tagの情報取得
                    ret = _C_EgTagGetPropertyByEntry(entry, stPropName, propPtr, (uint)dataSize);
                    if (ret == 0)
                    {
                        //if (structF == false)
                        /* @memo どうせobject型で返す為、PtrToStructure戻り値の型をｷｬｽﾄする必要はない */
                        value = (clr_type == typeof(string))
                            ? Marshal.PtrToStringAnsi(propPtr)
                            : Marshal.PtrToStructure(propPtr, clr_type);
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (propPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propPtr);
            }
        }
        /// <summary>
        /// タグの登録数取得
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgTagGetCount(ref uint itemNum)
        {
            int ret = -1;
            try
            {
                //EgTagGetCount
                ret = _C_EgTagGetCount(ref itemNum);
                if (ret != 0)
                {
                    string mess = "EgTagGetCount Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// タグの一覧取得
        /// </summary>
        /// <param name="indexArry"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgTagGetEnum(ref EGOBJINDEX[] indexArry, ref uint itemNum)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            int indexSize = Marshal.SizeOf(typeof(EGOBJINDEX));
            try
            {
                int size = indexArry.Count() * indexSize;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    inputPtr = Marshal.AllocCoTaskMem(size);
                }

                //EgTagGetEnum
                ret = _C_EgTagGetEnum(inputPtr, (uint)size, ref itemNum);
                if (ret == 0)
                {
                    for (int i = 0; i < itemNum; i++)
                    {
                        indexArry[i] = (EGOBJINDEX)Marshal.PtrToStructure(inputPtr + (indexSize * i), typeof(EGOBJINDEX));
                    }
                }
                else
                {
                    string mess = "EgTagGetEnum Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }

        /// <summary>
        /// タグ初期値セット
        /// </summary>
        /// <returns></returns>
        public int EgTagSetDefaultValue()
        {
            int ret = -1;
            try
            {
                //呼び出し
                ret = _C_EgTagSetDefaultValue();

                if (ret != 0)
                {
                    string mess = "EgTagSetDefaultValue Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// タグ初期値セット
        /// </summary>
        /// <returns></returns>
        public int EgTagSetDefaultValueByGroup(UInt16 wGroup)
        {
            int ret = -1;
            try
            {
                //呼び出し
                ret = _C_EgTagSetDefaultValueByGroup(wGroup);

                if (ret != 0)
                {
                    string mess = "EgTagSetDefaultValueByGroup Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        //-------------------------------------------------------------------------------
        //
        // EgDataset
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// EgDatasetCreate
        /// </summary>
        /// <param name="stName"></param>
        /// <returns></returns>
        public int EgDatasetCreate(string stName)
        {
            int ret = -1;
            try
            {
                //長さチェック データセット名称の長さ最大 EG_MAX_DSETNAME
                if (stName.Length > EGDEFINE.EG_MAX_DSETNAME)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                //呼び出し
                ret = _C_EgDatasetCreate(stName);
                if (ret != 0)
                {
                    string mess = "EgDatasetCreate Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// EgDatasetAddTag
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stTagName"></param>
        /// <returns></returns>
        public int EgDatasetAddTag(string stName, string stTagName)
        {
            int ret = -1;
            try
            {
                //引数チェック
                if (stName.Length == 0 || stTagName.Length == 0)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                //呼び出し
                ret = _C_EgDatasetAddTag(stName, stTagName);
                if (ret != 0)
                {
                    string mess = "EgDatasetAddTag Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// EgDatasetGetFirst
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stDslink"></param>
        /// <returns></returns>
        public int EgDatasetGetFirst(string stName, ref TDSLINK stDslink)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(TDSLINK)));
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(stDslink, inputPtr, false);
                ret = _C_EgDatasetGetFirst(stName, inputPtr);
                if (ret != 0)
                {
                    string mess = "_C_EgDatasetGetFirst Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                else
                stDslink = (TDSLINK)Marshal.PtrToStructure(inputPtr, typeof(TDSLINK));
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }

        /// <summary>
        /// EgDatasetGetNext
        /// ※第一引数が不要になったが、互換性の為残しておく
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stDslink"></param>
        /// <returns></returns>
        public int EgDatasetGetNext(string stName, ref TDSLINK stDslink)
        {
            int ret = -1;
            try
            {
                return EgDatasetGetNext(ref stDslink);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// EgDatasetGetNext
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stDslink"></param>
        /// <returns></returns>
        public int EgDatasetGetNext(ref TDSLINK stDslink)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(TDSLINK)));
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(stDslink, inputPtr, false);
                ret = _C_EgDatasetGetNext(inputPtr);
                if (ret != 0)
                {
                    string mess = "_C_EgDatasetGetFirst Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                else //Dslinkに変換
                    stDslink = (TDSLINK)Marshal.PtrToStructure(inputPtr, typeof(TDSLINK));

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }

        /// <summary>
        /// EgDatasetGetPropertyByIndex
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgDatasetGetPropertyByIndex(EGOBJINDEX index, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr indexPtr = IntPtr.Zero;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                //(Tagじゃないけど同じﾛｼﾞｯｸなので)
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.UNDEFINE;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    //ItemNumの場合
                    if (String.Compare(stPropName, "ItemNum", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(uint));
                        usType = EGDEFINE.egTagDataType.UInt32;
                    }
                    //Nameの取得
                    else if (String.Compare(stPropName, "Name", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_DSETNAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }

                    if (dataSize > 0)
                        propptr = Marshal.AllocCoTaskMem(dataSize);

                    // メモリ確保
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                    // マネージド構造体をアンマネージドにコピーする
                    Marshal.StructureToPtr(index, indexPtr, false);
                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Datasetの情報取得
                    ret = _C_EgDatasetGetPropertyByIndex(indexPtr, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(propptr, bytes, 0, dataSize);
                        //型にあったデータを取得(Tagじゃないけど同じﾛｼﾞｯｸなので)
                        value = egCommonFunc.GetTagAnyData(bytes, usType);
                        if (value != null)
                            return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }

        /// <summary>
        /// Datasetの登録数取得
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgDatasetGetCount(ref uint itemNum)
        {
            int ret = -1;
            try
            {
                //EgDatasetGetCount
                ret = _C_EgDatasetGetCount(ref itemNum);
                if (ret != 0)
                {
                    string mess = "EgDatasetGetCount Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// Datasetの一覧取得
        /// </summary>
        /// <param name="indexArry"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgDatasetGetEnum(ref EGOBJINDEX[] indexArry, ref uint itemNum)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            int indexSize = Marshal.SizeOf(typeof(EGOBJINDEX));
            try
            {
                int size = indexArry.Count() * indexSize;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    inputPtr = Marshal.AllocCoTaskMem(size);
                }

                //EgDatasetGetEnum
                ret = _C_EgDatasetGetEnum(inputPtr, (uint)size, ref itemNum);
                if (ret == 0)
                {
                    for (int i = 0; i < itemNum; i++)
                    {
                        indexArry[i] = (EGOBJINDEX)Marshal.PtrToStructure(inputPtr + (indexSize * i), typeof(EGOBJINDEX));
                    }
                }
                else
                {
                    string mess = "EgDatasetGetEnum Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }
        /// <summary>
        /// EgDatasetGetBinary ... データセットの読込
        /// </summary>
        /// <param name="DatasetName"></param>
        /// <param name="Dest"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        public Int32 EgDatasetGetBinary(String DatasetName, Byte[] Dest, UInt32 size)
        {
            Int32 Ret = 0;
            IntPtr ptr = IntPtr.Zero;

            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();

                // アンマネージ配列のメモリを確保
                ptr = Marshal.AllocCoTaskMem((int)size);

                // メモリ確保出来ている場合実行
                if (ptr != IntPtr.Zero)
                {
                    // データセット読込実行
                    Ret = _C_EgDatasetGetBinary(DatasetName, ptr, size);
                    if (Ret != 0) // 失敗
                    {
                        return Ret;
                    }
                    // マネージ配列へコピー
                    Marshal.Copy(ptr, Dest, 0, (int)size);
                    // アンマネージ配列のメモリを解放
                    Marshal.FreeCoTaskMem(ptr);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            

            return Ret;
        }
        /// <summary>
        /// EgDatasetSetBinary
        /// </summary>
        /// <param name="DatasetName"></param>
        /// <param name="source"></param>
        /// <param name="sourceSize"></param>
        /// <returns></returns>
        public Int32 EgDatasetSetBinary(String DatasetName, Byte[] source, UInt32 sourceSize)
        {
            Int32 Ret = 0;
            IntPtr ptr = IntPtr.Zero;

            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();

                // アンマネージ配列のメモリを確保
                ptr = Marshal.AllocCoTaskMem((int)sourceSize);

                // メモリ確保出来ている場合実行
                if (ptr != IntPtr.Zero)
                {
                    // マネージ配列からコピー
                    Marshal.Copy(source, 0, ptr, (int)sourceSize);

                    // データセット読込実行
                    Ret = _C_EgDatasetSetBinary(DatasetName, ptr, sourceSize);
                    if (Ret != 0) // 失敗
                    {
                        return Ret;
                    }

                    // アンマネージ配列のメモリを解放
                    Marshal.FreeCoTaskMem(ptr);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return Ret;
        }

        /// <summary>
        /// EgDatasetGetSize ... データセットのサイズ取得
        /// </summary>
        /// <param name="DatasetName"></param>
        /// <returns></returns>
        public Int32 EgDatasetGetSize(String DatasetName)
        {
            return _C_EgDatasetGetSize(DatasetName);
        }
        //-------------------------------------------------------------------------------
        //
        // Collector
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// EgCollectorCreate
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="Interval"></param>
        /// <param name="Priority"></param>
        /// <param name="ColSize"></param>
        /// <param name="RowSize"></param>
        /// <param name="DestName"></param>
        /// <returns></returns>
        public int EgCollectorCreate(string Name, string OwnerServiceName, uint Interval, byte Priority, uint ColSize, uint RowSize, string DestName)
        {
            int ret = -1;
            try
            {
                //長さチェック
                if (Name.Length > EGDEFINE.EG_MAX_CLCTNAME || DestName.Length > EGDEFINE.EG_MAX_DSETNAME)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                //呼び出し
                ret = _C_EgCollectorCreate(Name, OwnerServiceName, Interval, Priority, ColSize, RowSize, DestName);

                if (ret != 0)
                {
                    string mess = "EgCollectorCreate Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// EgCollectorGetProperty
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgCollectorGetProperty(string stName, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                //(Tagじゃないけど同じﾛｼﾞｯｸなので)
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.UNDEFINE;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    //Type、またはSizeの場合
                    if (String.Compare(stPropName, "Interval", true) == 0 ||
                        String.Compare(stPropName, "ColSize", true) == 0 ||
                        String.Compare(stPropName, "RowSize", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(uint));
                        usType = EGDEFINE.egTagDataType.UInt32;

                    }
                    //OwnerNameの場合
                    else if (String.Compare(stPropName, "OwnerName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICENAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //DataSetNameの場合
                    else if (String.Compare(stPropName, "DataSetName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_DSETNAME+1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //Priorityの場合
                    else if (String.Compare(stPropName, "Priority", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(byte));
                        usType = EGDEFINE.egTagDataType.Byte;
                    }
                    //Hash値の取得(未実装 必要に応じて実装予定)
                    else if (String.Compare(stPropName, "Hash", true) == 0)
                    {
                    }
                    //Nameの取得(Name指定なので不要)
                    else if (String.Compare(stPropName, "Name", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_CLCTNAME+1;
                        usType = EGDEFINE.egTagDataType.String;
                    }

                    if (dataSize > 0)
                        propptr = Marshal.AllocCoTaskMem(dataSize);

                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Collectorの情報取得
                    ret = _C_EgCollectorGetProperty(stName, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(propptr, bytes, 0, dataSize);
                        //型にあったデータを取得(Tagじゃないけど同じﾛｼﾞｯｸなので)
                        value = egCommonFunc.GetTagAnyData(bytes, usType);
                        if (value != null)
                            return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }

        /// <summary>
        /// EgCollectorGetPropertyByIndex
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgCollectorGetPropertyByIndex(EGOBJINDEX index, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr indexPtr = IntPtr.Zero;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                //(Tagじゃないけど同じﾛｼﾞｯｸなので)
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.UNDEFINE;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    //Type、またはSizeの場合
                    if (String.Compare(stPropName, "Interval", true) == 0 ||
                        String.Compare(stPropName, "ColSize", true) == 0 ||
                        String.Compare(stPropName, "RowSize", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(uint));
                        usType = EGDEFINE.egTagDataType.UInt32;

                    }
                    //OwnerNameの場合
                    else if (String.Compare(stPropName, "OwnerName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICENAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //DataSetNameの場合
                    else if (String.Compare(stPropName, "DataSetName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_DSETNAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //Priorityの場合
                    else if (String.Compare(stPropName, "Priority", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(byte));
                        usType = EGDEFINE.egTagDataType.Byte;
                    }
                    //Nameの取得
                    else if (String.Compare(stPropName, "Name", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_CLCTNAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }

                    if (dataSize > 0)
                        propptr = Marshal.AllocCoTaskMem(dataSize);

                    // メモリ確保
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                    // マネージド構造体をアンマネージドにコピーする
                    Marshal.StructureToPtr(index, indexPtr, false);
                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Collectorの情報取得
                    ret = _C_EgCollectorGetPropertyByIndex(indexPtr, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(propptr, bytes, 0, dataSize);
                        //型にあったデータを取得(Tagじゃないけど同じﾛｼﾞｯｸなので)
                        value = egCommonFunc.GetTagAnyData(bytes, usType);
                        if (value != null)
                            return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }

        /// <summary>
        /// Collectorの登録数取得
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgCollectorGetCount(ref uint itemNum)
        {
            int ret = -1;
            try
            {
                //EgCollectorGetCount
                ret = _C_EgCollectorGetCount(ref itemNum);
                if (ret != 0)
                {
                    string mess = "EgCollectorGetCount Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// Collectorの一覧取得
        /// </summary>
        /// <param name="indexArry"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgCollectorGetEnum(ref EGOBJINDEX[] indexArry, ref uint itemNum)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            int indexSize = Marshal.SizeOf(typeof(EGOBJINDEX));
            try
            {
                int size = indexArry.Count() * indexSize;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    inputPtr = Marshal.AllocCoTaskMem(size);
                }

                //EgCollectorGetEnum
                ret = _C_EgCollectorGetEnum(inputPtr, (uint)size, ref itemNum);
                if (ret == 0)
                {
                    for (int i = 0; i < itemNum; i++)
                    {
                        indexArry[i] = (EGOBJINDEX)Marshal.PtrToStructure(inputPtr + (indexSize * i), typeof(EGOBJINDEX));
                    }
                }
                else
                {
                    string mess = "EgCollectorGetEnum Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }

        /// <summary>
        /// キャッシュテーブルの登録名(MBX名+_R・_W・_RW)を取得
        /// </summary>
        /// <param name="MBXName">メールボックス名</param>
        /// <param name="uinAccessMode">アクセスモード</param>
        /// <returns>キャッシュテーブル登録名</returns>
        private string _GetCacheTBLRegName(string MBXName, uint uinAccessMode)
        {
            if (uinAccessMode == EGHG_FIFO_OPENMODE_R)
                return MBXName + "_R";
            else if (uinAccessMode == EGHG_FIFO_OPENMODE_W)
                return MBXName + "_W";
            else if (uinAccessMode == EGHG_FIFO_OPENMODE_RW)
                return MBXName + "_RW";
            else
                return MBXName;
        }

        //-------------------------------------------------------------------------------
        //
        // MailBox
        //
        //-------------------------------------------------------------------------------
        //一度開いたものを保持しておくキャッシュテーブル　仮
        /// <summary>
        /// RTCDハンドル + create情報
        /// </summary>

        public static readonly uint EGHG_FIFO_OPENMODE_R = 1;               //FIFO & Mailbox オープンモード リードのみ
        public static readonly uint EGHG_FIFO_OPENMODE_W = 2;               //FIFO & Mailbox オープンモード 書き込みのみ
        public static readonly uint EGHG_FIFO_OPENMODE_RW = 3;              //FIFO & Mailbox オープンモード 両方

        private static Dictionary<string, HIGHRTCDHANDLE> FIFO_DIC_DATA = new Dictionary<string, HIGHRTCDHANDLE>();
        private static Dictionary<string, string> DIC_SC_NAME_2_MBOX = new Dictionary<string, string>();    //サービスコンテナ名,ハッシュ値
        object SCDicLock = new object(); //Dictionalyのロック
        private int _GetSCMboxName(string ServiceName, ref string MboxName)
        {
            int ret = -1;
            try
            {
                //キャッシュテーブルに格納されていてアクティブな場合はキャッシュテーブルから取得
                if (DIC_SC_NAME_2_MBOX.ContainsKey(ServiceName))
                {
                    MboxName = DIC_SC_NAME_2_MBOX[ServiceName];
                    return 0;
                }
                else
                {
                    object objMailboxName = new object();
                    // SCのインデックス名取得
                    ret = EgServiceGetProperty(ServiceName, "MailboxName", ref objMailboxName);
                    if (ret == 0)
                    {
                        string stMailboxName = (string)objMailboxName;
                        MboxName = stMailboxName;
                        lock (SCDicLock)
                        {
                            DIC_SC_NAME_2_MBOX.Add(ServiceName, stMailboxName);
                        }
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// キャッシュテーブル操作
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="uinAccessMode"></param>
        /// <param name="g_tRtcdHdInfo"></param>
        /// <returns></returns>
        private int _GetRTCDHANDLE(string Name, uint uinAccessMode, ref RTCDHANDLEANDINFO g_tRtcdHdInfo)
        {
            int ret = -1;
            try
            {
                // キャッシュテーブル登録名取得
                string TBLRegName = _GetCacheTBLRegName(Name, uinAccessMode);

                //キャッシュテーブルに格納されていてアクティブな場合はキャッシュテーブルから取得
                if (FIFO_DIC_DATA.ContainsKey(TBLRegName) && FIFO_DIC_DATA[TBLRegName].IsActive)
                {
                    //アクセスモードが一致している場合はハンドルを取得
                    if (EGHG_FIFO_OPENMODE_RW == FIFO_DIC_DATA[TBLRegName].AccessMode || uinAccessMode == FIFO_DIC_DATA[TBLRegName].AccessMode)
                    {
                        g_tRtcdHdInfo = FIFO_DIC_DATA[TBLRegName].HandleInfo;
                        return 0;
                    }
                    //アクセスモードが一致していない場合は、一度閉じて再度指定されたアクセスモードで開く
                    else
                    {
                        _EgMailboxClose(TBLRegName);
                        ret = _EgMailboxOpen(Name, uinAccessMode);
                        if (ret == 0)
                        {
                            g_tRtcdHdInfo = FIFO_DIC_DATA[TBLRegName].HandleInfo;
                        }
                    }
                }
                //そもそもキャッシュテーブルに格納されていない or アクティブではないものは開く
                else
                {

                    ret = _EgMailboxOpen(Name, uinAccessMode);
                    if (ret == 0)
                    {
                        g_tRtcdHdInfo = FIFO_DIC_DATA[TBLRegName].HandleInfo;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// キャッシュテーブル (再登録)
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="uinAccessMode"></param>
        /// <param name="g_tRtcdHdInfo"></param>
        /// <returns></returns>
        private int _ReGetRTCDHANDLE(string Name, uint uinAccessMode, ref RTCDHANDLEANDINFO g_tRtcdHdInfo)
        {
            int ret = -1;
            try
            {
                // キャッシュテーブル登録名取得
                string TBLRegName = _GetCacheTBLRegName(Name, uinAccessMode);

                //キャッシュテーブルに格納されていてアクティブな場合はキャッシュテーブルから取得
                if (FIFO_DIC_DATA.ContainsKey(TBLRegName))
                {
                    //アクセスモードが一致していない場合は、一度閉じて再度指定されたアクセスモードで開く
                    _EgMailboxClose(TBLRegName);
                    ret = _EgMailboxOpen(Name, uinAccessMode);
                    if (ret == 0)
                    {
                        g_tRtcdHdInfo = FIFO_DIC_DATA[TBLRegName].HandleInfo;
                    }

                }
                //そもそもキャッシュテーブルに格納されていない
                else
                {
                    ret = _EgMailboxOpen(Name, uinAccessMode);
                    if (ret == 0)
                    {
                        g_tRtcdHdInfo = FIFO_DIC_DATA[TBLRegName].HandleInfo;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// RTCDHEADERの取得
        /// </summary>
        /// <param name="stHandle"></param>
        /// <param name="stHeader"></param>
        /// <returns></returns>
        public int _EgMailboxGetInfo(RTCDHANDLE stHandle, ref RTCDHEADER stHeader)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            IntPtr headerPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHANDLE)));
                    headerPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHEADER)));
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(stHandle, inputPtr, false);
                ret = _C_EgMailboxGetInfo(inputPtr, headerPtr);

                if (ret != 0)
                {
                    string mess = "_EgFIFOGetInfo Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                stHeader = (RTCDHEADER)Marshal.PtrToStructure(headerPtr, typeof(RTCDHEADER));
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
                if (headerPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(headerPtr);
            }
        }
        /// <summary>
        /// RTCDHEADERの取得
        /// </summary>
        /// <param name="stHandle"></param>
        /// <param name="stHeader"></param>
        /// <returns></returns>
        public int _EgMailboxGetInfo(string Name, ref RTCDHEADER stHeader)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            IntPtr headerPtr = IntPtr.Zero;
            try
            {

                //既に開いている場合はキャッシュから開いているハンドルを取得。開いていない場合は、開く
                RTCDHANDLEANDINFO g_tRtcdHdInfo = new RTCDHANDLEANDINFO();
                ret = _GetRTCDHANDLE(Name, EGHG_FIFO_OPENMODE_W, ref g_tRtcdHdInfo);
                if (ret != 0)
                {
                    string mess = "GetRTCDHANDLE Error " + ret.ToString();
                    Debug.WriteLine(mess);
                    return ret;
                }
                
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHANDLE)));
                    headerPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHEADER)));
                }


                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(g_tRtcdHdInfo.stHandle, inputPtr, false);
                ret = _C_EgMailboxGetInfo(inputPtr, headerPtr);

                if (ret != 0)
                {
                    string mess = "_EgMailboxGetInfo Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                stHeader = (RTCDHEADER)Marshal.PtrToStructure(headerPtr, typeof(RTCDHEADER));
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
                if (headerPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(headerPtr);
            }
        }

        /// <summary>
        /// EgMailboxCreateEx メールボックス作成 Ex(サイズ指定)
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="uiLength"></param>
        /// <param name="uiNum"></param>
        /// <param name="stMemInfo"></param>
        /// <returns></returns>        
        public int EgMailboxCreateEx(string Name, uint uiLength, uint uiNum, ref RTCDMEMINFO stMemInfo)
        {
            int ret = -1;
            IntPtr memptr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    memptr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDMEMINFO)));
                }
                //メールボックス作成
                ret = _C_EgMailboxCreateEx(Name, uiLength, uiNum, memptr);
                if (ret == 0)
                {
                    stMemInfo = (RTCDMEMINFO)Marshal.PtrToStructure(memptr, typeof(RTCDMEMINFO));
                }
                else
                {
                    string mess = "EgMailboxCreate Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (memptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(memptr);
            }
        }
        /// <summary>
        /// EgMailboxCreate メールボックス作成　サイズ固定
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="stMemInfo"></param>
        /// <returns></returns>
        public int EgMailboxCreate(string Name, ref RTCDMEMINFO stMemInfo)
        {
            int ret = -1;
            try
            {
                //メールボックス作成
                ret = EgMailboxCreateEx(Name, 2048, 1024, ref stMemInfo);
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// メールボックス削除
        /// </summary>
        /// <param name="stMemInfo"></param>
        /// <returns></returns>
        public int EgMailboxDelete(RTCDMEMINFO stMemInfo)
        {
            String[] TBLRegName = new String[3];
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            try
            {
                // キャッシュテーブル登録名全パターン分取得
                TBLRegName[0] = stMemInfo.name + "_R";
                TBLRegName[1] = stMemInfo.name + "_W";
                TBLRegName[2] = stMemInfo.name + "_RW";

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDMEMINFO)));
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(stMemInfo, inputPtr, false);


                // キャッシュテーブル登録名3個分(_R,_W,_RW)のクローズ処理
                for (int i=0; i<3; i++)
                {
                    // 開いている状態の場合は、クローズ処理を行う
                    if (FIFO_DIC_DATA.ContainsKey(TBLRegName[i]) && FIFO_DIC_DATA[TBLRegName[i]].IsActive)
                    {
                        //メールボックス閉じる
                        _EgMailboxClose(TBLRegName[i]);
                    }
                }

                // メールボックス削除
                ret = _C_EgMailboxDelete(inputPtr);
                if (ret != 0)
                {
                    string mess = "EgMailboxDelete Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                else
                {
                    stMemInfo = new RTCDMEMINFO();
                }


                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }
        object DicLock = new object(); //Dictionalyのロック
        /// <summary>
        /// メールボックスオープン
        /// </summary>
        /// <param name="MailBoxName"></param>
        /// <param name="uinAccessMode"></param>
        /// <returns></returns>
        private int _EgMailboxOpen(string MailBoxName, uint uinAccessMode)
        {
            int ret = -1;
            string TBLRegName; // メールボックス名
            try
            {
                // メールボックス名をキャッシュテーブル登録名に変換
                TBLRegName = _GetCacheTBLRegName(MailBoxName, uinAccessMode);

                //既に開いている場合は何もしない
                if (FIFO_DIC_DATA.ContainsKey(TBLRegName) && FIFO_DIC_DATA[TBLRegName].IsActive)
                {
                    string mess = "EgMailboxOpen Error Open FIFO Exist" + ret.ToString();
                    Debug.WriteLine(mess);
                }
                else
                {
                    RTCDHANDLEANDINFO stHInfo = new RTCDHANDLEANDINFO();
                    stHInfo.stHeader = new RTCDHEADER();
                    stHInfo.stHandle = new RTCDHANDLE();
                    //オープン
                    ret = _EgMailboxOpen(MailBoxName, uinAccessMode, ref stHInfo.stHandle);
                    if (ret == 0)
                    {

                        ret = _EgMailboxGetInfo(stHInfo.stHandle, ref stHInfo.stHeader);
                        if (ret == 0)
                        {
                            //キャッシュに名前が含まれているか？
                            if (FIFO_DIC_DATA.ContainsKey(TBLRegName))
                            {
                                //含まれている場合はオープンしているかチェック、
                                if (!FIFO_DIC_DATA[TBLRegName].IsActive)
                                {
                                    //起動していない場合は、登録
                                    FIFO_DIC_DATA[TBLRegName].HandleInfo = stHInfo;
                                    FIFO_DIC_DATA[TBLRegName].IsActive = true;
                                    FIFO_DIC_DATA[TBLRegName].AccessMode = uinAccessMode;
                                }
                            }
                            else
                            {
                                lock (DicLock)
                                {
                                    //含まれていない場合は登録
                                    HIGHRTCDHANDLE HIGHHANDLE = new HIGHRTCDHANDLE(MailBoxName, true, uinAccessMode, stHInfo);
                                    FIFO_DIC_DATA.Add(TBLRegName, HIGHHANDLE);
                                }
                            }
                        }
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// メールボックスオープン
        /// </summary>
        /// <param name="MailBoxName"></param>
        /// <param name="uinAccessMode"></param>
        /// <param name="pHandle"></param>
        /// <returns></returns>
        private int _EgMailboxOpen(string MailBoxName, uint uinAccessMode, ref RTCDHANDLE pHandle)
        {
            int ret = -1;
            IntPtr memptr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //メモリ確保
                    memptr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHANDLE)));
                }
                //オープン
                ret = _C_EgMailboxOpen(MailBoxName, uinAccessMode, memptr);
                if (ret == 0)
                    pHandle = (RTCDHANDLE)Marshal.PtrToStructure(memptr, typeof(RTCDHANDLE));
                else
                {
                    string mess = "EgOpenMailbox Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (memptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(memptr);
            }
        }
        /// <summary>
        /// メールボックスクローズ
        /// </summary>
        /// <param name="MailboxName"></param>
        /// <returns></returns>
        private int _EgMailboxClose(string MailboxName)
        {
            int ret = -1;
            try
            {
                //既に開いている場合は何もしない
                if (FIFO_DIC_DATA.ContainsKey(MailboxName) && FIFO_DIC_DATA[MailboxName].IsActive)
                {
                    //メールボックス閉じる
                    ret = _EgMailboxClose(FIFO_DIC_DATA[MailboxName].HandleInfo.stHandle);
                    if (ret == 0)
                    {
                        FIFO_DIC_DATA[MailboxName].IsActive = false;
                        FIFO_DIC_DATA[MailboxName].HandleInfo = new RTCDHANDLEANDINFO();
                    }
                    else
                    {
                        string mess = "EgFIFOClose Error " + ret.ToString();
                        Debug.WriteLine(mess);
                    }

                }
                else
                {
                    string mess = "EgMailboxClose Error Not Open FIFO" + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// メールボックスクローズ
        /// </summary>
        /// <param name="pHandle"></param>
        /// <returns></returns>
        private int _EgMailboxClose(RTCDHANDLE stHandle)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHANDLE)));
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(stHandle, inputPtr, false);
                ret = _C_EgMailboxClose(inputPtr);

                if (ret != 0)
                {
                    string mess = "EgCloseMailbox Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }
        /// <summary>
        /// EgFinalizeで呼び出すメールボックス強制クローズ関数
        /// </summary>
        /// <returns></returns>
        private int _EgMailboxCloseAll()
        {
            int ret = -1;
            try
            {
                //オープンしているものすべてクローズ
                foreach (KeyValuePair<string, HIGHRTCDHANDLE> MboxItem in FIFO_DIC_DATA)
                {
                    if (MboxItem.Value != null)//存在する場合のみクローズ
                    {
                        ret = _EgMailboxClose(MboxItem.Value.HandleInfo.stHandle);
                        if(ret != 0)
                            Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : _EgMailboxClose ret = " + ret.ToString());
                    }
                }
                return 0;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// メール送信
        /// </summary>
        /// <param name="pHandle"></param>
        /// <param name="ReceiverName"></param>
        /// <param name="SenderName"></param>
        /// <param name="MessageNo"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        object SendLock = new object();
        private int __EgMailboxSend(RTCDHANDLEANDINFO g_tRtcdHdInfo,
                                   string ReceiverName,
                                   string SenderName,
                                   int MessageNo,
                                   byte[] args,
                                   bool bRetryF = true) //オープンに失敗している場合があるので内部でオープン後呼ぶように実装
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            IntPtr buffPtr = IntPtr.Zero;
            int argsize = 0;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHANDLE)));
                    //書き込むbuff作成
                    if (args != null)
                    {
                        argsize = args.Length;
                        buffPtr = Marshal.AllocCoTaskMem(argsize);
                    }
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(g_tRtcdHdInfo.stHandle, inputPtr, false);

                // byte[]をアンマネージドにコピーする
                if (buffPtr != IntPtr.Zero)
                    Marshal.Copy(args, 0, buffPtr, args.Length);

                //lock (SendLock)
                //{
                    ret = _C_EgMailboxSend(inputPtr, SenderName, MessageNo, buffPtr, (uint)argsize);
                //}
                //エラーの場合はオープンしなおしてから実施
                if (ret != 0 && bRetryF)
                {
                    //既に開いている場合はキャッシュから開いているハンドルを取得。開いていない場合は、開く
                    RTCDHANDLEANDINFO g_tNewRtcdHdInfo = new RTCDHANDLEANDINFO();
                    int ret2 = _ReGetRTCDHANDLE(ReceiverName, EGHG_FIFO_OPENMODE_W, ref g_tNewRtcdHdInfo);
                    if (ret2 == 0)
                        ret2 = __EgMailboxSend(g_tNewRtcdHdInfo, ReceiverName, SenderName, MessageNo, args, false);

                    if (ret2 != 0)
                    {
                        string mess = "EgMailboxSend Retry Error  " + ret2.ToString();
                        Debug.WriteLine(mess);
                    }
                    else
                        ret = 0;

                }

                if (ret != 0)
                {
                    string mess = "EgMailboxSend Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
                if (buffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(buffPtr);
            }
        }
        /// <summary>
        /// メール送信 byte配列指定
        /// </summary>
        /// <param name="ReceiverName"></param>
        /// <param name="SenderName"></param>
        /// <param name="MessageNo"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public int EgMailboxSend(string ReceiverName,
                                        string SenderName,
                                        int MessageNo,
                                        byte[] args)
        {
            int ret = -1;
            try
            {
                //既に開いている場合はキャッシュから開いているハンドルを取得。開いていない場合は、開く
                RTCDHANDLEANDINFO g_tRtcdHdInfo = new RTCDHANDLEANDINFO();
                ret = _GetRTCDHANDLE(ReceiverName, EGHG_FIFO_OPENMODE_W, ref g_tRtcdHdInfo);
                if (ret == 0)
                {
                    ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, args);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        //
        // サービスコンテナメールボックスへ送信専用API
        //
        public int EgMailboxSendSC(string SeriviceName,
                                        string SenderName,
                                        int MessageNo,
                                        byte[] args = null)
        {
            int ret = -1;
            try
            {
                //既に開いている場合はキャッシュから開いているハンドルを取得。開いていない場合は、開く
                RTCDHANDLEANDINFO g_tRtcdHdInfo = new RTCDHANDLEANDINFO();
                string ReceiverName = "";
                ret = _GetSCMboxName(SeriviceName, ref ReceiverName);
                if (ret == 0)
                {
                    ret = _GetRTCDHANDLE(ReceiverName, EGHG_FIFO_OPENMODE_W, ref g_tRtcdHdInfo);
                    if (ret == 0)
                    {
                        ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, args);
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        
        /// <summary>
        /// メール送信 string文字列指定　[ なるべく使用禁止 消したいが使っているユーザがいるかもしれない為残す 直す場合は string 一つ受け取る関数作成 これは今後Noメンテ]
         /// </summary>
        /// <param name="ReceiverName"></param>
        /// <param name="SenderName"></param>
        /// <param name="MessageNo"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public int EgMailboxSend(string ReceiverName,
                                        string SenderName,
                                        int MessageNo,
                                        params string[] args)
        {
            int ret = -1;
            try
            {
                //既に開いている場合はキャッシュから開いているハンドルを取得。開いていない場合は、開く
                RTCDHANDLEANDINFO g_tRtcdHdInfo = new RTCDHANDLEANDINFO();
                ret = _GetRTCDHANDLE(ReceiverName, EGHG_FIFO_OPENMODE_W, ref g_tRtcdHdInfo);
                if (ret == 0)
                {
                    //引数が無い場合は通常送信
                    if (args.Count() == 0)
                        ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, null);
                    //引数が一つの文字列の場合
                    else if (args.Count() == 1)
                    {
                        if (args[0].Length > EGDEFINE.RGMS_PRAMSTRING1L)
                            return -2;
                        if (args[0].Length == 0)
                            ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, null);
                        else
                        {
                            //構造体に変更
                            EGDEFINE.RGMSPRAMTSTRING1 st1 = new EGDEFINE.RGMSPRAMTSTRING1();
                            st1.stParam = args[0];
                            byte[] byArgs = new byte[Marshal.SizeOf(typeof(EGDEFINE.RGMSPRAMTSTRING1))];
                            ret = egCommonFunc.fnObjToBy(st1, ref byArgs);
                            if (ret == 0)
                                ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, byArgs);
                        }
                    }
                    //引数が二つの文字列の場合
                    else if (args.Count() == 2)
                    {
                        if (args[0].Length > EGDEFINE.RGMS_PRAMSTRING2L || args[1].Length > EGDEFINE.RGMS_PRAMSTRING2L)
                            return -2;
                        EGDEFINE.RGMSPRAMTSTRING2 st2 = new EGDEFINE.RGMSPRAMTSTRING2();
                        st2.stParam1 = args[0];
                        st2.stParam2 = args[1];
                        byte[] byArgs = new byte[Marshal.SizeOf(typeof(EGDEFINE.RGMSPRAMTSTRING2))];
                        ret = egCommonFunc.fnObjToBy(st2, ref byArgs);
                        if (ret == 0)
                            ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, byArgs);
                    }
                    //引数が三つの文字列の場合
                    else if (args.Count() == 3)
                    {
                        if (args[0].Length > EGDEFINE.RGMS_PRAMSTRING4L || args[1].Length > EGDEFINE.RGMS_PRAMSTRING4L || args[2].Length > EGDEFINE.RGMS_PRAMSTRING4L)
                            return -2;
                        EGDEFINE.RGMSPRAMTSTRING4 st4 = new EGDEFINE.RGMSPRAMTSTRING4();
                        st4.stParam1 = args[0];
                        st4.stParam2 = args[1];
                        st4.stParam3 = args[2];
                        byte[] byArgs = new byte[Marshal.SizeOf(typeof(EGDEFINE.RGMSPRAMTSTRING4))];
                        ret = egCommonFunc.fnObjToBy(st4, ref byArgs);
                        if (ret == 0)
                            ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, byArgs);
                    }
                    //引数が四つの文字列の場合
                    else if (args.Count() == 4)
                    {
                        if (args[0].Length > EGDEFINE.RGMS_PRAMSTRING4L || args[1].Length > EGDEFINE.RGMS_PRAMSTRING4L
                            || args[2].Length > EGDEFINE.RGMS_PRAMSTRING4L || args[3].Length > EGDEFINE.RGMS_PRAMSTRING4L)
                            return -2;
                        EGDEFINE.RGMSPRAMTSTRING4 st4 = new EGDEFINE.RGMSPRAMTSTRING4();
                        st4.stParam1 = args[0];
                        st4.stParam2 = args[1];
                        st4.stParam3 = args[2];
                        st4.stParam4 = args[3];
                        byte[] byArgs = new byte[Marshal.SizeOf(typeof(EGDEFINE.RGMSPRAMTSTRING4))];
                        ret = egCommonFunc.fnObjToBy(st4, ref byArgs);
                        if (ret == 0)
                            ret = __EgMailboxSend(g_tRtcdHdInfo, ReceiverName, SenderName, MessageNo, byArgs);
                    }
                    else
                        Debug.WriteLine("ERROR EgMailboxSend param out of range.");
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// EgMailboxReceive ハンドル渡し
        /// </summary>
        /// <param name="pHandle"></param>
        /// <param name="SenderName"></param>
        /// <param name="ReturnFlag"></param>
        /// <param name="MessageNo"></param>
        /// <param name="Params"></param>
        /// <param name="uitimeout"></param>
        /// <returns></returns>
        private int __EgMailboxReceive(RTCDHANDLEANDINFO g_tRtcdHdInfo,
                                           string ReceiverName,
                                           ref string SenderName,
                                           ref int MessageNo,
                                           ref byte[] Params,
                                           uint uitimeout,
                                           bool bRetryF = true) //オープンに失敗している場合があるので内部でオープン後呼ぶように実装
        {

            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            IntPtr ReadBuffPtr = IntPtr.Zero;
            try
            {
                //読み込むbuff作成
                int size = (int)g_tRtcdHdInfo.stHeader.record_size;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // RTCDメモリ情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(RTCDHANDLE)));
                    ReadBuffPtr = Marshal.AllocCoTaskMem(size);
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(g_tRtcdHdInfo.stHandle, inputPtr, false);

                StringBuilder sbSenderName = new StringBuilder(12);
                ret = _C_EgMailboxReceive(inputPtr, sbSenderName, 12, ref MessageNo, ReadBuffPtr, (uint)size, uitimeout);
                if (ret != 0 && bRetryF)
                {
                    //タイムアウト時はスルー
                    if(ret != EGDEFINE.EDGE_RTCD_NO_UPDATE )
                    {
                        //既に開いている場合はキャッシュから開いているハンドルを取得。開いていない場合は、開く
                        RTCDHANDLEANDINFO g_tNewRtcdHdInfo = new RTCDHANDLEANDINFO();
                        int ret2 = _ReGetRTCDHANDLE(ReceiverName, EGHG_FIFO_OPENMODE_W, ref g_tNewRtcdHdInfo);
                        if (ret2 == 0)
                            ret2 = __EgMailboxReceive(g_tNewRtcdHdInfo, ReceiverName, ref SenderName, ref MessageNo, ref Params, uitimeout, false);

                        if (ret2 != 0 && ret2 != EGDEFINE.EDGE_RTCD_NO_UPDATE)
                {
                            string mess = "EgMailboxReceive Retry Error  " + ret2.ToString();
                            Debug.WriteLine(mess);
                        }
                    }
                }
                else
                {
                    SenderName = sbSenderName.ToString();
                    byte[] byData = new byte[size];
                    Marshal.Copy(ReadBuffPtr, byData, 0, size);
                    Params = byData;
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
                if (ReadBuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(ReadBuffPtr);
            }
        }
        /// <summary>
        /// メッセージ受信
        /// </summary>
        /// <param name="ReceiverName"></param>
        /// <param name="Params"></param>
        /// <param name="uitimeout"></param>
        /// <returns></returns>
        public int _EgMailboxReceive(string ReceiverName,
                                    ref byte[] Params,
                                    uint uitimeout)
        {
            int ret = -1;
            try
            {
                // いらないもの
                string SenderName = "";
                int MessageNo = 0;

                ret = EgMailboxReceive(ReceiverName, ref SenderName, ref MessageNo, ref Params, uitimeout);

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }


        /// <summary>
        /// メッセージ受信
        /// </summary>
        /// <param name="ReceiverName"></param>
        /// <param name="SenderName"></param>
        /// <param name="MessageNo"></param>
        /// <param name="Params"></param>
        /// <param name="uitimeout"></param>
        /// <returns></returns>
        public int EgMailboxReceive(string ReceiverName,
                                             ref string SenderName,
                                             ref int MessageNo,
                                             ref byte[] Params,
                                             uint uitimeout)
        {
            int ret = -1;
            try
            {
                //既に開いている場合はキャッシュから開いているハンドルを取得。開いていない場合は、開く
                RTCDHANDLEANDINFO g_tRtcdHdInfo = new RTCDHANDLEANDINFO();
                ret = _GetRTCDHANDLE(ReceiverName, EGHG_FIFO_OPENMODE_R, ref g_tRtcdHdInfo);
                if (ret == 0)
                {
                    ret = __EgMailboxReceive(g_tRtcdHdInfo, ReceiverName, ref SenderName, ref MessageNo, ref Params, uitimeout);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// EgMailboxGetProperty
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgMailboxGetProperty(string stName, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                //(Tagじゃないけど同じﾛｼﾞｯｸなので)
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.UNDEFINE;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    //Sizeの場合
                    if (String.Compare(stPropName, "ColSize", true) == 0 ||
                        String.Compare(stPropName, "RowSize", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(uint));
                        usType = EGDEFINE.egTagDataType.UInt32;

                    }
                    //Nameの取得(Name指定なので不要)
                    else if (String.Compare(stPropName, "Name", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_CLCTNAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }

                    if (dataSize > 0)
                        propptr = Marshal.AllocCoTaskMem(dataSize);

                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Mailboxの情報取得
                    ret = _C_EgMailboxGetProperty(stName, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(propptr, bytes, 0, dataSize);
                        //型にあったデータを取得(Tagじゃないけど同じﾛｼﾞｯｸなので)
                        value = egCommonFunc.GetTagAnyData(bytes, usType);
                        if (value != null)
                            return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }

        /// <summary>
        /// EgMailboxGetPropertyByIndex
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgMailboxGetPropertyByIndex(EGOBJINDEX index, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr indexPtr = IntPtr.Zero;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                //(Tagじゃないけど同じﾛｼﾞｯｸなので)
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.UNDEFINE;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    //Sizeの場合
                    if (String.Compare(stPropName, "ColSize", true) == 0 ||
                        String.Compare(stPropName, "RowSize", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(uint));
                        usType = EGDEFINE.egTagDataType.UInt32;

                    }
                    //Nameの取得
                    else if (String.Compare(stPropName, "Name", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_CLCTNAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }

                    if (dataSize > 0)
                        propptr = Marshal.AllocCoTaskMem(dataSize);

                    // メモリ確保
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                    // マネージド構造体をアンマネージドにコピーする
                    Marshal.StructureToPtr(index, indexPtr, false);
                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Mailboxの情報取得
                    ret = _C_EgMailboxGetPropertyByIndex(indexPtr, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(propptr, bytes, 0, dataSize);
                        //型にあったデータを取得(Tagじゃないけど同じﾛｼﾞｯｸなので)
                        value = egCommonFunc.GetTagAnyData(bytes, usType);
                        if (value != null)
                            return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }

        /// <summary>
        /// Mailboxの登録数取得
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgMailboxGetCount(ref uint itemNum)
        {
            int ret = -1;
            try
            {
                //EgMailboxGetCount
                ret = _C_EgMailboxGetCount(ref itemNum);
                if (ret != 0)
                {
                    string mess = "EgMailboxGetCount Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// Mailboxの一覧取得
        /// </summary>
        /// <param name="indexArry"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgMailboxGetEnum(ref EGOBJINDEX[] indexArry, ref uint itemNum)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            int indexSize = Marshal.SizeOf(typeof(EGOBJINDEX));
            try
            {
                int size = indexArry.Count() * indexSize;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    inputPtr = Marshal.AllocCoTaskMem(size);
                }

                //_C_EgMailboxGetEnum
                ret = _C_EgMailboxGetEnum(inputPtr, (uint)size, ref itemNum);
                if (ret == 0)
                {
                    for (int i = 0; i < itemNum; i++)
                    {
                        indexArry[i] = (EGOBJINDEX)Marshal.PtrToStructure(inputPtr + (indexSize * i), typeof(EGOBJINDEX));
                    }
                }
                else
                {
                    string mess = "EgMailboxGetEnum Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }

        //-------------------------------------------------------------------------------
        //
        // EgService
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// EgServiceCreate
        /// </summary>
        /// <param name="stServiceName"></param>
        /// <param name="stPath"></param>
        /// <param name="stArg"></param>
        /// <param name="pid"></param>
        /// <returns></returns>
        public int EgServiceCreate(string stServiceName, string stPath, string stArg, ref long pid)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(pid));
                }
                ret = _C_EgServiceCreate(stServiceName, stPath, stArg, inputPtr);
                if (ret == 0)
                {
                    byte[] bytes = new byte[Marshal.SizeOf(pid)];
                    Marshal.Copy(inputPtr, bytes, 0, Marshal.SizeOf(pid));
                    //型にあったデータを取得
                    object value = egCommonFunc.GetTagAnyData(bytes, EGDEFINE.egTagDataType.Int64);
                    if (value != null)
                    {
                        pid = (long)value;
                        return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }
        /// <summary>
        /// EgServiceCreateEx
        /// </summary>
        /// <param name="stServiceName"></param>
        /// <param name="stPath"></param>
        /// <param name="stArg"></param>
        /// <param name="pid"></param>
        /// <param name="config"></param>
        /// <returns></returns>
        public int EgServiceCreateEx(string stServiceName, string stPath, string stArg, ref long pid, edge_API.EDGESERVICECONFIG config)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            IntPtr pPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(pid));
                    pPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(edge_API.EDGESERVICECONFIG)));
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(config, pPtr, false);

                ret = _C_EgServiceCreateEx(stServiceName, stPath, stArg, inputPtr, pPtr);
                if (ret == 0)
                {
                    byte[] bytes = new byte[Marshal.SizeOf(pid)];
                    Marshal.Copy(inputPtr, bytes, 0, Marshal.SizeOf(pid));
                    //型にあったデータを取得
                    object value = egCommonFunc.GetTagAnyData(bytes, EGDEFINE.egTagDataType.Int64);
                    if (value != null)
                    {
                        pid = (long)value;
                        return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
                if (pPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(pPtr);
            }
        }

        /// <summary>
        /// EgServiceGetProperty
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgServiceGetProperty(string stName, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                //(Tagじゃないけど同じﾛｼﾞｯｸなので)
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.UNDEFINE;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    //Pathの場合
                    if (String.Compare(stPropName, "Path", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICEPATH + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //Argの場合
                    else if (String.Compare(stPropName, "Arg", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICEARG + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //NodeNameの場合
                    else if (String.Compare(stPropName, "NodeName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_NODENAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //Nameの取得(Name指定なので不要)
                    else if (String.Compare(stPropName, "Name", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICENAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //MailboxNameの取得
                    else if (String.Compare(stPropName, "MailboxName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICEMAILBOXNAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //IndexValueの取得
                    else if (String.Compare(stPropName, "IndexValue", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(uint));
                        usType = EGDEFINE.egTagDataType.UInt32;
                    }

                    if (dataSize > 0)
                        propptr = Marshal.AllocCoTaskMem(dataSize);

                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Serviceの情報取得
                    ret = _C_EgServiceGetProperty(stName, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(propptr, bytes, 0, dataSize);
                        //型にあったデータを取得(Tagじゃないけど同じﾛｼﾞｯｸなので)
                        value = egCommonFunc.GetTagAnyData(bytes, usType);
                        if (value != null)
                            return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }

        /// <summary>
        /// EgServiceGetPropertyByIndex
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="stPropName"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public int EgServiceGetPropertyByIndex(EGOBJINDEX index, string stPropName, ref object value)
        {
            int ret = -1;
            IntPtr indexPtr = IntPtr.Zero;
            IntPtr propptr = IntPtr.Zero;
            try
            {
                int dataSize = 0;
                //(Tagじゃないけど同じﾛｼﾞｯｸなので)
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.UNDEFINE;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    //プロパティ毎にサイズが違うため、処理をわける
                    //Pathの場合
                    if (String.Compare(stPropName, "Path", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICEPATH + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //Argの場合
                    else if (String.Compare(stPropName, "Arg", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICEARG + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //NodeNameの場合
                    else if (String.Compare(stPropName, "NodeName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_NODENAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //Nameの取得
                    else if (String.Compare(stPropName, "Name", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICENAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //MailboxNameの取得
                    else if (String.Compare(stPropName, "MailboxName", true) == 0)
                    {
                        dataSize = EGDEFINE.EG_MAX_SERVICEMAILBOXNAME + 1;
                        usType = EGDEFINE.egTagDataType.String;
                    }
                    //IndexValueの取得
                    else if (String.Compare(stPropName, "IndexValue", true) == 0)
                    {
                        dataSize = Marshal.SizeOf(typeof(uint));
                        usType = EGDEFINE.egTagDataType.UInt32;
                    }

                    if (dataSize > 0)
                        propptr = Marshal.AllocCoTaskMem(dataSize);

                    // メモリ確保
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                    // マネージド構造体をアンマネージドにコピーする
                    Marshal.StructureToPtr(index, indexPtr, false);
                }
                //Intptrが確保できている場合実行
                if (propptr != IntPtr.Zero)
                {
                    //Serviceの情報取得
                    ret = _C_EgServiceGetPropertyByIndex(indexPtr, stPropName, propptr, (uint)dataSize);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(propptr, bytes, 0, dataSize);
                        //型にあったデータを取得(Tagじゃないけど同じﾛｼﾞｯｸなので)
                        value = egCommonFunc.GetTagAnyData(bytes, usType);
                        if (value != null)
                            return 0;
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
                if (propptr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(propptr);
            }
        }

        /// <summary>
        /// Serviceの登録数取得
        /// </summary>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgServiceGetCount(ref uint itemNum)
        {
            int ret = -1;
            try
            {
                //EgServiceGetCount
                ret = _C_EgServiceGetCount(ref itemNum);
                if (ret != 0)
                {
                    string mess = "EgServiceGetCount Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// サービスの一覧取得
        /// </summary>
        /// <param name="indexArry"></param>
        /// <param name="itemNum"></param>
        /// <returns></returns>
        public int EgServiceGetEnum(ref EGOBJINDEX[] indexArry, ref uint itemNum)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            int indexSize = Marshal.SizeOf(typeof(EGOBJINDEX));
            try
            {
                int size = indexArry.Count() * indexSize;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    inputPtr = Marshal.AllocCoTaskMem(size);
                }

                //_C_EgServiceGetEnum
                ret = _C_EgServiceGetEnum(inputPtr, (uint)size, ref itemNum);
                if (ret == 0)
                {
                    for (int i = 0; i < itemNum; i++)
                    {
                        indexArry[i] = (EGOBJINDEX)Marshal.PtrToStructure(inputPtr + (indexSize * i), typeof(EGOBJINDEX));
                    }
                }
                else
                {
                    string mess = "EgServiceGetEnum Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }

        //-------------------------------------------------------------------------------
        //
        // etc
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// EgGetTimeStamp
        /// </summary>
        /// <param name="stTimeStamp"></param>
        /// <returns></returns>
        public int EgGetTimeStamp(ref EG_TIME_STAMP stTimeStamp)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // stTimeStamp情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EG_TIME_STAMP)));
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(stTimeStamp, inputPtr, false);
                ret = _C_EgGetTime(inputPtr);

                if (ret != 0)
                {
                    string mess = "_C_EgGetTime Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                stTimeStamp = (EG_TIME_STAMP)Marshal.PtrToStructure(inputPtr, typeof(EG_TIME_STAMP));
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }

        /// <summary>
        /// EgGetTimeVal
        /// </summary>
        /// <param name="stTimeVal"></param>
        /// <returns></returns>
        public int EgGetTimeVal(ref EG_TIME_VAL stTimeVal)
        {
            int ret = -1;
            IntPtr inputPtr = IntPtr.Zero;
            try
            {
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // stTimeVal情報
                    inputPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EG_TIME_VAL)));
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(stTimeVal, inputPtr, false);
                ret = _C_EgGetTimeVal(inputPtr);

                if (ret != 0)
                {
                    string mess = "_C_EgGetTimeVal Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                stTimeVal = (EG_TIME_VAL)Marshal.PtrToStructure(inputPtr, typeof(EG_TIME_VAL));
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (inputPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(inputPtr);
            }
        }
        /// <summary>
        /// EgDecode
        /// </summary>
        /// <param name="byDst"></param>
        /// <param name="dstsize"></param>
        /// <param name="bySrc"></param>
        /// <param name="srcsize"></param>
        /// <param name="addrformat"></param>
        /// <param name="actualsize"></param>
        /// <returns></returns>
        public int EgDecode(ref byte[] byDst,
                             byte[] bySrc,
                             string addrformat,
                             ref uint actualsize)
        {
            int ret = -1;
            IntPtr dstPtr = IntPtr.Zero;
            IntPtr srcPtr = IntPtr.Zero;
            try
            {
                if (byDst == null || byDst.Length == 0 || bySrc == null || bySrc.Length == 0)
                    return -1;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // ポインタ
                    dstPtr = Marshal.AllocCoTaskMem(byDst.Length);
                    srcPtr = Marshal.AllocCoTaskMem(bySrc.Length);
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.Copy(bySrc, 0, srcPtr, bySrc.Length);
                Marshal.Copy(byDst, 0, dstPtr, byDst.Length);
                ret = _C_EgDecode(dstPtr, (uint)byDst.Length, srcPtr, (uint)bySrc.Length, addrformat, ref actualsize);
                if (ret != 0)
                {
                    string mess = "_C_EgDecode Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                else
                {
                    Marshal.Copy(dstPtr, byDst, 0, byDst.Length);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (dstPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(dstPtr);
                if (srcPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(srcPtr);
            }
        }
        /// <summary>
        /// EgEncode
        /// </summary>
        /// <param name="byDst"></param>
        /// <param name="dstsize"></param>
        /// <param name="bySrc"></param>
        /// <param name="srcsize"></param>
        /// <param name="addrformat"></param>
        /// <param name="actualsize"></param>
        /// <returns></returns>
        public int EgEncode(ref byte[] byDst,
                             byte[] bySrc,
                             string addrformat,
                             ref uint actualsize)
        {
            int ret = -1;
            IntPtr dstPtr = IntPtr.Zero;
            IntPtr srcPtr = IntPtr.Zero;
            try
            {
                if (byDst == null || byDst.Length == 0 || bySrc == null || bySrc.Length == 0)
                    return -1;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // ポインタ
                    dstPtr = Marshal.AllocCoTaskMem(byDst.Length);
                    srcPtr = Marshal.AllocCoTaskMem(bySrc.Length);
                }
                // マネージド構造体をアンマネージドにコピーする
                Marshal.Copy(bySrc, 0, srcPtr, bySrc.Length);
                Marshal.Copy(byDst, 0, dstPtr, byDst.Length);
                ret = _C_EgEncode(dstPtr, (uint)byDst.Length, srcPtr, (uint)bySrc.Length, addrformat, ref actualsize);
                if (ret != 0)
                {
                    string mess = "_C_EgEncode Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }
                else
                {
                    Marshal.Copy(dstPtr, byDst, 0, byDst.Length);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (dstPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(dstPtr);
                if (srcPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(srcPtr);
            }
        }
        /// <summary>
        /// EgDecode/EgEncodeアドレス作成
        /// </summary>
        /// <param name="offset"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        public string EgMakeAddress(int offset, short size)
        {
            IntPtr retPtr = IntPtr.Zero;
            try
            {
                uint buffsize = 24; //24byteあれば足りるでしょう！
                StringBuilder sbBuff = new StringBuilder((int)buffsize);
                retPtr = _C_EgMakeAddress(offset, size, sbBuff, buffsize);
                if(retPtr != IntPtr.Zero)
                    return sbBuff.ToString();

                return "";
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return "";
            }
        }
        /// <summary>
        /// EgParseAddress
        /// </summary>
        /// <param name="addrformat"></param>
        /// <param name="offset"></param>
        /// <param name="size"></param>
        /// <param name="bit"></param>
        /// <param name="typemode"></param>
        /// <returns></returns>
        public int EgParseAddress(string addrformat, ref int offset, ref uint size, ref short bit, ref byte typemode)
        {
            int ret = -1;
            try
            {
                ret = _C_EgParseAddress(addrformat, ref offset, ref size, ref bit, ref typemode);
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// タグポインタ生成初期化処理
        /// </summary>
        /// <returns></returns>
        public int EgTagInitSegment()
        {
            int ret = -1;
            try
            {
                //呼び出し
                ret = _C_EgTagInitSegment();

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の値読み込み
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <returns></returns>
        public int EgTagReadSegment(string stName, ushort wOffset, ref byte[] pBuffer)
        {
            int ret = -1;
            IntPtr BuffPtr = IntPtr.Zero;
            ushort usSize = 0;
            try
            {
                usSize = (ushort)pBuffer.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    BuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                //呼び出し
                ret = _C_EgTagReadSegment(stName, wOffset, BuffPtr, usSize);

                if (ret == 0)
                {
                    Marshal.Copy(BuffPtr, pBuffer, 0, pBuffer.Length);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (BuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(BuffPtr);
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の値書き込み
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <returns></returns>
        public int EgTagWriteSegment(string stName, ushort wOffset, byte[] pBuffer)
        {
            int ret = -1;
            IntPtr BuffPtr = IntPtr.Zero;
            ushort usSize = 0;
            try
            {
                usSize = (ushort)pBuffer.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    BuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                Marshal.Copy(pBuffer, 0, BuffPtr, pBuffer.Length);

                //呼び出し
                ret = _C_EgTagWriteSegment(stName, wOffset, BuffPtr, usSize);

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (BuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(BuffPtr);
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)生存確認
        /// </summary>
        /// <returns></returns>
        public int EgTagIsAliveSegment(string stName)
        {
            int ret = -1;
            try
            {
                //呼び出し
                ret = _C_EgTagIsAliveSegment(stName);

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)生成
        /// </summary>
        /// <returns></returns>
        public int EgTagCreateSegment(string stName, ushort Type, ushort Size, string stSource, string stComment, ref object regName, ref ushort isLink)
        {
            int ret = -1;
            IntPtr regNamePtr = IntPtr.Zero;
            try
            {
                //長さチェック
                if (stName.Length > EGDEFINE.EG_MAX_TAGNAME || stSource.Length > EGDEFINE.EG_MAX_TAGADDRESS)
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                int dataSize = EGDEFINE.EG_MAX_TAGNAME + 1;
                EGDEFINE.egTagDataType usType = EGDEFINE.egTagDataType.String;
                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    regNamePtr = Marshal.AllocCoTaskMem(dataSize);
                }

                //Intptrが確保できている場合実行
                if (regNamePtr != IntPtr.Zero)
                {
                    //呼び出し
                    ret = _C_EgTagCreateSegment(stName, Type, Size, stSource, stComment, regNamePtr, (uint)dataSize, ref isLink);
                    if (ret == 0)
                    {
                        byte[] bytes = new byte[dataSize];
                        Marshal.Copy(regNamePtr, bytes, 0, dataSize);
                        //タグ登録名を取得
                        regName = egCommonFunc.GetTagAnyData(bytes, usType);
                    }
                    else
                    {
                        string mess = "EgTagCreateSegment Error " + ret.ToString();
                        Debug.WriteLine(mess);
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (regNamePtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(regNamePtr);
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の文字列読み込み
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pStrinrg"></param>
        /// <param name="wSize"></param>
        /// <returns></returns>
        public int EgTagReadString(string stName, ref string pString, ushort wSize)
        {
            int ret = -1;
            try
            {
                StringBuilder srBuf = new StringBuilder(wSize);

                //呼び出し
                ret = _C_EgTagReadString(stName, srBuf, wSize);

                if (ret == 0)
                {
                    pString = srBuf.ToString();
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の文字列書き込み
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pString"></param>
        /// <returns></returns>
        public int EgTagWriteString(string stName, string pString)
        {
            int ret = -1;
            try
            {
                //呼び出し
                ret = _C_EgTagWriteString(stName, pString);

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の値読み込み(INDEX指定)
        /// </summary>
        /// <param name="index"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <returns></returns>
        public int EgTagReadSegmentByIndex(EGOBJINDEX index, ushort wOffset, ref byte[] pBuffer)
        {
            int ret = -1;
            IntPtr BuffPtr = IntPtr.Zero;
            IntPtr indexPtr = IntPtr.Zero;
            ushort usSize = 0;
            try
            {
                usSize = (ushort)pBuffer.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // メモリ確保
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                    BuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.StructureToPtr(index, indexPtr, false);

                //呼び出し
                ret = _C_EgTagReadSegmentByIndex(indexPtr, wOffset, BuffPtr, usSize);

                if (ret == 0)
                {
                    Marshal.Copy(BuffPtr, pBuffer, 0, pBuffer.Length);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
                if (BuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(BuffPtr);
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の値書き込み(INDEX指定)
        /// </summary>
        /// <param name="index"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <returns></returns>
        public int EgTagWriteSegmentByIndex(EGOBJINDEX index, ushort wOffset, byte[] pBuffer)
        {
            int ret = -1;
            IntPtr BuffPtr = IntPtr.Zero;
            IntPtr indexPtr = IntPtr.Zero;
            ushort usSize = 0;
            try
            {
                usSize = (ushort)pBuffer.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // メモリ確保
                    indexPtr = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(EGOBJINDEX)));
                    BuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.Copy(pBuffer, 0, BuffPtr, pBuffer.Length);
                Marshal.StructureToPtr(index, indexPtr, false);

                //呼び出し
                ret = _C_EgTagWriteSegmentByIndex(indexPtr, wOffset, BuffPtr, usSize);

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (indexPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(indexPtr);
                if (BuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(BuffPtr);
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の値読み込み(Entry指定)
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <returns></returns>
        public int EgTagReadSegmentByEntry(IntPtr entry, ushort wOffset, ref byte[] pBuffer)
        {
            int ret = -1;
            IntPtr BuffPtr = IntPtr.Zero;
            ushort usSize = 0;
            try
            {
                usSize = (ushort)pBuffer.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // メモリ確保
                    BuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                //呼び出し
                ret = _C_EgTagReadSegmentByEntry(entry, wOffset, BuffPtr, usSize);

                if (ret == 0)
                {
                    Marshal.Copy(BuffPtr, pBuffer, 0, pBuffer.Length);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (BuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(BuffPtr);
            }
        }
        /// <summary>
        /// タグポインタ(共有メモリ)の値書き込み(Entry指定)
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="wOffset"></param>
        /// <param name="pBuffer"></param>
        /// <returns></returns>
        public int EgTagWriteSegmentByEntry(IntPtr entry, ushort wOffset, byte[] pBuffer)
        {
            int ret = -1;
            IntPtr BuffPtr = IntPtr.Zero;
            ushort usSize = 0;
            try
            {
                usSize = (ushort)pBuffer.Length;

                //メモリリークをさせないためのおまじない
                System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
                try { }
                finally
                {
                    // メモリ確保
                    BuffPtr = Marshal.AllocCoTaskMem(usSize);
                }

                // マネージド構造体をアンマネージドにコピーする
                Marshal.Copy(pBuffer, 0, BuffPtr, pBuffer.Length);

                //呼び出し
                ret = _C_EgTagWriteSegmentByEntry(entry, wOffset, BuffPtr, usSize);

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
            finally
            {
                if (BuffPtr != IntPtr.Zero)
                    Marshal.FreeCoTaskMem(BuffPtr);
            }
        }
        //-------------------------------------------------------------------------------
        //
        // Framework
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// サービスコンテナにタグトリガーの登録を行う
        /// </summary>
        /// <param name="tagName"></param>
        /// <returns></returns>
        public int EgFW_AddTagTrigger(string tagName)
        {
            int ret = -1;

            try
            {
                if (tagName.Equals(""))
                    return (int)EGDEFINE.EDGE_WRAP_PARAM_INVALID;

                ret = _C_EgFW_AddTagTrigger(tagName);
                if(ret != 0)
                {
                    string mess = "EgFW_AddTagTrigger Error " + ret.ToString();
                    Debug.WriteLine(mess);
                }

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        //-------------------------------------------------------------------------------
        //
        // FIFOﾀｸﾞ
        //
        //-------------------------------------------------------------------------------
        /// <summary>
        /// FIFO Tagの追加
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="RecordSize"></param>
        /// <param name="RecordCount"></param>
        /// <param name="stComment"></param>
        /// <param name="regName"></param>
        /// <param name="isLink"></param>
        /// <returns></returns>
        public int EgTagCreateFIFO(string stName, ushort RecordSize, ushort RecordCount, bool Reserved1_LetFalse, string stComment, ref object regName, ref ushort isLink)
        {
            int ret = -1;

            try
            {
                /* regName受け取り用ﾊﾞｯﾌｧ */
                var regName_buf = new StringBuilder(EGDEFINE.EG_MAX_TAGNAME + 1);

                //呼び出し
                ret = _C_EgTagCreateFIFO(stName, RecordSize, RecordCount, Reserved1_LetFalse, stComment, regName_buf, (ushort)regName_buf.Capacity, out isLink);
                if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());

                /* @memo 本当はregNameをstring型にすべきなのだが、他のCreate関数と揃える。 */
                regName = regName_buf.ToString();

                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// FIFOタグから1レコード取り出す(Entry指定)
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagDequeueFIFOByEntry<T>(IntPtr entry, ref T pBuffer, ref ushort wRemain)
        {
            int ret = -1;

            try
            {
                /* @memo ﾋﾟﾝ止めできる配列は、blittable型の1次元配列のみ。なので、ここでは多次元・入れ子を考慮しなくてよい */
                int size = typeof(T).IsArray ? (Marshal.SizeOf(typeof(T).GetElementType()) * ((Array)(object)pBuffer).Length) : Marshal.SizeOf<T>();
                GCHandle gch = GCHandle.Alloc(pBuffer, GCHandleType.Pinned);
                try
                {
                    ret = _C_EgTagDequeueFIFOByEntry(entry, gch.AddrOfPinnedObject(), (ushort)size, out wRemain);
                    if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                }
                finally
                {
                    gch.Free();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            return ret;
        }

        /// <summary>
        /// FIFOタグから1レコード取り出す(INDEX指定)
        /// </summary>
        /// <param name="index"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagDequeueFIFOByIndex<T>(EGOBJINDEX index, ref T pBuffer, ref ushort wRemain)
        {
            int ret = -1;

            try
            {
                /* @memo ﾋﾟﾝ止めできる配列は、blittable型の1次元配列のみ。なので、ここでは多次元・入れ子を考慮しなくてよい */
                int size = typeof(T).IsArray ? (Marshal.SizeOf(typeof(T).GetElementType()) * ((Array)(object)pBuffer).Length) : Marshal.SizeOf<T>();
                GCHandle gch = GCHandle.Alloc(pBuffer, GCHandleType.Pinned);
                try
                {
                    ret = _C_EgTagDequeueFIFOByIndex(ref index, gch.AddrOfPinnedObject(), (ushort)size, out wRemain);
                    if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                }
                finally
                {
                    gch.Free();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            return ret;
        }

        /// <summary>
        /// FIFOタグから1レコード取り出す
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagDequeueFIFO<T>(string stName, ref T pBuffer, ref ushort wRemain)
        {
            int ret = -1;

            try
            {
                /* @memo ﾋﾟﾝ止めできる配列は、blittable型の1次元配列のみ。なので、ここでは多次元・入れ子を考慮しなくてよい */
                int size = typeof(T).IsArray ? (Marshal.SizeOf(typeof(T).GetElementType()) * ((Array)(object)pBuffer).Length) : Marshal.SizeOf<T>();
                GCHandle gch = GCHandle.Alloc(pBuffer, GCHandleType.Pinned);
                try
                {
                    ret = _C_EgTagDequeueFIFO(stName, gch.AddrOfPinnedObject(), (ushort)size, out wRemain);
                    if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                }
                finally
                {
                    gch.Free();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            return ret;
        }

        /// <summary>
        /// FIFOタグに1レコード書き込む(Entry指定)
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagEnqueueFIFOByEntry<T>(IntPtr entry, ref T pBuffer, ref ushort wRemain)
        {
            int ret = -1;
            
            try
            {
                /* @memo ﾋﾟﾝ止めできる配列は、blittable型の1次元配列のみ。なので、ここでは多次元・入れ子を考慮しなくてよい */
                int size = typeof(T).IsArray ? (Marshal.SizeOf(typeof(T).GetElementType()) * ((Array)(object)pBuffer).Length) : Marshal.SizeOf<T>();
                GCHandle gch = GCHandle.Alloc(pBuffer, GCHandleType.Pinned);
                try
                {
                    ret = _C_EgTagEnqueueFIFOByEntry(entry, gch.AddrOfPinnedObject(), (ushort) size, out wRemain);
                    if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                }
                finally{
                    gch.Free();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            return ret;
        }

        /// <summary>
        /// FIFOタグに1レコード書き込む(INDEX指定)
        /// </summary>
        /// <param name="index"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagEnqueueFIFOByIndex<T>(EGOBJINDEX index, ref T pBuffer, ref ushort wRemain)
        {
            int ret = -1;
            
            try
            {
                /* @memo ﾋﾟﾝ止めできる配列は、blittable型の1次元配列のみ。なので、ここでは多次元・入れ子を考慮しなくてよい */
                int size = typeof(T).IsArray ? (Marshal.SizeOf(typeof(T).GetElementType()) * ((Array)(object)pBuffer).Length) : Marshal.SizeOf<T>();
                GCHandle gch = GCHandle.Alloc(pBuffer, GCHandleType.Pinned);
                try
                {
                    ret = _C_EgTagEnqueueFIFOByIndex(ref index, gch.AddrOfPinnedObject(), (ushort)size, out wRemain);
                    if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                }
                finally{
                    gch.Free();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            return ret;
        }

        /// <summary>
        /// FIFOタグに1レコード書き込む
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="pBuffer"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagEnqueueFIFO<T>(string stName, ref T pBuffer, ref ushort wRemain)
        {
            int ret = -1;
            
            try
            {
                /* @memo ﾋﾟﾝ止めできる配列は、blittable型の1次元配列のみ。なので、ここでは多次元・入れ子を考慮しなくてよい */
                int size = typeof(T).IsArray ? (Marshal.SizeOf(typeof(T).GetElementType()) * ((Array)(object)pBuffer).Length) : Marshal.SizeOf<T>();
                GCHandle gch = GCHandle.Alloc(pBuffer, GCHandleType.Pinned);
                try
                {
                    ret = _C_EgTagEnqueueFIFO(stName, gch.AddrOfPinnedObject(), (ushort)size, out wRemain);
                    if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                }
                finally{
                    gch.Free();
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            return ret;
        }

        /// <summary>
        /// FIFOタグに残っているレコード数を取得する(Entry指定)
        /// </summary>
        /// <param name="entry"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagGetRemainFIFOByEntry(IntPtr entry, ref ushort wRemain)
        {
            int ret = -1;

            try
            {
                ret = _C_EgTagGetRemainFIFOByEntry(entry, out wRemain);
                if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// FIFOタグに残っているレコード数を取得する(INDEX指定)
        /// </summary>
        /// <param name="index"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagGetRemainFIFOByIndex(EGOBJINDEX index, ref ushort wRemain)
        {
            int ret = -1;

            try
            {
                ret = _C_EgTagGetRemainFIFOByIndex(ref index, out wRemain);
                if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// FIFOタグに残っているレコード数を取得する
        /// </summary>
        /// <param name="stName"></param>
        /// <param name="wRemain"></param>
        /// <returns></returns>
        public int EgTagGetRemainFIFO(string stName, ref ushort wRemain)
        {
            int ret = -1;

            try
            {
                ret = _C_EgTagGetRemainFIFO(stName, out wRemain);
                if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// FIFOタグの中身をクリアする(Entry指定)
        /// </summary>
        /// <param name="entry"></param>
        /// <returns></returns>
        public int EgTagClearFIFOByEntry(IntPtr entry)
        {
            int ret = -1;

            try
            {
                ret = _C_EgTagClearFIFOByEntry(entry);
                if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// FIFOタグの中身をクリアする(INDEX指定)
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public int EgTagClearFIFOByIndex(EGOBJINDEX index)
        {
            int ret = -1;

            try
            {
                ret = _C_EgTagClearFIFOByIndex(ref index);
                if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

        /// <summary>
        /// FIFOタグの中身をクリアする
        /// </summary>
        /// <param name="stName"></param>
        /// <returns></returns>
        public int EgTagClearFIFO(string stName)
        {
            int ret = -1;

            try
            {
                ret = _C_EgTagClearFIFO(stName);
                if (ret != 0) Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " Error " + ret.ToString());
                return ret;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return ret;
            }
        }

    }
}
