﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using RTedge;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Windows.Data;
using System.Collections.Generic;

namespace EgBrow
{
    /// <summary>
    /// コレクタ情報
    /// </summary>
    public class ClctInfo
    {
        #region フィールド
        public string Name;         // コレクタ名
        public string OwnerName;    // 収集するサービス名
        public uint Interval;       // 収集周期
        public byte Priority;       // スレッドプライオリティ
        public uint ColSize;        // 格納する1レコードのサイズ
        public uint RowSize;        // 格納数
        public string DsetName;     // データセット名
        public bool Type;           // コレクタタイプ(true:INtime, false:Win)
        public uint WritePos;       // 書込位置
        public uint RcvDataNum;     // 受信データ件数(バッファーに溜まっている件数)
        #endregion
    }

    /// <summary>
    /// ListViewに表示されるアイテム(１行)
    /// </summary>
    public class ClctItem : INotifyPropertyChanged
    {
        #region フィールド

        /// <summary>
        /// コレクタ情報
        /// </summary>
        public ClctInfo rec;
        /// <summary>
        /// リスト表示用No
        /// </summary>
        public int No;
        /// <summary>
        /// インデックス
        /// </summary>
        public edge_API.EGOBJINDEX index;
        /// <summary>
        /// 書込み位置の前回値
        /// </summary>
        private uint lastWritePosvalue;
        /// <summary>
        /// 受信データ件数の前回値
        /// </summary>
        private uint lastRcvDataNum;
        #endregion

        #region プロパティ
        public bool Update                                  // 書込み位置、受信データ件数の変化有無
        {
            get
            {
                bool changed = (!rec.WritePos.Equals(lastWritePosvalue) || !rec.RcvDataNum.Equals(lastRcvDataNum));
                lastWritePosvalue = rec.WritePos;
                lastRcvDataNum = rec.RcvDataNum;
                return changed;     // true:変化あり false:変化なし
            }

        }
        /// <summary>
        /// コレクタ名
        /// </summary>
        public string Name { get { return rec.Name; } }
        /// <summary>
        /// 収集するサービス名
        /// </summary>
        public string OwnerName { get { return rec.OwnerName; } }
        /// <summary>
        /// 収集周期
        /// </summary>
        public uint Interval { get { return rec.Interval; } }
        /// <summary>
        /// スレッドプライオリティ
        /// NOTE:コレクタタイプがINtimeの場合のみ表示。Windowsの場合はハイフン表示とする)
        /// </summary>
        public string Priority { get { return rec.Type ? rec.Priority.ToString() : "-"; } }
        /// <summary>
        /// 格納する1レコードのサイズ
        /// </summary>
        public uint ColSize { get { return rec.ColSize; } }
        /// <summary>
        /// 格納数
        /// </summary>
        public uint RowSize { get { return rec.RowSize; } }
        /// <summary>
        /// データセット名
        /// </summary>
        public string DsetName { get { return rec.DsetName; } }
        /// <summary>
        /// コレクタタイプ
        /// </summary>
        public string Type { get { return rec.Type ? "Real Time" : "-"; } }
        /// <summary>
        /// 書込位置(書込みカウンタ)
        /// </summary>
        public uint WritePos { get { return rec.WritePos; } }
        /// <summary>
        /// 受信データ件数(バッファーに溜まっている件数)
        /// </summary>
        public uint RcvDataNum { get { return rec.RcvDataNum; } }
        #endregion

        #region コンストラクタ
        public ClctItem(int iNo, edge_API.EGOBJINDEX idx, ClctInfo item)
        {
            No = iNo;       // リスト表示用No
            index = idx;    // タグインデックス情報
            rec = item;     // タグ情報
        }
        #endregion

        #region プライベート
        /// <summary>
        /// プロパティ変更通知
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
        private void Notify(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        #region パブリック
        /// <summary>
        /// データ更新指令
        /// </summary>
        public void Refresh()
        {
            Notify("Name");
            Notify("WritePos");
            Notify("RcvDataNum");
            Notify("Update");
        }
        #endregion

    }

    /// <summary>
    /// ListViewに表示されるリストデータ構造
    /// </summary>
    public class ClctList : ObservableCollection<ClctItem>, INotifyPropertyChanged
    {
        public ClctList()
        {
            // 複数スレッドからコレクション操作できるようにする
            BindingOperations.EnableCollectionSynchronization(this, new object());
        }
        // トータル数
        public int NumItems
        {
            get { return this.Count; }
        }
        public new event PropertyChangedEventHandler PropertyChanged;
        private void Notifiy(String propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        public void Refresh()
        {
            Notifiy("NumItems");
        }
        /// <summary>
        /// CSV出力用データ取得
        /// </summary>
        /// <returns></returns>
        public List<List<string>> GetCsvOutData()
        {
            // CSVに出力するためのデータを生成します。
            List<List<string>> data = new List<List<string>>();

            // ヘッダー部
            List<string> h = new List<string>();
            h.Add("Name");
            h.Add("Type");
            h.Add("OwnerName");
            h.Add("Interval[ms]");
            h.Add("Priority");
            h.Add("ColSize");
            h.Add("RowSize");
            h.Add("DatasetName");
            h.Add("WritePos");
            h.Add("Queue");
            data.Add(h);

            // データ部
            foreach (var item in this)
            {
                List<string> d = new List<string>();
                d.Add(item.Name);
                d.Add(item.Type);
                d.Add(item.OwnerName);
                d.Add(item.Interval.ToString());
                d.Add(item.Priority);
                d.Add(item.ColSize.ToString());
                d.Add(item.RowSize.ToString());
                d.Add(item.DsetName);
                d.Add(item.WritePos.ToString());
                d.Add(item.RcvDataNum.ToString());
                data.Add(d);
            }
            return data;
        }
    }
}

