﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RTedgeXmlObj;     // RTedgeの構造定義
using EgBoot;           // XMLシリアライズ・デシリアライズ
using System.Xml.Linq;                      // LINQtoXML
using System.IO.Compression;
using System.IO;
using System.Windows.Forms;

namespace EgConfig
{
    /// <summary>
    /// ECIファイル管理用クラス
    /// </summary>
    public class EciFileMgmt
    {
        public string eciFileName;      // ECIファイル名
        public RTedge eciData;          // ECIファイルデータ
        public bool isChanged;          // ファイル変更状態保持

        /// <summary>
        /// ECIファイル(.xml)読み込み
        /// </summary>
        /// <param name="filename">ECIファイルパス</param>
        /// <returns>ECIファイル管理データ</returns>
        public static EciFileMgmt ReadEciFile(string filename)
        {
            EciFileMgmt eciFileMgmt = null;    // 読み込んだECIデータを保持
            RTedgeXmlObj.RTedge rtedge;                     // ECIファイルデシリアライズ用クラス

            // ECIファイルデシリアライズ
            // TODO:ファイルが見つからない場合nullとなる のちのTreeViewに表示するリストを作成する際、NullExceptionが発生する
            try
            {
                rtedge = MnXmlUtility.LoadFromFile<RTedgeXmlObj.RTedge>(filename);
                if(rtedge != null)
                {
                    eciFileMgmt = new EciFileMgmt();
                    eciFileMgmt.eciFileName = filename;
                    eciFileMgmt.eciData = rtedge;
                }
                
            }
            catch (Exception)
            {
                // ファイルがない・データを読み込めなかった
            }

            return eciFileMgmt;
        }

        /// <summary>
        /// ECIファイルを保存する
        /// </summary>
        /// <returns>保存成功:true, 保存失敗:false</returns>
        public bool Save()
        {
            bool isDone = false;
            try
            {
                MnXmlUtility.SaveToFile(this.eciFileName, this.eciData, typeof(RTedge));  // XMLシリアライズ
                isChanged = false;
                isDone = true;
            }
            catch (Exception)
            {
                isDone = false;
            }

            return isDone;
        }

        /// <summary>
        /// Tagを追加する
        /// </summary>
        /// <param name="tag">Tagデータ</param>
        public void AddTag(Tag tag)
        {
            eciData.Tags.Add(tag);
        }

        /// <summary>
        /// Serviceを追加する
        /// </summary>
        /// <param name="service">Serviceデータ</param>
        public void AddService(Service service)
        {
            eciData.Services.Add(service);
        }

        /// <summary>
        /// Service以下Tagを追加する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="tag">Tagデータ</param>
        public void AddServiceTag(int serviceIndex, Tag tag)
        {
            // serviceのINDEXが存在しない場合追加処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }

            if (eciData.Services[serviceIndex].Tags == null)
            {
                eciData.Services[serviceIndex].Tags = new List<Tag>();
            }
            eciData.Services[serviceIndex].Tags.Add(tag);
        }
        /// <summary>
        /// Service以下Datasetを追加する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="dataset">Datasetデータ</param>
        public void AddServiceDataset(int serviceIndex, Dataset dataset)
        {
            // serviceのINDEXが存在しない場合追加処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }

            if (eciData.Services[serviceIndex].Datasets == null)
            {
                eciData.Services[serviceIndex].Datasets = new List<Dataset>();
            }
            eciData.Services[serviceIndex].Datasets.Add(dataset);
        }
        

        /// <summary>
        /// Mailboxを追加する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="mailBox"Mailboxデータ></param>
        public void AddMailbox(int serviceIndex, MailBox mailBox)
        {
            // serviceのINDEXが存在しない場合追加処理を行わない
            if(!(serviceIndex < eciData.Services.Count))
            {
                return;
            }

            if(eciData.Services[serviceIndex].MailBoxes == null)
            {
                eciData.Services[serviceIndex].MailBoxes = new List<MailBox>();
            }
            eciData.Services[serviceIndex].MailBoxes.Add(mailBox);
        }

        /// <summary>
        /// Tagsを削除する
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTags()
        {
            eciData.Tags.Clear();
        }
        /// <summary>
        /// Tagを削除する
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTag(TreeItem treeItem)
        {
            // tagのINDEXが存在しない場合削除処理を行わない
            if (!(treeItem.Index < eciData.Tags.Count))
            {
                return;
            }
            eciData.Tags.RemoveAt(treeItem.Index);
        }
        /// <summary>
        /// Service以下のTagsを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveTags(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Tags.Clear();
        }
        /// <summary>
        /// Service以下のTagを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveTag(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはTagのindexが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].Tags.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Tags.RemoveAt(treeItem.Index);
        }


        /// <summary>
        /// Datasetsを削除する
        /// </summary>
        public void RemoveDatasets()
        {
            eciData.Datasets.Clear();
        }

        /// <summary>
        /// Datasetを削除する
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveDataset(TreeItem treeItem)
        {
            // DatasetのINDEXが存在しない場合削除処理を行わない
            if (!(treeItem.Index < eciData.Datasets.Count))
            {
                return;
            }
            eciData.Datasets.RemoveAt(treeItem.Index);
        }
        /// <summary>
        /// Service以下のDatasetsを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveDatasets(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Datasets.Clear();
        }
        /// <summary>
        /// Service以下のDatasetsを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveDataset(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはDatasetのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].Datasets.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Datasets.RemoveAt(treeItem.Index);
        }
        /// <summary>
        /// DatasetTagRefsを削除する
        /// </summary>
        /// <param name="datasetIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveDatasetTagRefs(TreeItem treeItem)
        {
            // DatasetのINDEXが存在しない場合削除処理を行わない
            if (!(treeItem.Index < eciData.Datasets.Count))
            {
                return;
            }
            eciData.Datasets[treeItem.Index].TagRefs.Clear();
        }

        /// <summary>
        /// DatasetTagRefを削除する
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveDatasetTagRef(int datasetIndex, TreeItem treeItem)
        {
            // DatasetのINDEXまたはTagRefのINDEXが存在しない場合、削除処理を行わない
            if (!(datasetIndex < eciData.Datasets.Count) || !(treeItem.Index < eciData.Datasets[datasetIndex].TagRefs.Count))
            {
                return;
            }
            eciData.Datasets[datasetIndex].TagRefs.RemoveAt(treeItem.Index);
        }
        /// <summary>
        /// Service以下のDatasetのDatasetTagRefsを削除する
        /// </summary>
        /// <param name="datasetIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveDatasetTagRefs(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            // DatasetのINDEXが存在しない場合削除処理を行わない
            if (!(treeItem.Index < eciData.Services[serviceIndex].Datasets.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Datasets[treeItem.Index].TagRefs.Clear();
        }

        /// <summary>
        /// Service以下のDatasetTagRefを削除する
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveDatasetTagRef(int serviceIndex, int datasetIndex, TreeItem treeItem)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            // DatasetのINDEXが存在しない場合削除処理を行わない
            if (!(datasetIndex < eciData.Services[serviceIndex].Datasets.Count) || !(treeItem.Index < eciData.Services[serviceIndex].Datasets[datasetIndex].TagRefs.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Datasets[datasetIndex].TagRefs.RemoveAt(treeItem.Index);
        }

        /// <summary>
        /// TagRefsを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveTagRefs(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].TagRefs.Clear();
        }

        /// <summary>
        /// TagRefを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveTagRef(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはTagRefのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].TagRefs.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].TagRefs.RemoveAt(treeItem.Index);
        }

        /// <summary>
        /// TagRefsINを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveTagRefsIN(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].TagRefs_IN.Clear();
        }

        /// <summary>
        /// TagRefINを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveTagRefIN(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはTagRefのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].TagRefs_IN.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].TagRefs_IN.RemoveAt(treeItem.Index);
        }

        /// <summary>
        /// TagRefsOUTを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveTagRefsOUT(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].TagRefs_OUT.Clear();
        }

        /// <summary>
        /// TagRefOUTを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveTagRefOUT(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはTagRefのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].TagRefs_OUT.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].TagRefs_OUT.RemoveAt(treeItem.Index);
        }

        /// <summary>
        /// Collectorsを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveCollectors(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Collectors.Clear();
        }

        /// <summary>
        /// Collectorを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveCollector(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはCollectorのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].Collectors.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].Collectors.RemoveAt(treeItem.Index);
        }

        /// <summary>
        /// CollectorRefsを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveCollectorRefs(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].CollectorRefs.Clear();
        }

        /// <summary>
        /// CollectorRefを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveCollectorRef(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはCollectorRefのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].CollectorRefs.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].CollectorRefs.RemoveAt(treeItem.Index);
        }

        /// <summary>
        /// Mailboxesを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveMailboxes(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].MailBoxes.Clear();
        }

        /// <summary>
        /// Mailboxを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        /// <param name="treeItem"></param>
        public void RemoveMailbox(int serviceIndex, TreeItem treeItem)
        {
            // ServiceのINDEXまたはMailBoxのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count) || !(treeItem.Index < eciData.Services[serviceIndex].MailBoxes.Count))
            {
                return;
            }
            eciData.Services[serviceIndex].MailBoxes.RemoveAt(treeItem.Index);
        }

        /// <summary>
        /// Servicesを削除する
        /// </summary>
        public void RemoveServices()
        {
            eciData.Services.Clear();
        }

        /// <summary>
        /// Serviceを削除する
        /// </summary>
        /// <param name="serviceIndex"></param>
        public void RemoveService(int serviceIndex)
        {
            // ServiceのINDEXが存在しない場合、削除処理を行わない
            if (!(serviceIndex < eciData.Services.Count))
            {
                return;
            }
            eciData.Services.RemoveAt(serviceIndex);
        }
    }

    /// <summary>
    /// ECIプロジェクト管理用クラス
    /// </summary>
    public class EciProjMgmt
    {
        public string eciProjName;          // プロジェクト名
        public List<EciFileMgmt> eciList;   // ECIファイル管理リスト

        public EciProjMgmt()
        {
            eciProjName = "";
            eciList = new List<EciFileMgmt>();
        }

        /// <summary>
        /// ECIプロジェクトファイル(.eciproj)の作成
        /// </summary>
        /// <param name="filename">ECIプロジェクトファイルを保存するパス</param>
        public static void CreateNewProject(string filename)
        {
            ///////////////////////////
            // バージョン情報取得
            ///////////////////////////
            System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
            System.Reflection.AssemblyName asmName = assembly.GetName();
            System.Version version = asmName.Version;

            ///////////////////////////
            // XMLファイル作成処理
            ///////////////////////////
            var xml = new XDocument
            (
                new XDeclaration("1.0", "UTF-8", "true"),
                new XElement
                (
                    "EciProject",
                    new XElement("Version", new XAttribute("ToolVer", version.ToString())),
                    new XElement("EciFiles",
                                  new XElement("EciFile", new XAttribute("Name", "EgBoot.xml")))
                )
            );

            xml.Save(filename);                   // 指定したフォルダにXMLファイルを保存
        }

        /// <summary>
        /// .eciprojファイルの保存
        /// </summary>
        /// <returns>作成成功:true, 作成失敗:false</returns>
        public bool SaveEciProject()
        {
            bool isDone = false;

            EciProject eciProject = new EciProject();
            EciFile eciFile;

            eciProject.Version = new Version();
            eciProject.EciFiles = new List<EciFile>();

            ///////////////////////////
            // バージョン情報取得
            ///////////////////////////
            System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
            System.Reflection.AssemblyName asmName = assembly.GetName();
            System.Version version = asmName.Version;

            eciProject.Version.ToolVer = version.ToString();    // バージョンの設定


            /////////////////////////////////////////////////////
            // 管理しているECIファイルの名前をリスト化
            /////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFileMgmt in this.eciList)
            {
                eciFile = new EciFile();
                eciFile.Name = System.IO.Path.GetFileName(eciFileMgmt.eciFileName);
                eciProject.EciFiles.Add(eciFile);
            }

            try
            {
                EgBoot.MnXmlUtility.SaveToFile(this.eciProjName, eciProject, typeof(EciProject));
                isDone = true;
            }
            catch (Exception)
            {
                isDone = false;
            }

            return isDone;
        }
             

        /// <summary>
        /// ECIプロジェクトファイル(.eciproj)読み込み
        /// </summary>
        /// <param name="filename">ECIプロジェクトファイルパス</param>
        /// <returns>ECIプロジェクト管理データ</returns>
        public static EciProjMgmt ReadProjectFile(string filename, out List<string> readFailures)
        {
            EciProjMgmt projMgmt = new EciProjMgmt();           // プロジェクト管理クラス
            readFailures = new List<string>();                  // ECIファイル読み込み失敗リスト
            
            EciProject eciProXmlObj = new EciProject();         // プロジェクトファイルデシリアライズ用クラス
            string srEciFilePath;                               // ECIファイルパス
            EciFileMgmt fileMgmt;                               // ECIファイル管理クラス

            projMgmt.eciProjName = filename;                                                // プロジェクトファイル名格納

            ////////////////////////////////////////////////////////////////////////////
            // プロジェクトファイルで管理しているすべてのECIファイル読み込み
            ////////////////////////////////////////////////////////////////////////////
            eciProXmlObj = MnXmlUtility.LoadFromFile<EgConfig.EciProject>(filename);        // プロジェクトファイルをデシリアライズ
            foreach (EciFile file in eciProXmlObj.EciFiles)
            {
                srEciFilePath = System.IO.Path.Combine(System.IO.Path.GetDirectoryName(filename), file.Name);   // ファイルパス作成

                //////////////////////////////////////
                // ECIファイル読み込み
                //////////////////////////////////////
                fileMgmt = new EciFileMgmt();
                fileMgmt = EciFileMgmt.ReadEciFile(srEciFilePath);
                if(fileMgmt is EciFileMgmt)
                {
                    projMgmt.eciList.Add(fileMgmt);
                }
                else
                {
                    readFailures.Add(srEciFilePath);    // ファイルを読み込めなかった
                }
            }

            return projMgmt;
        }

        /// <summary>
        /// ECIファイル読み込み
        /// (プロジェクトなしの読み込み)
        /// </summary>
        /// <param name="filename">ECIファイル名</param>
        public static EciProjMgmt ReadEciFile(string filename)
        {
            EciProjMgmt projMgmt = new EciProjMgmt();           // プロジェクト管理クラス
            EciFileMgmt fileMgmt;

            // ECIファイル読み込み
            fileMgmt = EciFileMgmt.ReadEciFile(filename);

            // プロジェクト管理情報を設定
            projMgmt.eciProjName = null;
            projMgmt.eciList.Add(fileMgmt);

            return projMgmt;
        }

        /// <summary>
        /// エクスポート処理
        /// プロジェクトファイルおよび管理しているECIファイルを圧縮して保存する
        /// </summary>
        /// <param name="path">保存先のファイルパス</param>
        /// <returns>true:エクスポート成功, false:エクスポート失敗</returns>
        public bool Export(string path)
        {
            bool isDone = false;
            string Name;

            try
            {
                //////////////////////////////////////
                // 圧縮ファイルを作成する
                //////////////////////////////////////
                using (ZipArchive zip = ZipFile.Open(path, ZipArchiveMode.Update))  // 指定したパスとモードでzipアーカイブを開く
                {
                    
                    // 圧縮ファイルの内容をすべてクリアする
                    for(int i = zip.Entries.Count - 1; i >= 0; i--)
                    {
                        zip.Entries[i].Delete();
                    }

                    // プロジェクトファイルを追加する
                    Name = System.IO.Path.GetFileName(this.eciProjName);
                    zip.CreateEntryFromFile(this.eciProjName, Name, CompressionLevel.NoCompression);              // zipアーカイブにファイルを追加する

                    // ECIファイルを追加する
                    foreach (EciFileMgmt eciFileMgmt in this.eciList)
                    {
                        Name = System.IO.Path.GetFileName(eciFileMgmt.eciFileName);
                        zip.CreateEntryFromFile(eciFileMgmt.eciFileName, Name, CompressionLevel.NoCompression);   // zipアーカイブにファイルを追加する
                    }

                    isDone = true;
                }
            }
            catch (Exception)
            {
                isDone = false;
            }
            
            return isDone;
        }

        /// <summary>
        /// インポート処理
        /// </summary>
        /// <param name="filepath">インポートするファイルパス(.ecipack)</param>
        /// <param name="folderpath">インポート先のファイル</param>
        /// <returns>true:インポート成功, false:インポート失敗</returns>
        public bool Import(string filepath, string folderpath)
        {
            bool isDone = false;
            EciProjMgmt eciProjMgmt;
            string projPath = "", filenames = "";

            // 解凍
            try
            {
                using (ZipArchive zip = ZipFile.Open(filepath, ZipArchiveMode.Update))
                {
                    foreach(ZipArchiveEntry entry in zip.Entries)
                    {
                        entry.ExtractToFile(System.IO.Path.Combine(folderpath, entry.Name));
                        // プロジェクトファイル名保持
                        if (entry.Name.Contains(".eciproj") == true)
                        {
                            projPath = System.IO.Path.Combine(folderpath, entry.FullName);
                        }
                    }

                    isDone = true;
                }
            }
            catch (Exception)
            {
                return isDone;
            }

            // 読み込み
            // TODO:プロジェクトファイル名が読めなかったら？
            eciProjMgmt = EciProjMgmt.ReadProjectFile(projPath, out List<string> readFailures);
            this.eciProjName = eciProjMgmt.eciProjName;
            this.eciList = eciProjMgmt.eciList;

            if (0 < readFailures.Count)
            {
                foreach (string readFailure in readFailures)
                {
                    try
                    {
                        filenames = filenames + System.IO.Path.GetFileName(readFailure) + "\n";
                    }
                    catch (Exception)
                    {
                        filenames = filenames + "" + "\n";
                    }
                }
                System.Windows.Forms.MessageBox.Show(String.Format(Properties.Resources.MSGBOX_ERROR_003_MSGTEXT, filenames), Properties.Resources.MSGBOX_ERROR_003_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Hand);
            }

            return isDone;
        }

        /// <summary>
        /// ファイルの更新
        /// </summary>
        /// <param name="filename">ECIファイル名</param>
        /// <returns>-1以外: ファイル更新成功(ECIファイルリストの更新したインデックス), -1:ファイル更新失敗
        /// (失敗原因:プロジェクトにファイルが管理されていない)</returns>
        public int UpdateEciFile(string filename)
        {
            int index;
            EciFileMgmt eciFileMgmt;

            index = SearchEciIndex(filename);
            if (index < 0)
            {
                return index;
            }

            // ECIファイルの読み込み
            eciFileMgmt = EciFileMgmt.ReadEciFile(filename);
            if(eciFileMgmt == null) // 読み込み失敗(ECIファイルの形式が違う)
            {
                return -1;
            }

            this.eciList.RemoveAt(index);
            if(index < eciList.Count)
            {
                this.eciList.Insert(index, eciFileMgmt);
            }
            else
            {
                this.eciList.Add(eciFileMgmt);
                index = eciList.Count - 1;
            }

            return index;
        }

        /// <summary>
        /// プロジェクトファイルおよび管理中のECIファイルにおいて、変更されているファイルパスを取得
        /// </summary>
        /// <param name="paths">変更されているファイルパス</param>
        /// <returns>true:変更済みのファイルがあるか, false:変更ファイルがないか</returns>
        public bool GetChangedFilename(out List<string> paths)
        {
            bool isChange = false;
            paths = new List<string>();
            
            // 変更されているファイルパス取得
            foreach(EciFileMgmt eciFileMgmt in eciList)
            {
                if(eciFileMgmt?.isChanged == true)
                {
                    if(isChange == false)
                    {
                        isChange = true;
                    }
                    paths.Add(eciFileMgmt.eciFileName);
                }
            }

            return isChange;
        }

        /// <summary>
        /// プロジェクトで管理するECIファイルを追加
        /// </summary>
        /// <param name="filename"></param>
        public void AddEciFile(string filename)
        {
            EciFileMgmt eciFileMgmt;
            string path, dir, name;

            dir = System.IO.Path.GetDirectoryName(this.eciProjName);
            name = System.IO.Path.GetFileName(filename);
            path = System.IO.Path.Combine(dir, name);

            eciFileMgmt = EciFileMgmt.ReadEciFile(filename);                // ECIファイル読み込み
            eciFileMgmt.eciFileName = path;                                 // ファイルパスをプロジェクトと同ディレクトリに変更する
            eciList.Add(eciFileMgmt);                                       // ECIファイル管理リストに追加
        }

        /// <summary>
        /// プロジェクトからECIファイルを除外する
        /// </summary>
        /// <param name="filename">管理から除外するECIファイル</param>
        /// <returns>0以上:除外成功,-1:除外失敗</returns>
        public int RemoveEciFile(string filename)
        {
            int index;

            // 管理しているECIリストのIndexを検索
            index = SearchEciIndex(filename);
            if (index < 0) { return index; }    // ファイルが見つからなかった
            
            eciList.RemoveAt(index);
            return index;
        }

        /// <summary>
        /// ファイルを保存
        /// </summary>
        /// <param name="filename">保存するファイル名</param>
        /// <returns>true:成功, false:失敗</returns>
        public bool SaveFile(string filename)
        {
            bool result = false;    // ファイル保存結果
            int index = 0;          // 検索用インデックス番号

            // リスト検索し、一致したIndexNoを返却する
            index = eciList.FindIndex(eciFile => eciFile.eciFileName == filename);
            if (0 <= index) // 一致したIndexがある
            {
                result = eciList[index].Save();
                result = true;
            }

            return result;
        }

        /// <summary>
        /// プロジェクトを保存する
        /// </summary>
        /// <returns></returns>
        public bool SaveAll()
        {
            bool isDone = false;

            // ECIプロジェクトの保存
            isDone = SaveEciProject();
            if (isDone == false)
            {
                return isDone;
            }

            // ECIファイルを保存
            foreach(EciFileMgmt eciFileMgmt in this.eciList)
            {
                isDone = eciFileMgmt.Save();
            }

            return isDone;
        }

        /// <summary>
        /// 対象のECIファイルのファイル変更状態を設定する
        /// </summary>
        /// <param name="isChanged">ファイル変更状態あり:true, なし:false</param>
        /// <param name="filename">対象のECIファイル</param>
        /// <returns>変更成功:true, 変更失敗:false</returns>
        public bool SetIsChanged2EciFile(bool isChanged, string filename)
        {
            bool isDone = false;
            int index = -1;

            // リスト検索し、一致したIndexNoを返却する
            index = eciList.FindIndex(eciFile => eciFile.eciFileName == filename);
            if(index < 0)
            {
                return isDone;
            }

            eciList[index].isChanged = isChanged;
            isDone = true;

            return isDone;
        }

        /// <summary>
        /// TreeViewで選択した項目から管理データを取得
        /// </summary>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        public object GetItemData(TreeItem treeItem)
        {
            object data = 0;
            int index = 0;          // 検索用インデックス番号

            // 選択されたViewから元データのファイルパスを検索する
            // リスト検索し、一致したIndexNoを返却する
            index = eciList.FindIndex(eciFile => eciFile.eciFileName == treeItem.Filename);

            // 項目タイプ、IndexNoをもとにデータを取り出す
            switch (treeItem.ItemType)
            {
                case (int)ItemType.ECI:
                    data = GetEciData(index);
                    break;
                case (int)ItemType.Tags:
                    data = GetTagsData(index, treeItem);
                    break;
                case (int)ItemType.Tag:
                    data = GetTagData(index, treeItem);
                    break;
                case (int)ItemType.Datasets:
                    data = GetDatasetsData(index, treeItem);
                    break;
                case (int)ItemType.Dataset:
                    data = GetDatasetData(index, treeItem);
                    break;
                case (int)ItemType.DatasetTagRefs:
                    data = GetDatasetTagRefsData(index, treeItem);
                    break;
                case (int)ItemType.DatasetTagRef:
                    data = GetDatasetTagRefData(index, treeItem);
                    break;
                case (int)ItemType.TagRefs:
                    data = GetTagRefsData(index, treeItem);
                    break;
                case (int)ItemType.TagRef:
                    data = GetTagRefData(index, treeItem);
                    break;
                case (int)ItemType.TagRefsIN:
                    data = GetTagRefsINData(index, treeItem);
                    break;
                case (int)ItemType.TagRefIN:
                    data = GetTagRefINData(index, treeItem);
                    break;
                case (int)ItemType.TagRefsOUT:
                    data = GetTagRefsOUTData(index, treeItem);
                    break;
                case (int)ItemType.TagRefOUT:
                    data = GetTagRefOUTData(index, treeItem);
                    break;
                case (int)ItemType.Services:
                    data = GetServicesData(index);
                    break;
                case (int)ItemType.Service:
                    data = GetServiceData(index, treeItem);
                    break;
                case (int)ItemType.Collectors:
                    data = GetCollectorsData(index, treeItem);
                    break;
                case (int)ItemType.Collector:
                    data = GetCollectorData(index, treeItem);
                    break;
                case (int)ItemType.CollectorRefs:
                    data = GetCollectorRefsData(index, treeItem);
                    break;
                case (int)ItemType.CollectorRef:
                    data = GetCollectorRefData(index, treeItem);
                    break;
                case (int)ItemType.MailBoxes:
                    data = GetMailboxesData(index, treeItem);
                    break;
                case (int)ItemType.MailBox:
                    data = GetMailboxData(index, treeItem);
                    break;
                default:
                    break;
            }

            return data;
        }

        /// <summary>
        /// ECIファイル名を取得
        /// </summary>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetEciData(int listNo)
        {
            RTedgeECI rtedgeECI = new RTedgeECI(listNo);

            return rtedgeECI;
        }

        /// <summary>
        /// Tagsのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <returns></returns>
        private object GetTagsData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;
            // Serviceを取得
            TreeItem Service = treeItem.SearchService();
            // 管理データ取得
            if (listNo < eciList.Count)
            {
                obj = eciList[listNo].eciData.Tags;
                if (Service == null) //トップのTags
                {
                    obj = eciList[listNo].eciData.Tags;
                }
                else//サービス以下の場合
                {
                    indexParent = treeItem.Parent.Index;
                    if (indexParent < eciList[listNo].eciData.Services.Count)//Index値がServiceの数内であれば処理
                    {
                        obj = eciList[listNo].eciData.Services[indexParent].Tags;//対象のTagsを取得
                    }
                }
            }
            return obj;
        }

        /// <summary>
        /// Tagのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        private object GetTagData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;
            // Serviceを取得
            TreeItem Service = treeItem.SearchService();
            if (listNo < eciList.Count)
            {
                if (Service == null) //トップのDataset
                {
                    if (treeItem.Index < eciList[listNo].eciData.Tags.Count) //Index値がTagの数内であれば処理
                    {
                        // 管理データ取得
                        obj = eciList[listNo].eciData.Tags[treeItem.Index];//対象のtagを取得
                    }
                }
                else//サービス以下の場合
                {
                    indexParent = GetIndexOfParentList(treeItem);
                    if (indexParent < eciList[listNo].eciData.Services.Count)//Index値がTagの数内であれば処理
                    {
                        if (treeItem.Index < eciList[listNo].eciData.Services[indexParent].Tags.Count) //Index値がTagの数内であれば処理
                        {
                            // 管理データ取得
                            obj = eciList[listNo].eciData.Services[indexParent].Tags[treeItem.Index];//対象のTagを取得
                        }                      
                    }
                }
            }
            return obj;
        }

        /// <summary>
        /// Datasetsのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <returns></returns>
        private object GetDatasetsData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;
            // Serviceを取得
            TreeItem Service = treeItem.SearchService();
            // 管理データ取得
            if (listNo < eciList.Count)
            {
                obj = eciList[listNo].eciData.Datasets;
                if (Service == null) //トップのDataset
                {
                    obj = eciList[listNo].eciData.Datasets;
                }
                else//サービス以下の場合
                {
                    indexParent = treeItem.Parent.Index;
                    if (indexParent < eciList[listNo].eciData.Services.Count)//Index値がServiceの数内であれば処理
                    {
                        obj = eciList[listNo].eciData.Services[indexParent].Datasets;//対象のDatasetsを取得
                    }
                }
            }

            return obj;
        }

        /// <summary>
        /// Datasetのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        private object GetDatasetData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;
            // Serviceを取得
            TreeItem Service = treeItem.SearchService();
            if (listNo < eciList.Count)
            {
                if (Service == null) //トップのDataset
                {
                    if(treeItem.Index < eciList[listNo].eciData.Datasets.Count) //Index値がDatasetの数内であれば処理
                    {
                        // 管理データ取得
                        obj = eciList[listNo].eciData.Datasets[treeItem.Index];//対象のDatasetを取得
                    }
                }
                else//サービス以下の場合
                {
                    indexParent = GetIndexOfParentList(treeItem);
                    if (indexParent < eciList[listNo].eciData.Services.Count)//Index値がDatasetの数内であれば処理
                    {
                        if (treeItem.Index < eciList[listNo].eciData.Services[indexParent].Datasets.Count) //Index値がDatasetの数内であれば処理
                        {
                            // 管理データ取得
                            obj = eciList[listNo].eciData.Services[indexParent].Datasets[treeItem.Index];//対象のDatasetを取得
                        }
                    }
                }
            }
            return obj;
        }

        /// <summary>
        /// Tagrefs(Dataset)のデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        private object GetDatasetTagRefsData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;
            TreeItem Service = treeItem.SearchService();
            if (listNo < eciList.Count)
            {
                // 選択しているTagRefsがDatasetリストのどのIndexか取得
                if (treeItem.Parent is TreeItem)
                {
                    indexParent = treeItem.Parent.Index;
                    if (Service == null) //トップのDataset
                    {
                        // 管理データ取得
                        if (treeItem.Index < eciList[listNo].eciData.Datasets.Count)
                        {
                            obj = eciList[listNo].eciData.Datasets[indexParent].TagRefs;
                        }
                    }
                    else//サービス以下の場合
                    {
                        // 管理データ取得
                        if (treeItem.Index < eciList[listNo].eciData.Services[Service.Index].Datasets.Count)
                        {
                            obj = eciList[listNo].eciData.Services[Service.Index].Datasets[indexParent].TagRefs;
                        }
                    }
                }
            }

            return obj;
        }

        /// <summary>
        /// Tagref(Dataset)のデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        private object GetDatasetTagRefData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;
            TreeItem Service = treeItem.SearchService();
            if (listNo < eciList.Count)
            {
                // 選択しているDatasetがDatasetリストのどのIndexか取得
                if (treeItem.Parent is TreeItem)
                {
                    indexParent = GetIndexOfParentList(treeItem);
                    if (Service == null) //トップのDataset
                    {
                        // 管理データ取得
                        if (indexParent < eciList[listNo].eciData.Datasets.Count &&
                            treeItem.Index < eciList[listNo].eciData.Datasets[indexParent].TagRefs.Count)
                        {
                            obj = eciList[listNo].eciData.Datasets[indexParent].TagRefs[treeItem.Index];
                        }
                    }
                    else//サービス以下の場合
                    {
                        // 管理データ取得
                        if (indexParent < eciList[listNo].eciData.Services[Service.Index].Datasets.Count &&
                            treeItem.Index < eciList[listNo].eciData.Services[Service.Index].Datasets[indexParent].TagRefs.Count)
                        {
                            obj = eciList[listNo].eciData.Services[Service.Index].Datasets[indexParent].TagRefs[treeItem.Index];
                        }
                    }
                }
            }

            return obj;
        }

        /// <summary>
        /// TagRefs(Service内)のデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetTagRefsData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = treeItem.Parent.Index;

                // 管理データ取得
                if(listNo < eciList.Count && indexParent < eciList[listNo].eciData.Services.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].TagRefs;
                }
            }
                

            return obj;
        }

        /// <summary>
        /// TagRef(Service内)のデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetTagRefData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = GetIndexOfParentList(treeItem);

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count &&
                    treeItem.Index < eciList[listNo].eciData.Services[indexParent].TagRefs.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].TagRefs[treeItem.Index];
                }
            }

            return obj;
        }

        /// <summary>
        /// TagRefs_INのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetTagRefsINData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = treeItem.Parent.Index;

                // 管理データ取得
                if (listNo < eciList.Count && indexParent < eciList[listNo].eciData.Services.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].TagRefs_IN;
                }

            }

            return obj;
        }

        /// <summary>
        /// TagRef_INのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetTagRefINData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = GetIndexOfParentList(treeItem);

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count &&
                    treeItem.Index < eciList[listNo].eciData.Services[indexParent].TagRefs_IN.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].TagRefs_IN[treeItem.Index];
                }
            }

            return obj;
        }

        /// <summary>
        /// TagRefs_OUTのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetTagRefsOUTData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = treeItem.Parent.Index;

                // 管理データ取得
                if (listNo < eciList.Count && indexParent < eciList[listNo].eciData.Services.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].TagRefs_OUT;
                }
            }

            return obj;
        }

        /// <summary>
        /// TagRef_OUTのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetTagRefOUTData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = GetIndexOfParentList(treeItem);

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count &&
                    treeItem.Index < eciList[listNo].eciData.Services[indexParent].TagRefs_OUT.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].TagRefs_OUT[treeItem.Index];
                }
            }

            return obj;
        }

        /// <summary>
        /// Serviceのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetServicesData(int listNo)
        {
            object obj = 0;

            // 管理データ取得
            if(listNo < eciList.Count)
            {
                obj = eciList[listNo].eciData.Services;
            }

            return obj;
        }

        /// <summary>
        /// Serviceのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetServiceData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            
            // 管理データ取得
            if (listNo < eciList.Count &&
                treeItem.Index < eciList[listNo].eciData.Services.Count)
            {
                obj = eciList[listNo].eciData.Services[treeItem.Index];
            }

            return obj;
        }

        /// <summary>
        /// Collectorsのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetCollectorsData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = treeItem.Parent.Index;

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].Collectors;
                }
            }

            return obj;
        }

        /// <summary>
        /// Collectorのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetCollectorData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがサービスリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = GetIndexOfParentList(treeItem);

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count &&
                    treeItem.Index < eciList[listNo].eciData.Services[indexParent].Collectors.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].Collectors[treeItem.Index];
                }
            }
                

            return obj;
        }

        /// <summary>
        /// CollectorRefsのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetCollectorRefsData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = treeItem.Parent.Index;

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].CollectorRefs;
                }
            }
            
            return obj;
        }

        /// <summary>
        /// CollectorRefのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetCollectorRefData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがサービスリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = GetIndexOfParentList(treeItem);

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count &&
                    treeItem.Index < eciList[listNo].eciData.Services[indexParent].CollectorRefs.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].CollectorRefs[treeItem.Index];
                }
            }
            

            return obj;
        }

        /// <summary>
        /// Mailboxesのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        private object GetMailboxesData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがServiceリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = treeItem.Parent.Index;

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].MailBoxes;
                }
            }
            

            return obj;
        }

        /// <summary>
        /// メールボックスのデータ取得
        /// </summary>
        /// <param name="listNo"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        private object GetMailboxData(int listNo, TreeItem treeItem)
        {
            object obj = 0;
            int indexParent = 0;

            // 選択しているサービスがサービスリストのどのIndexか取得
            if (treeItem.Parent is TreeItem)
            {
                indexParent = GetIndexOfParentList(treeItem);

                // 管理データ取得
                if (listNo < eciList.Count &&
                    indexParent < eciList[listNo].eciData.Services.Count &&
                    treeItem.Index < eciList[listNo].eciData.Services[indexParent].MailBoxes.Count)
                {
                    obj = eciList[listNo].eciData.Services[indexParent].MailBoxes[treeItem.Index];
                }
            }
            

            return obj;
        }

        /// <summary>
        /// 親リストのIndexを取得
        /// </summary>
        /// <returns></returns>
        private int GetIndexOfParentList(TreeItem treeItem)
        {
            int index = 0;

            index = treeItem.Parent.Parent.Index;

            return index;
        }

        /// <summary>
        /// 管理下のECIファイルのトップ、またはService配下にTagを追加する
        /// </summary>
        /// <param name="treeItem"></param>
        /// <param name="tag"></param>
        public void AddTag(TreeItem treeItem, Tag tag)
        {
            int index;

            // Serviceを取得
            TreeItem Service = treeItem.SearchService();

            index = SearchEciIndex(treeItem.Filename);
            if (0 <= index)
            {
                if(Service == null) //トップのTagの追加処理
                    eciList[index].AddTag(tag);
                else//Service以下のTagの追加処理
                    eciList[index].AddServiceTag(Service.Index, tag);
            }
        }

        /// <summary>
        /// MailBoxを追加
        /// </summary>
        /// <param name="treeItem"></param>
        /// <param name="mailBox"></param>
        public void AddMailbox(TreeItem treeItem, MailBox mailBox)
        {
            int index;

            // Serviceを取得
            TreeItem Service = treeItem.SearchService();

            index = SearchEciIndex(treeItem.Filename);
            if (0 <= index)
            {
                eciList[index].AddMailbox(Service.Index, mailBox);
            }

        }

        /// <summary>
        /// トップのDataset項目を追加、またはService配下にDataset項目を追加する
        /// </summary>
        /// <param name="treeItem"></param>
        /// <param name="tag"></param>
        public void AddDatasetElem(TreeItem treeItem, Dataset dataset)
        {
            int index;

            // Serviceを取得
            TreeItem Service = treeItem.SearchService();

            index = SearchEciIndex(treeItem.Filename);
            if (0 <= index)
            {
                if (Service == null) //トップのDatasetの追加処理
                    eciList[index].eciData.Datasets.Add(dataset);
                else//Service以下のDatasetの追加処理
                    eciList[index].AddServiceDataset(Service.Index, dataset);
            }
        }



        /// <summary>
        /// 管理下のECIファイルにServiceを追加する
        /// </summary>
        /// <param name="filename"></param>
        /// <param name="service"></param>
        public void AddService(string filename, Service service)
        {
            int index;

            index = eciList.FindIndex(eciFile => eciFile.eciFileName == filename);
            if (0 <= index)
            {
                eciList[index].AddService(service);
            }

        }

        /// <summary>
        /// ECI管理ファイル検索
        /// </summary>
        /// <param name="filename"></param>
        /// <returns>ECI管理ファイル</returns>
        public EciFileMgmt SearchEciFileMgmt(string filename)
        {
            EciFileMgmt eciFileMgmt = null;
            int index = -1;

            // ECI管理ファイル検索
            index = eciList.FindIndex(eciFile => eciFile.eciFileName == filename);
            if (0 <= index)
            {
                eciFileMgmt = this.eciList[index];
            }

            return eciFileMgmt;
        }

        /// <summary>
        /// ECI管理ファイル検索
        /// </summary>
        /// <param name="filename"></param>
        /// <returns>ECI管理リストのIndexNo</returns>
        public int SearchEciIndex(string filename)
        {
            int index = -1;

            // ECI管理ファイル検索
            index = eciList.FindIndex(eciFile => eciFile.eciFileName == filename);

            return index;
        }

        /// <summary>
        /// 管理しているECIファイル名にファイル名が存在するか
        /// </summary>
        /// <param name="name">ファイル名</param>
        /// <returns>true:存在する, false:存在しない</returns>
        public bool SearchListForSameName(string name)
        {
            bool isDone = false;
            int index;

            // ECI管理ファイル検索
            index = eciList.FindIndex(eciFile => System.IO.Path.GetFileName(eciFile.eciFileName) == name);
            if(!(index < 0))
            {
                isDone = true;
            }

            return isDone;
        }

        /// <summary>
        /// Datasetの取得
        /// </summary>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        public Dataset GetDataset(TreeItem treeItem)
        {
            Dataset dataset = new Dataset();
            EciFileMgmt eciFileMgmt;
            int index = treeItem.Index;

            switch (treeItem.ItemType)
            {
                case (int)ItemType.Dataset:
                    index = treeItem.Index;
                    break;
                case (int)ItemType.DatasetTagRefs:
                    index = treeItem.Parent.Index;
                    break;
                case (int)ItemType.DatasetTagRef:
                    index = treeItem.Parent.Parent.Index;
                    break;
                default:
                    break;
            }

            // Serviceを取得
            TreeItem Service = treeItem.SearchService();
            eciFileMgmt = SearchEciFileMgmt(treeItem.Filename);
            if(eciFileMgmt is EciFileMgmt)
            {
                if (Service == null) //トップのDatasetの追加処理
                {
                    if (index < eciFileMgmt.eciData.Datasets.Count)
                        dataset = eciFileMgmt.eciData.Datasets[index];
                }
                else
                {
                    if (eciFileMgmt.eciData.Services[Service.Index].Datasets != null && (index < eciFileMgmt.eciData.Services[Service.Index].Datasets.Count))
                        dataset = eciFileMgmt.eciData.Services[Service.Index].Datasets[index];
                }
            }

            return dataset;
        }

        /// <summary>
        /// Serviceの取得
        /// </summary>
        /// <param name="treeItem"></param>
        /// <returns></returns>
        public Service GetService(TreeItem treeItem)
        {
            Service service = new Service();
            EciFileMgmt eciFileMgmt;

            //////////////////////////////
            // ServiceのIndexを取得
            //////////////////////////////
            int index = treeItem.Index;
            switch (treeItem.ItemType)
            {
                case (int)ItemType.TagRefs:
                    index = treeItem.Parent.Index;
                    break;
                case (int)ItemType.TagRef:
                    index = treeItem.Parent.Parent.Index;   
                    break;
                case (int)ItemType.TagRefsIN:
                    index = treeItem.Parent.Index;
                    break;
                case (int)ItemType.TagRefIN:
                    index = treeItem.Parent.Parent.Index;
                    break;
                case (int)ItemType.TagRefsOUT:
                    index = treeItem.Parent.Index;
                    break;
                case (int)ItemType.TagRefOUT:
                    index = treeItem.Parent.Parent.Index;
                    break;
                case (int)ItemType.Collectors:
                    index = treeItem.Parent.Index;
                    break;
                case (int)ItemType.Collector:
                    index = treeItem.Parent.Parent.Index;
                    break;
                case (int)ItemType.CollectorRefs:
                    index = treeItem.Parent.Index;
                    break;
                case (int)ItemType.CollectorRef:
                    index = treeItem.Parent.Parent.Index;
                    break;
                case (int)ItemType.MailBoxes:
                    index = treeItem.Parent.Index;
                    break;
                case (int)ItemType.MailBox:
                    index = treeItem.Parent.Parent.Index;
                    break;
                default:
                    break;
            }

            eciFileMgmt = SearchEciFileMgmt(treeItem.Filename);
            if (eciFileMgmt is EciFileMgmt && index < eciFileMgmt.eciData.Services.Count)
            {
                service = eciFileMgmt.eciData.Services[index];
            }

            return service;
        }

        /// <summary>
        /// Tagsの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTags(TreeItem treeItem)
        {
            TreeItem Service;
            int index;
            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            index = SearchEciIndex(treeItem.Filename);
            if (0 <= index)
            {
                if(Service == null)//トップのタグ
                    eciList[index].RemoveTags();
                else//サービス以下のタグ
                    eciList[index].RemoveTags(Service.Index);

            }
        }

        /// <summary>
        /// Tagの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTag(TreeItem treeItem)
        {
            TreeItem Service;
            int index;
            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            index = SearchEciIndex(treeItem.Filename);
            if (0 <= index)
            {
                if (Service == null)//トップのタグ
                    eciList[index].RemoveTag(treeItem);
                else//サービス以下のタグ
                    eciList[index].RemoveTag(Service.Index, treeItem);
            }

        }

        /// <summary>
        /// Datasetsの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveDatasets(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;
            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            fileIndex = SearchEciIndex(treeItem.Filename);
            if(0 <= fileIndex)
            {
                if (Service == null)//トップのデータセット
                    eciList[fileIndex].RemoveDatasets();
                else//サービス以下のデータセット
                    eciList[fileIndex].RemoveDatasets(Service.Index);
            }

        }

        /// <summary>
        /// Datasetの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveDataset(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;
            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            fileIndex = SearchEciIndex(treeItem.Filename);
            if (0 <= fileIndex)
            {
                if (Service == null)//トップのデータセット
                    eciList[fileIndex].RemoveDataset(treeItem);
                else//サービス以下のデータセット
                    eciList[fileIndex].RemoveDataset(Service.Index, treeItem);
            }
        }

        /// <summary>
        /// DatasetTagRefsの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveDatasetTagRefs(TreeItem treeItem)
        {
            TreeItem Dataset;
            int fileIndex;
            TreeItem Service;
            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            Dataset = treeItem.SearchDataset();
            if(Dataset is TreeItem)
            {
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {                
                    if (Service == null)//トップのデータセット
                        eciList[fileIndex].RemoveDatasetTagRefs(Dataset);
                    else//サービス以下のデータセット
                        eciList[fileIndex].RemoveDataset(Service.Index, treeItem);
                }
            }
            
        }

        /// <summary>
        /// DatasetTagRefの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveDatasetTagRef(TreeItem treeItem)
        {
            TreeItem Dataset;
            int fileIndex;
            TreeItem Service;
            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            Dataset = treeItem.SearchDataset();
            if (Dataset is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if (0 <= fileIndex)
                {
                    

                    if (Service == null)//トップのデータセット
                        eciList[fileIndex].RemoveDatasetTagRef(Dataset.Index, treeItem);
                    else//サービス以下のデータセット
                        eciList[fileIndex].RemoveDatasetTagRef(Service.Index, Dataset.Index, treeItem);
                }
            }

        }

        /// <summary>
        /// TagRefsの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTagRefs(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveTagRefs(Service.Index);
                }
            }
        }

        /// <summary>
        /// TagRefの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTagRef(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveTagRef(Service.Index, treeItem);
                }
            }

        }

        /// <summary>
        /// TagRefsINの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTagRefsIN(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveTagRefsIN(Service.Index);
                }
            }

        }

        /// <summary>
        /// TagRefINの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTagRefIN(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if(Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveTagRefIN(Service.Index, treeItem);
                }
            }

        }

        /// <summary>
        /// TagRefsOUTの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTagRefsOUT(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveTagRefsOUT(Service.Index);
                }
            }
        }

        /// <summary>
        /// TagRefOUTの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveTagRefOUT(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveTagRefOUT(Service.Index, treeItem);
                }
            }
        }

        /// <summary>
        /// Collectorsの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveCollectors(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveCollectors(Service.Index);
                }
            }
        }

        /// <summary>
        /// Collectorの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveCollector(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if(0<= fileIndex)
                {
                    eciList[fileIndex].RemoveCollector(Service.Index, treeItem);
                }
            }
        }

        /// <summary>
        /// CollectorRefsの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveCollectorRefs(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if (0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveCollectorRefs(Service.Index);
                }
            }
        }

        /// <summary>
        /// CollectorRefの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveCollectorRef(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if (0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveCollectorRef(Service.Index, treeItem);
                }
            }

        }

        /// <summary>
        /// Mailboxesの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveMailboxes(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if (0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveMailboxes(Service.Index);
                }
            }
        }

        /// <summary>
        /// Mailboxの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveMailbox(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if (0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveMailbox(Service.Index, treeItem);

                }
            }
        }

        /// <summary>
        /// Servicesの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveServices(TreeItem treeItem)
        {
            int fileIndex;

            // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
            fileIndex = SearchEciIndex(treeItem.Filename);
            if (0 <= fileIndex)
            {
                eciList[fileIndex].RemoveServices();

            }
        }

        /// <summary>
        /// Serviceの削除
        /// </summary>
        /// <param name="treeItem"></param>
        public void RemoveService(TreeItem treeItem)
        {
            TreeItem Service;
            int fileIndex;

            // ServiceリストのIndexNoを知るため、ServiceのTreeItemを検索
            Service = treeItem.SearchService();
            if (Service is TreeItem)
            {
                // TreeItemのファイル名情報からECIファイル管理リストのIndexを取得
                fileIndex = SearchEciIndex(treeItem.Filename);
                if (0 <= fileIndex)
                {
                    eciList[fileIndex].RemoveService(treeItem.Index);

                }
            }
        }
    }
}
