﻿////////////////////////////////////////////////////////////////////////////////////////
/// 機能       :   ECIの要素・ファイル単体・プロジェクト全体の整合性確認機能
/// 作成日     :   2022/02/08
/// 作成者     :   mizuno
/// 
/// RT-edge XML編集ツール 機能仕様書
/// file://192.168.1.240/Jobs/KA1601(RT-edge開発）/trunk/0051_Documnet(社内)/0200_egCoreService/20211021_RT-edge_XML編集ツール_機能仕様書.pptx
/// P22. 機能詳細 エラーチェック
/// P36. プログラム仕様 エラーチェック処理① 各要素の単体チェック
/// P37. プログラム仕様 エラーチェック処理② 要素間の整合性チェック


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RTedgeXmlObj;
using System.Collections.ObjectModel;
using System.Text.RegularExpressions;

namespace EgConfig
{
    /// <summary>
    /// 要素の整合性確認クラス
    /// </summary>
    public class CheckConsistencyOfElem
    {
        #region パブリックメソッド

        /// <summary>
        /// プロジェクトの整合性チェック処理(TreeItemから検索)
        /// </summary>
        /// <param name="treeItems">検索するTreeItem</param>
        /// <param name="eciProjMgmt">プロジェクト管理ファイル</param>
        /// <returns>チェック結果</returns>
        public static ObservableCollection<ElemCheckMsg> CheckProjConsistency(ObservableCollection<TreeItem> treeItems, EciProjMgmt eciProjMgmt)
        {
            // チェック結果返却用
            ObservableCollection<ElemCheckMsg> checkResult = new ObservableCollection<ElemCheckMsg>();
            List<ElemCheckMsg> msgs = new List<ElemCheckMsg>();
            int errorCode = Define.ErrorCode.NONE;
            string filename = "", servicename = "";

            // ツリーアイテムを上から順に検索
            foreach (TreeItem treeItem in treeItems)
            {
                filename = treeItem.Name;
                servicename = filename.Replace(".xml", "");

                msgs.AddRange(CheckTreeItem(treeItem, eciProjMgmt, filename, servicename));
            }

            // エラーがない
            if(msgs.Count == 0)
            {
                msgs.Add(new ElemCheckMsg(ErrorKind.Info.ToString(), 0x0000, "", "", "", "[エラーチェック]の結果、エラーはありませんでした。", new TreeItem()));
            }

            // ListをObservableCollectionに変換する
            foreach (ElemCheckMsg m in msgs)
            {
                checkResult.Add(m);
            }

            return checkResult;
        }

        /// <summary>
        /// 現在のTreeItemから子孫要素まで要素間のチェックを行う
        /// </summary>
        /// <param name="treeItem">検索対象のTreeItem</param>
        /// <param name="eciProjMgmt"></param>
        /// <param name="filename">メッセージ作成用ファイル名</param>
        /// <param name="servicename">メッセージ作成用サービス名</param>
        /// <returns>チェック結果</returns>
        private static List<ElemCheckMsg> CheckTreeItem(TreeItem treeItem, EciProjMgmt eciProjMgmt, string filename, string servicename)
        {
            List<ElemCheckMsg> msgs = new List<ElemCheckMsg>();
            ElemCheckMsg msg;
            int errorCode = Define.ErrorCode.NONE;
            string elemname = "", errmsg = "", errkind = "";

            foreach (TreeItem item in treeItem.Children)
            {
                // 条件
                switch (item.ItemType)
                {
                    case (int)ItemType.Tag:
                        if(eciProjMgmt.GetItemData(item) is Tag tag)
                        {
                            msgs.AddRange(CheckTagData(tag, filename, servicename, item));
                        }
                        break;
                    case (int)ItemType.Dataset:
                        if (eciProjMgmt.GetItemData(item) is Dataset dataset)
                        {
                            elemname = "Dataset";
                            /////////////////////////////////////////////////
                            // Dataset 重複チェック
                            /////////////////////////////////////////////////
                            errorCode = CheckSameDatasetName(dataset, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, dataset.Name);
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Dataset Nameチェック
                            ////////////////////////////////////////////
                            errorCode = DatasetName(dataset.Name);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Dataset", "Name");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                        }
                        break;
                    case (int)ItemType.DatasetTagRef:
                        if (eciProjMgmt.GetItemData(item) is TagRef datasetTagref)
                        {
                            elemname = "TagRef";

                            // Tagに同名があるかチェック
                            errorCode = CheckDatasetTagRefInTags(datasetTagref, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, datasetTagref.Name);
                                }

                                ////////////////////////////////////////////////
                                // リストに追加
                                ////////////////////////////////////////////////
                                if (eciProjMgmt.eciProjName == null)
                                {
                                    errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                }
                                else
                                {
                                    errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                }

                                msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                            
                        }
                        break;
                    case (int)ItemType.Service:
                        if (eciProjMgmt.GetItemData(item) is Service service)
                        {
                            elemname = "Service";

                            ///////////////////////////////////////////////////////////////////////////////
                            // "ServiceのDependent"が"Services内のName"にどこにも存在しないかチェック
                            ///////////////////////////////////////////////////////////////////////////////
                            errorCode = CheckSrvdepInServices(service, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, service.DEPENDENT);
                                }

                                ////////////////////////////////////////////////
                                // リストに追加
                                ////////////////////////////////////////////////
                                if (eciProjMgmt.eciProjName == null)
                                {
                                    errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                }
                                else
                                {
                                    errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                }

                                msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Service Nameチェック
                            ////////////////////////////////////////////
                            errorCode = ServiceName(service.Name);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Service", "Name");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Service Pathチェック
                            ////////////////////////////////////////////
                            errorCode = ServicePath(service.Path);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Service", "Path");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Service Priorityチェック
                            ////////////////////////////////////////////
                            errorCode = ServicePriority(service.Priority);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    if (errorCode == Define.ErrorCode.NUM_OUT_OF_RANGE)
                                    {
                                        errmsg = String.Format(errmsg, "Service", "Priority", "0～255");
                                    }
                                    else
                                    {
                                        errmsg = String.Format(errmsg, "Service", "Priority");
                                    }
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Service Timeoutチェック
                            ////////////////////////////////////////////
                            errorCode = ServiceTimeout(service.Timeout);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Service", "Timeout");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Service Nodeチェック
                            ////////////////////////////////////////////
                            errorCode = ServiceNode(service.Node);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Service", "Node");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Service Dependentチェック
                            ////////////////////////////////////////////
                            int index, i=0;
                            foreach (EciFileMgmt eciFileMgmt in eciProjMgmt.eciList)
                            {
                                index = eciFileMgmt.eciFileName.IndexOfAny(System.IO.Path.GetInvalidPathChars());           // ファイルパスに使用できない文字が含まれていないかチェック
                                if (index<0 && System.IO.Path.GetFileName(eciFileMgmt.eciFileName) == "EgBoot.xml")         // 設定したDependentがEgBoot.xmlのServiceに定義されているかチェック
                                {
                                    errorCode = CheckServiceDependent(service, eciProjMgmt.eciList[i]);
                                    if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                                    {
                                        if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                        {
                                            errmsg = Define.ErrorMsg[errorCode];
                                        }

                                        ////////////////////////////////////////////////
                                        // リストに追加
                                        ////////////////////////////////////////////////
                                        if (eciProjMgmt.eciProjName == null)
                                        {
                                            errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                        }
                                        else
                                        {
                                            errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                        }

                                        msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                        msgs.Add(msg);
                                    }
                                    break;
                                }

                                i++;
                            }
                            ////////////////////////////////////////////
                            // ECI Pathチェック
                            ////////////////////////////////////////////
                            errorCode = ServicePath(service.ECI);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Service", "ECI");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                        }
                        break;
                    case (int)ItemType.TagRef:
                        if (eciProjMgmt.GetItemData(item) is TagRef tagRef)
                        {
                            elemname = "TagRef";
                            
                            // Tagに同名があるかチェック
                            errorCode = CheckTagRefInTags(tagRef, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, tagRef.Name);
                                }

                                ////////////////////////////////////////////////
                                // リストに追加
                                ////////////////////////////////////////////////
                                if (eciProjMgmt.eciProjName == null)
                                {
                                    errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                }
                                else
                                {
                                    errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                }

                                msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                            
                        }
                        break;
                    case (int)ItemType.TagRefIN:
                        if (eciProjMgmt.GetItemData(item) is TagRef tagRefIN)
                        {
                            elemname = "TagRefIN";
                            
                            // Tagに同名があるかチェック
                            errorCode = CheckTagRefINInTags(tagRefIN, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, tagRefIN.Name);
                                }

                                ////////////////////////////////////////////////
                                // リストに追加
                                ////////////////////////////////////////////////
                                if (eciProjMgmt.eciProjName == null)
                                {
                                    errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                }
                                else
                                {
                                    errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                }

                                msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                        }
                        break;
                    case (int)ItemType.TagRefOUT:
                        if (eciProjMgmt.GetItemData(item) is TagRef tagRefOUT)
                        {
                            elemname = "TagRefOUT";
                            // Tagに同名があるかチェック
                            errorCode = CheckTagRefOUTInTags(tagRefOUT, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, tagRefOUT.Name);
                                }

                                ////////////////////////////////////////////////
                                // リストに追加
                                ////////////////////////////////////////////////
                                if (eciProjMgmt.eciProjName == null)
                                {
                                    errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                }
                                else
                                {
                                    errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                }

                                msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                        }
                        break;
                    case (int)ItemType.Collector:
                        if (eciProjMgmt.GetItemData(item) is Collector collector)
                        {
                            elemname = "Collector";
                            
                            /////////////////////////////////////////////////
                            // Collector 重複チェック
                            /////////////////////////////////////////////////
                            errorCode = CheckSameCollectorName(collector, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, collector.Name);
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ///////////////////////////////////////////////////////////////////////////////
                            // "CollectorのDatasetName"が"DatasetのName"にどこにも存在しないかチェック
                            ///////////////////////////////////////////////////////////////////////////////
                            errorCode = CheckCollectorInDataset(collector, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, collector.DatasetName);
                                }

                                ////////////////////////////////////////////////
                                // リストに追加
                                ////////////////////////////////////////////////
                                if (eciProjMgmt.eciProjName == null)
                                {
                                    errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                }
                                else
                                {
                                    errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                }

                                msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Collector Nameチェック
                            ////////////////////////////////////////////
                            errorCode = CollectorName(collector.Name);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Collector", "Name");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Collector Intervalチェック
                            ////////////////////////////////////////////
                            errorCode = CollectorInterval(collector.Interval);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Collector", "Interval");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Collector Priorityチェック
                            ////////////////////////////////////////////
                            errorCode = CollectorPriority(collector.Priority);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    if (errorCode == Define.ErrorCode.NUM_OUT_OF_RANGE)
                                    {
                                        errmsg = String.Format(errmsg, "Collector", "Priority", "0～255");
                                    }
                                    else
                                    {
                                        errmsg = String.Format(errmsg, "Collector", "Priority");
                                    }
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Collector DatasetNameチェック
                            ////////////////////////////////////////////
                            errorCode = CollectorDatasetName(collector.DatasetName);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Collector", "DatasetName");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Collector ColSizeチェック
                            ////////////////////////////////////////////
                            errorCode = CollectorColsize(collector.ColSize);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Collector", "ColSize");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // Collector RowSizeチェック
                            ////////////////////////////////////////////
                            errorCode = CollectorRowsize(collector.RowSize);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "Collector", "RowSize");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                        }
                        break;
                    case (int)ItemType.CollectorRef:
                        if (eciProjMgmt.GetItemData(item) is CollectorRef collectorRef)
                        {
                            elemname = "CollectorRef";
                            
                            // Collectorに同名があるかチェック
                            errorCode = CheckCollectorRefInCollectors(collectorRef, eciProjMgmt.eciList);
                            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, collectorRef.Name);
                                }

                                ////////////////////////////////////////////////
                                // リストに追加
                                ////////////////////////////////////////////////
                                if (eciProjMgmt.eciProjName == null)
                                {
                                    errkind = ErrorKind.Info.ToString();        // ECIファイル単体読み込み時の種別はINFO
                                }
                                else
                                {
                                    errkind = ErrorKind.Error.ToString();       // プロジェクトファイル読み込み時の種別はError
                                }

                                msg = new ElemCheckMsg(errkind, errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                            
                        }
                        break;
                    case (int)ItemType.MailBox:
                        if (eciProjMgmt.GetItemData(item) is MailBox mailBox)
                        {
                            elemname = "MailBox";
                            
                            ////////////////////////////////////////////
                            // MailBox Nameチェック
                            ////////////////////////////////////////////
                            errorCode = MailboxName(mailBox.Name);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "MailBox", "Name");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // MailBox ColSizeチェック
                            ////////////////////////////////////////////
                            errorCode = MailboxColsize(mailBox.ColSize);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "MailBox", "ColSize");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }

                            ////////////////////////////////////////////
                            // MailBox RowSizeチェック
                            ////////////////////////////////////////////
                            errorCode = MailboxRowsize(mailBox.RowSize);
                            if (errorCode != Define.ErrorCode.NONE)                     // エラーチェック条件に合致した際、エラーメッセージを作成する
                            {
                                if (Define.ErrorMsg.ContainsKey(errorCode) == true)     // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                                {
                                    errmsg = Define.ErrorMsg[errorCode];
                                    errmsg = String.Format(errmsg, "MailBox", "RowSize");
                                }

                                // リストに追加
                                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, item);
                                msgs.Add(msg);
                            }
                            
                        }
                        break;
                    default:
                        break;
                }
                msgs.AddRange(CheckTreeItem(item, eciProjMgmt, filename, servicename));
            }

            return msgs;
        }


        private static List<ElemCheckMsg> CheckTagData(Tag tag, string filename, string servicename, TreeItem treeItem)
        {
            List<ElemCheckMsg> msgs = new List<ElemCheckMsg>();
            ElemCheckMsg msg;
            int errorCode = Define.ErrorCode.NONE;
            string elemname = "", errmsg = "";

            elemname = "Tag";
            ////////////////////////////////////////////
            // Tag Nameチェック
            ////////////////////////////////////////////
            errorCode = TagName(tag.Name);
            if (errorCode != Define.ErrorCode.NONE)                  // エラーチェック条件に合致した際、エラーメッセージを作成する
            {
                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                {
                    errmsg = Define.ErrorMsg[errorCode];
                    errmsg = String.Format(errmsg, elemname, "Name");
                }

                // リストに追加
                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, treeItem);
                msgs.Add(msg);
            }

            ////////////////////////////////////////////
            // Tag Addressチェック
            ////////////////////////////////////////////
            errorCode = TagAddress(tag.Address);
            if (errorCode != Define.ErrorCode.NONE)                  // エラーチェック条件に合致した際、エラーメッセージを作成する
            {
                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                {
                    errmsg = Define.ErrorMsg[errorCode];
                    errmsg = String.Format(errmsg, elemname, "Address");
                }

                // リストに追加
                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, treeItem);
                msgs.Add(msg);
            }

            ////////////////////////////////////////////
            // Tag Typeチェック
            ////////////////////////////////////////////
            errorCode = TagType(tag.Type);
            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
            {
                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                {
                    errmsg = Define.ErrorMsg[errorCode];
                    if (errorCode == Define.ErrorCode.NUM_OUT_OF_RANGE)
                    {
                        errmsg = String.Format(errmsg, elemname, "Type", "1～11");
                    }
                    else
                    {
                        errmsg = String.Format(errmsg, elemname, "Type");
                    }
                }

                // リストに追加
                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, treeItem);
                msgs.Add(msg);
            }

            ////////////////////////////////////////////
            // Tag Valueチェック
            ////////////////////////////////////////////
            errorCode = TagValue(tag.Type, tag.Value, out string range);
            if (errorCode != Define.ErrorCode.NONE)                 // エラーチェック条件に合致した際、エラーメッセージを作成する
            {
                if (Define.ErrorMsg.ContainsKey(errorCode) == true) // エラーコードに対応するひな形メッセージを取得し、内容を設定する
                {
                    errmsg = Define.ErrorMsg[errorCode];
                    if (errorCode == Define.ErrorCode.NUM_OUT_OF_RANGE)
                    {
                        errmsg = String.Format(errmsg, elemname, "Value", range);
                    }
                }

                // リストに追加
                msg = new ElemCheckMsg(ErrorKind.Error.ToString(), errorCode, filename, servicename, elemname, errmsg, treeItem);
                msgs.Add(msg);
            }

            return msgs;
        }

        

        

        #region Tag要素のエラーチェック

        /// <summary>
        /// Tag要素Nameプロパティエラーチェック
        /// </summary>
        /// <param name="name">Tag要素Nameプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int TagName(string name)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isWS, isHWAC, isOver;              // 条件判定用

            ///////////////////////////////////////////
            // 空白文字列の場合エラー
            ///////////////////////////////////////////
            isWS = IsStrWhiteSpace(name);
            if (isWS == true)
            {
                return Define.ErrorCode.NULL;
            }

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(name);
            if(isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 48Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(name, 48);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        /// <summary>
        /// Tag要素Addressプロパティエラーチェック
        /// </summary>
        /// <param name="address">Tag要素Addressプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int TagAddress(string address)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isHWAC, isOver;                    // 条件判定用

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(address);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 48Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(address, 48);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        /// <summary>
        /// Tag要素Typeプロパティエラーチェック
        /// </summary>
        /// <param name="type">Tag要素Typeプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int TagType(string type)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isNum, isWithinRange;              // 条件判定用

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(type);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            ///////////////////////////////////////////
            // 1~11以外の場合エラー
            ///////////////////////////////////////////
            isWithinRange = IsStrWithinRange(type, 1, 11);
            if (isWithinRange == false)
            {
                return Define.ErrorCode.NUM_OUT_OF_RANGE;
            }

            return errorCode;
        }
        
        /// <summary>
        /// Tag要素Valueプロパティエラーチェック
        /// </summary>
        /// <param name="type">Tag要素のType属性</param>
        /// <param name="value">Tag要素のValue属性</param>
        /// <param name="range">Valueに入力可能な値範囲</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int TagValue(string type, string value, out string range)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            string min = "", max = "";
            range = "";
            
            // Valueが空白の場合、以降のチェック処理は行わない
            if(String.IsNullOrEmpty(value) == true)
            {
                return errorCode;
            }

            // Type型別にValueのエラーチェックを行う
            if(type == ((int)Define.PropertyWindow.TagType.Boolean).ToString())
            {
                min = "0";
                max = "1";
                range = min + "～" + max + ", True, False";
                if(!(Regex.IsMatch(value, "^[0]$|^[1]$|^[tT][rR][uU][eE]$|^[fF][aA][lL][sS][eE]$")))
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if(type == ((int)Define.PropertyWindow.TagType.SByte).ToString())
            {
                min = SByte.MinValue.ToString();
                max = SByte.MaxValue.ToString();
                range = min + "～" + max;
                if (SByte.TryParse(value, out sbyte res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.Byte).ToString())
            {
                min = Byte.MinValue.ToString();
                max = Byte.MaxValue.ToString();
                range = min + "～" + max;
                if (Byte.TryParse(value, out byte res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.Int16).ToString())
            {
                min = Int16.MinValue.ToString();
                max = Int16.MaxValue.ToString();
                range = min + "～" + max;
                if (Int16.TryParse(value, out short res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.UInt16).ToString())
            {
                min = UInt16.MinValue.ToString();
                max = UInt16.MaxValue.ToString();
                range = min + "～" + max;
                if (UInt16.TryParse(value, out ushort res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.Int32).ToString())
            {
                min = Int32.MinValue.ToString();
                max = Int32.MaxValue.ToString();
                range = min + "～" + max;
                if (Int32.TryParse(value, out int res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.UInt32).ToString())
            {
                min = UInt32.MinValue.ToString();
                max = UInt32.MaxValue.ToString();
                range = min + "～" + max;
                if (UInt32.TryParse(value, out uint res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.Int64).ToString())
            {
                min = Int64.MinValue.ToString();
                max = Int64.MaxValue.ToString();
                range = min + "～" + max;
                if (Int64.TryParse(value, out long res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.UInt64).ToString())
            {
                min = UInt64.MinValue.ToString();
                max = UInt64.MaxValue.ToString();
                range = min + "～" + max;
                if (UInt64.TryParse(value, out ulong res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.Float).ToString())
            {
                min = float.MinValue.ToString();
                max = float.MaxValue.ToString();
                range = min + "～" + max;
                if (float.TryParse(value, out float res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }
            else if (type == ((int)Define.PropertyWindow.TagType.Double).ToString())
            {
                min = double.MinValue.ToString();
                max = double.MaxValue.ToString();
                range = min + "～" + max;
                if (double.TryParse(value, out double res) == false)
                {
                    errorCode = Define.ErrorCode.NUM_OUT_OF_RANGE;
                }
            }

            return errorCode;
        }

        #endregion

        #region Dataset要素のエラーチェック
        /// <summary>
        /// Dataset要素Nameプロパティエラーチェック
        /// </summary>
        /// <param name="name">Dataset要素Nameプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int DatasetName(string name)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isWS, isHWAC, isOver;              // 条件判定用

            ///////////////////////////////////////////
            // 空白文字列の場合エラー
            ///////////////////////////////////////////
            isWS = IsStrWhiteSpace(name);
            if (isWS == true)
            {
                return Define.ErrorCode.NULL;
            }

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(name);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 32Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(name, 32);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }
        #endregion

        #region Service要素のエラーチェック
        /// <summary>
        /// Service要素Nameプロパティエラーチェック
        /// </summary>
        /// <param name="name">Service要素Nameプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int ServiceName(string name)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isWS, isHWAC, isOver;              // 条件判定用

            ///////////////////////////////////////////
            // 空白文字列の場合エラー
            ///////////////////////////////////////////
            isWS = IsStrWhiteSpace(name);
            if (isWS == true)
            {
                return Define.ErrorCode.NULL;
            }

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(name);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // サービス名最大長 Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(name, Define.DEF_EG_MAX_SERVICENAME);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        /// <summary>
        /// Service要素Pathプロパティ/ECIプロパティエラーチェック
        /// </summary>
        /// <param name="path">Service要素Pathプロパティ/ECIプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int ServicePath(string path)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isOver;                            // 条件判定用

            ///////////////////////////////////////////
            // 260Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(path, 260);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        /// <summary>
        /// Service要素Priorityプロパティエラーチェック
        /// </summary>
        /// <param name="priority">Service要素Priorityプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int ServicePriority(string priority)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty,isNum, isWR;               // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(priority);
            if(isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(priority);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            ///////////////////////////////////////////
            // 0~255以外の場合エラー
            ///////////////////////////////////////////
            isWR = IsStrWithinRange(priority, 0, 255);
            if (isWR == false)
            {
                return Define.ErrorCode.NUM_OUT_OF_RANGE;
            }

            return errorCode;
        }

        /// <summary>
        /// Service要素Timeoutプロパティエラーチェック
        /// </summary>
        /// <param name="timeout">Service要素Timeoutプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int ServiceTimeout(string timeout)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty, isNum;                    // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(timeout);
            if (isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(timeout);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            return errorCode;
        }

        /// <summary>
        /// Service要素Nodeプロパティエラーチェック
        /// </summary>
        /// <param name="node">Service要素Nodeプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int ServiceNode(string node)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isHWAC, isOver;                    // 条件判定用

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(node);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 64Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(node, 64);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }
        #endregion

        #region Collector要素のエラーチェック
        /// <summary>
        /// Collector要素Nameプロパティエラーチェック
        /// </summary>
        /// <param name="name">Collector要素Nameプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int CollectorName(string name)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isWS, isHWAC, isOver;              // 条件判定用

            ///////////////////////////////////////////
            // 空白文字列の場合エラー
            ///////////////////////////////////////////
            isWS = IsStrWhiteSpace(name);
            if (isWS == true)
            {
                return Define.ErrorCode.NULL;
            }

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(name);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 8Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(name, 8);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        /// <summary>
        /// Collector要素Intervalプロパティエラーチェック
        /// </summary>
        /// <param name="interval">Collector要素Intervalプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int CollectorInterval(string interval)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty, isNum;                    // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(interval);
            if (isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(interval);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            return errorCode;
        }

        /// <summary>
        /// Collector要素Priorityプロパティエラーチェック
        /// </summary>
        /// <param name="priority">Collector要素Priorityプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int CollectorPriority(string priority)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty, isNum, isWR;              // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(priority);
            if (isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(priority);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            ///////////////////////////////////////////
            // 0~255以外の場合エラー
            ///////////////////////////////////////////
            isWR = IsStrWithinRange(priority, 0, 255);
            if (isWR == false)
            {
                return Define.ErrorCode.NUM_OUT_OF_RANGE;
            }

            return errorCode;
        }

        /// <summary>
        /// Collector要素DatasetNameプロパティエラーチェック
        /// </summary>
        /// <param name="datasetName">Collector要素DatasetNameプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int CollectorDatasetName(string datasetName)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isWS, isHWAC, isOver;              // 条件判定用

            ///////////////////////////////////////////
            // 空白文字列の場合エラー
            ///////////////////////////////////////////
            isWS = IsStrWhiteSpace(datasetName);
            if (isWS == true)
            {
                return Define.ErrorCode.NULL;
            }

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(datasetName);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 32Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(datasetName, 32);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        /// <summary>
        /// Collector要素ColSizeプロパティエラーチェック
        /// </summary>
        /// <param name="colSize">Collector要素ColSizeプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int CollectorColsize(string colSize)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty, isNum;                    // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(colSize);
            if (isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(colSize);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            return errorCode;
        }

        /// <summary>
        /// Collector要素RowSizeプロパティエラーチェック
        /// </summary>
        /// <param name="rowSize">Collector要素RowSizeプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int CollectorRowsize(string rowSize)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty, isNum;                    // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(rowSize);
            if (isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(rowSize);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            return errorCode;
        }
        #endregion

        #region Mailbox要素のエラーチェック
        /// <summary>
        /// Mailbox要素Nameプロパティエラーチェック
        /// </summary>
        /// <param name="name">Mailbox要素Nameプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int MailboxName(string name)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isWS, isHWAC, isOver;              // 条件判定用

            ///////////////////////////////////////////
            // 空白文字列の場合エラー
            ///////////////////////////////////////////
            isWS = IsStrWhiteSpace(name);
            if (isWS == true)
            {
                return Define.ErrorCode.NULL;
            }

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(name);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 8Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(name, 8);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        /// <summary>
        /// Mailbox要素ColSizeプロパティエラーチェック
        /// </summary>
        /// <param name="colSize">Mailbox要素ColSizeプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int MailboxColsize(string colSize)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty, isNum;                    // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(colSize);
            if (isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(colSize);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            return errorCode;
        }

        /// <summary>
        /// Mailbox要素RowSizeプロパティエラーチェック
        /// </summary>
        /// <param name="rowSize">Mailbox要素RowSizeプロパティ</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int MailboxRowsize(string rowSize)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isEmpty, isNum;                    // 条件判定用

            ///////////////////////////////////////////
            // 空文字はエラーにしない
            ///////////////////////////////////////////
            isEmpty = String.IsNullOrEmpty(rowSize);
            if (isEmpty == true)
            {
                return Define.ErrorCode.NONE;
            }

            ///////////////////////////////////////////
            // 数字以外の場合エラー
            ///////////////////////////////////////////
            isNum = IsStrNum(rowSize);
            if (isNum == false)
            {
                return Define.ErrorCode.EXCEPT_NUM;
            }

            return errorCode;
        }
        #endregion

        #region 要素間の整合性チェック(ECIファイル単体のチェック時)
        /// <summary>
        /// TagRefのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="tags">Tagリスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckTagRefInTags(TagRef tagRef, List<Tag> tags)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isInTags;

            isInTags = IsTagRefInTags(tagRef, tags);
            if (isInTags == false)
            {
                errorCode = Define.ErrorCode.NOT_EXIST_TAGREF_IN_TAGS;
            }

            return errorCode;
        }

        /// <summary>
        /// TagRefINのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="tags">Tagリスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckTagRefINInTags(TagRef tagRef, List<Tag> tags)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isInTags;

            isInTags = IsTagRefInTags(tagRef, tags);
            if (isInTags == false)
            {
                errorCode = Define.ErrorCode.NOT_EXIST_TAGREFIN_IN_TAGS;
            }

            return errorCode;
        }

        /// <summary>
        /// TagRefOUTのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="tags">Tagリスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckTagRefOUTInTags(TagRef tagRef, List<Tag> tags)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isInTags;

            isInTags = IsTagRefInTags(tagRef, tags);
            if (isInTags == false)
            {
                errorCode = Define.ErrorCode.NOT_EXIST_TAGREFOUT_IN_TAGS;
            }

            return errorCode;
        }

        /// <summary>
        /// DatasetTagRefのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="tags">Tagリスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckDatasetTagRefInTags(TagRef tagRef, List<Tag> tags)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isInTags;

            isInTags = IsTagRefInTags(tagRef, tags);
            if (isInTags == false)
            {
                errorCode = Define.ErrorCode.NOT_EXIST_DSETTAGREF_IN_TAGS;
            }

            return errorCode;
        }


        /// <summary>
        /// Datasetsの中に重複したDatasetのNameがないかチェック
        /// </summary>
        /// <param name="dataset">Dataset</param>
        /// <param name="datasets">検索対象のDatasets</param>
        /// <returns>エラーコード:0x0000)重複なし, それ以外)重複あり</returns>
        public static int CheckSameDatasetName(Dataset dataset, List<Dataset> datasets)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isDup = false;

            foreach(Dataset data in datasets)
            {
                if((dataset != data) && (dataset.Name == data.Name))
                {
                    isDup = true;
                    break;
                }
            }

            if (isDup == true)
            {
                errorCode = Define.ErrorCode.EXIST_SAME_NAMES_IN_DSET;
            }

            return errorCode;
        }

        /// <summary>
        /// Collectorsの中に重複したCollectorのNameがないかチェック
        /// </summary>
        /// <param name="collector">Collector</param>
        /// <param name="collectors">検索対象のCollector</param>
        /// <returns>エラーコード:0x0000)重複なし, それ以外)重複あり</returns>
        public static int CheckSameCollectorName(Collector collector, List<Collector> collectors)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isDup = false;

            foreach (Collector data in collectors)
            {
                if ((collector != data) && (collector.Name == data.Name))
                {
                    isDup = true;
                    break;
                }
            }

            if (isDup == true)
            {
                errorCode = Define.ErrorCode.EXIST_SAME_NAMES_IN_COLL;
            }

            return errorCode;
        }

        /// <summary>
        /// CollectorのDatasetNameがDatasetsの中に含まれているかチェック
        /// </summary>
        /// <param name="collector">Collector</param>
        /// <param name="datasets">検索対象のDatasets</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckCollectorInDataset(Collector collector, List<Dataset> datasets)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isInDsets = false;

            foreach(Dataset dataset in datasets)
            {
                if(collector.DatasetName == dataset.Name)
                {
                    isInDsets = true;
                    break;
                }
            }

            if(isInDsets == false)
            {
                errorCode = Define.ErrorCode.NOT_EXIST_COLLDSET_IN_DSETS;
            }

            return errorCode;
        }

        /// <summary>
        /// CollectorRefのNameがCollectorsの中に含まれているかチェック
        /// </summary>
        /// <param name="collectorRef">CollectorRef</param>
        /// <param name="collectors">検索対象のCollectors</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckCollectorRefInCollectors(CollectorRef collectorRef, List<Collector> collectors)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isInColls = false;

            foreach (Collector collector in collectors)
            {
                if (collectorRef.Name == collector.Name)
                {
                    isInColls = true;
                    break;
                }
            }

            if (isInColls == false)
            {
                errorCode = Define.ErrorCode.NOT_EXIST_COLLREF_IN_COLLREFS;
            }

            return errorCode;
        }

        /// <summary>
        /// Serviceの依存関係がServicesの中に含まれているかチェック
        /// </summary>
        /// <param name="service">Service</param>
        /// <param name="services">検索対象のServices</param>
        /// <returns>エラーコード:0x0000)含まれているor依存関係が空文字, それ以外)含まれていない</returns>
        public static int CheckSrvdepInServices(Service service, List<Service> services)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isInSrvs = false;

            ////////////////////////////////////////////////////////////////////
            // 依存関係の設定がNull・空文字の場合は以降のチェックを行わない
            ////////////////////////////////////////////////////////////////////
            if (String.IsNullOrEmpty(service.Dependent) == true)
            {
                return errorCode;
            }

            foreach (Service sv in services)
            {
                if (service.Dependent == sv.Name)
                {
                    isInSrvs = true;
                    break;
                }
            }

            if (isInSrvs == false)
            {
                errorCode = Define.ErrorCode.NOT_EXIST_DEP_IN_SRVS;
            }

            return errorCode;
        }

        /// <summary>
        /// Servicesの中に重複したServiceのNameがないかチェック
        /// </summary>
        /// <param name="service">Service</param>
        /// <param name="services">検索対象のService</param>
        /// <returns>エラーコード:0x0000)重複なし, それ以外)重複あり</returns>
        public static int CheckSameServiceName(Service service, List<Service> services)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isDup = false;

            foreach (Service data in services)
            {
                if ((service != data) && (service.Name == data.Name))
                {
                    isDup = true;
                    break;
                }
            }

            if (isDup == true)
            {
                errorCode = Define.ErrorCode.EXIST_SAME_NAMES_IN_SRV;
            }

            return errorCode;
        }


        /// <summary>
        /// Serviceの起動順番が成立しない依存関係かチェック
        /// </summary>
        /// <param name="service">Service</param>
        /// <param name="eciFile">EgBootのECI管理</param>
        /// <returns>エラーコード:0x0000)依存関係が成立する, それ以外)成立しない依存関係がある</returns>
        public static int CheckServiceDependent(Service service, EciFileMgmt eciFile)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isOK = false;

            ///////////////////////////////////////////////////////////////////////////
            // Serviceのdependentが、プロジェクトで管理しているEgBootの
            // Serviceの中に含まれているかチェック
            ///////////////////////////////////////////////////////////////////////////
            if (String.IsNullOrEmpty(service.Dependent) == true)    // 依存関係が設定されていない場合、エラーとしない
            {
                return errorCode;
            }

            foreach (Service sv in eciFile.eciData.Services)
            {
                if (service.Dependent == sv.Name)
                {
                    isOK = true;
                    break;
                }
            }

            if (isOK == false)
            {
                errorCode = Define.ErrorCode.NOT_SATISFIED_SRVDEP;
            }

            return errorCode;
        }
        #endregion

        #region 要素間の整合性チェック(プロジェクト全体でのチェック時)
        /// <summary>
        /// TagRefのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckTagRefInTags(TagRef tagRef, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // TagRefのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Tagsの中に含まれているかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                //トップのTagsを検索
                errorCode = CheckTagRefInTags(tagRef, eciFile.eciData.Tags);
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
                //サービス以下のTagsを検索
                foreach (Service service in eciFile.eciData.Services)
                {
                    if (service.Tags != null)
                    {
                        errorCode = CheckTagRefInTags(tagRef, service.Tags);
                        if (errorCode == Define.ErrorCode.NONE)
                        {
                            break;  // チェックして見つかった場合、その後のチェック処理は行わない
                        }
                    }
                }
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }

        /// <summary>
        /// TagRefINのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckTagRefINInTags(TagRef tagRef, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // TagRefINのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Tagsの中に含まれているかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                //トップのTagsを検索
                errorCode = CheckTagRefINInTags(tagRef, eciFile.eciData.Tags);
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
                //サービス以下のTagsを検索
                foreach (Service service in eciFile.eciData.Services)
                {
                    if (service.Tags != null)
                    {
                        errorCode = CheckTagRefINInTags(tagRef, service.Tags);
                        if (errorCode == Define.ErrorCode.NONE)
                        {
                            break;  // チェックして見つかった場合、その後のチェック処理は行わない
                        }
                    }
                }
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }
            
            return errorCode;
        }
        
        /// <summary>
        /// TagRefOUTのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckTagRefOUTInTags(TagRef tagRef, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // TagRefINのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Tagsの中に含まれているかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                //トップのTagsを検索
                errorCode = CheckTagRefOUTInTags(tagRef, eciFile.eciData.Tags);
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
                //サービス以下のTagsを検索
                foreach (Service service in eciFile.eciData.Services)
                {
                    if (service.Tags != null)
                    {
                        errorCode = CheckTagRefOUTInTags(tagRef, service.Tags);
                        if (errorCode == Define.ErrorCode.NONE)
                        {
                            break;  // チェックして見つかった場合、その後のチェック処理は行わない
                        }
                    }
                }
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }

        /// <summary>
        /// DatasetTagRefのNameがTagsの中に含まれているかチェック
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckDatasetTagRefInTags(TagRef tagRef, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // DatasetTagRefのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Tagsの中に含まれているかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                //トップのTagsを検索
                errorCode = CheckDatasetTagRefInTags(tagRef, eciFile.eciData.Tags);
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
                //サービス以下のTagsを検索
                foreach (Service service in eciFile.eciData.Services)
                {
                    if (service.Tags != null)
                    {
                        errorCode = CheckDatasetTagRefInTags(tagRef, service.Tags);
                        if (errorCode == Define.ErrorCode.NONE)
                        {
                            break;  // チェックして見つかった場合、その後のチェック処理は行わない
                        }
                    }
                }
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }
            
            return errorCode;
        }
        
        /// <summary>
        /// Datasetsの中に重複したDatasetのNameがないかチェック
        /// </summary>
        /// <param name="dataset">Dataset</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)重複なし, それ以外)重複あり</returns>
        public static int CheckSameDatasetName(Dataset dataset, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // DatasetのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Datasetsに重複がないかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                //トップのDatasetsを検索
                errorCode = CheckSameDatasetName(dataset, eciFile.eciData.Datasets);
                if (errorCode != Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
                //サービス以下のDatasetsを検索
                foreach (Service service in eciFile.eciData.Services)
                {
                    if (service.Datasets != null)
                    {
                        errorCode = CheckSameDatasetName(dataset, service.Datasets);
                        if (errorCode != Define.ErrorCode.NONE)
                        {
                            break;  // チェックして見つかった場合、その後のチェック処理は行わない
                        }
                    }
                }
                if (errorCode != Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }
        
        /// <summary>
        /// Collectorsの中に重複したCollectorのNameがないかチェック
        /// </summary>
        /// <param name="collector">Collector</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)重複なし, それ以外)重複あり</returns>
        public static int CheckSameCollectorName(Collector collector, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // CollectorのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Collectorsに重複がないかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                foreach(Service service in eciFile.eciData.Services)
                {
                    errorCode = CheckSameCollectorName(collector, service.Collectors);
                    if (errorCode != Define.ErrorCode.NONE)
                    {
                        break;  // チェックして見つかった場合、その後のチェック処理は行わない
                    }
                }

                if (errorCode != Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }
        

        /// <summary>
        /// CollectorのDatasetNameがDatasetsの中に含まれているかチェック
        /// </summary>
        /// <param name="collector">Collector</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckCollectorInDataset(Collector collector, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            //////////////////////////////////////////////////////////////////////////////
            // CollectorのDatasetNameが、プロジェクトで管理しているすべてのECIファイルの
            // Datasetsの中に含まれているかチェック
            //////////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                errorCode = CheckCollectorInDataset(collector, eciFile.eciData.Datasets);
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
                //サービス以下のDatasetsを検索
                foreach (Service service in eciFile.eciData.Services)
                {
                    if (service.Datasets != null)
                    {
                        errorCode = CheckCollectorInDataset(collector, service.Datasets);
                        if (errorCode == Define.ErrorCode.NONE)
                        {
                            break;  // チェックして見つかった場合、その後のチェック処理は行わない
                        }
                    }
                }

                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }



        /// <summary>
        /// CollectorRefのNameがCollectorsの中に含まれているかチェック
        /// </summary>
        /// <param name="collectorRef">検索対象のCollectors</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)含まれている, それ以外)含まれていない</returns>
        public static int CheckCollectorRefInCollectors(CollectorRef collectorRef, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // CollectorRefのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Collectorsの中に含まれているかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                foreach (Service service in eciFile.eciData.Services)
                {
                    errorCode = CheckCollectorRefInCollectors(collectorRef, service.Collectors);
                    if (errorCode == Define.ErrorCode.NONE)
                    {
                        break;  // チェックして見つかった場合、その後のチェック処理は行わない
                    }
                }

                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }

        /// <summary>
        /// Serviceの依存関係がServicesの中に含まれているかチェック
        /// </summary>
        /// <param name="service">Service</param>
        /// <param name="eciFiles">ECI管理リスト</param>
        /// <returns>エラーコード:0x0000)含まれているor依存関係が空文字, それ以外)含まれていない</returns>
        public static int CheckSrvdepInServices(Service service, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // Serviceのdependentが、プロジェクトで管理しているすべてのECIファイルの
            // Serviceの中に含まれているかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                errorCode = CheckSrvdepInServices(service, eciFile.eciData.Services);
                if (errorCode == Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }


        /// <summary>
        /// Servicesの中に重複したServiceのNameがないかチェック
        /// </summary>
        /// <param name="service">Service</param>
        /// <param name="services">検索対象のService</param>
        /// <returns>エラーコード:0x0000)重複なし, それ以外)重複あり</returns>
        public static int CheckSameServiceName(Service service, List<EciFileMgmt> eciFiles)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用

            ///////////////////////////////////////////////////////////////////////////
            // ServiceのNameが、プロジェクトで管理しているすべてのECIファイルの
            // Servicesに重複がないかチェック
            ///////////////////////////////////////////////////////////////////////////
            foreach (EciFileMgmt eciFile in eciFiles)
            {
                errorCode = CheckSameServiceName(service, eciFile.eciData.Services);
                if (errorCode != Define.ErrorCode.NONE)
                {
                    break;  // チェックして見つかった場合、その後のチェック処理は行わない
                }
            }

            return errorCode;
        }
        #endregion

        /// <summary>
        /// プロジェクトファイルの名前エラーチェック
        /// </summary>
        /// <param name="name">プロジェクトファイルの名前</param>
        /// <returns>エラーコード:0x0000)エラーなし, それ以外)エラー</returns>
        public static int ProjectName(string name)
        {
            int errorCode = Define.ErrorCode.NONE;  // エラーコード返却用
            bool isWS, isHWAC, isOver;              // 条件判定用

            ///////////////////////////////////////////
            // 空白文字列の場合エラー
            ///////////////////////////////////////////
            isWS = IsStrWhiteSpace(name);
            if (isWS == true)
            {
                return Define.ErrorCode.NULL;
            }

            ///////////////////////////////////////////
            // 半角英数字以外の場合エラー
            ///////////////////////////////////////////
            isHWAC = IsStrHWAC(name);
            if (isHWAC == false)
            {
                return Define.ErrorCode.EXCEPT_HWAC;
            }

            ///////////////////////////////////////////
            // 100Byteを超える場合エラー
            ///////////////////////////////////////////
            isOver = IsStrOverByteNum(name, 100);
            if (isOver == true)
            {
                return Define.ErrorCode.OVER_BYTE;
            }

            return errorCode;
        }

        #endregion パブリックメソッド

        #region プライベートメソッド

        /// <summary>
        /// 文字列が空・空白かチェック
        /// </summary>
        /// <param name="str">チェックする文字列</param>
        /// <returns>true:文字列が空・空白, false:文字列が空・空白以外</returns>
        private static bool IsStrWhiteSpace(string str)
        {
            bool isNull = false;

            if (String.IsNullOrWhiteSpace(str) == true)
            {
                isNull = true;
            }

            return isNull;
        }

        /// <summary>
        /// 文字列が指定したバイト数を超えたかチェック
        /// </summary>
        /// <param name="str">チェックする文字列</param>
        /// <param name="byteNum">バイト数</param>
        /// <returns>true:バイト数を超えた、false:バイト数超えていない</returns>
        private static bool IsStrOverByteNum(string str, int byteNum)
        {
            int byteCount;
            bool isOver = false;

            byteCount = System.Text.Encoding.GetEncoding("Shift_JIS").GetByteCount(str); // byte数を取得
            if (byteNum < byteCount)
            {
                isOver = true;
            }

            return isOver;
        }

        /// <summary>
        /// 文字列が半角英数字(half-width alphanumeric characters)で構成されているかチェック
        /// </summary>
        /// <param name="str">チェックする文字列</param>
        /// <returns>true:文字列が半角英数字で構成されている, false;半角英数字で構成されていない</returns>
        private static bool IsStrHWAC(string str)
        {
            int byteCount;
            bool isHWAC = false;

            ///////////////////////////////////////////////////////////////////////
            // 入力した文字数とバイト数が等しいなら半角英数字のみの構成
            ///////////////////////////////////////////////////////////////////////
            byteCount = System.Text.Encoding.GetEncoding("Shift_JIS").GetByteCount(str);
            if (byteCount == str.Length)
            {
                isHWAC = true;
            }

            return isHWAC;
        }

        /// <summary>
        /// 文字列をInt32型数値に変換できるかチェック
        /// </summary>
        /// <param name="str">文字列</param>
        /// <returns>true:Int32型数値に変換できる, false:Int32型数値に変換できない</returns>
        private static bool IsStrNum(string str)
        {
            bool isNum;

            isNum = Int32.TryParse(str, out int num);

            return isNum;
        }

        /// <summary>
        /// 文字列が指定した数値の範囲内かチェック
        /// </summary>
        /// <param name="str">チェックする文字列</param>
        /// <param name="min">最小値</param>
        /// <param name="max">最大値</param>
        /// <returns>true:範囲内である, false:範囲外または数値ではない</returns>
        private static bool IsStrWithinRange(string str, int min, int max)
        {
            int num;
            bool isNum, isWR;

            //////////////////////////////////////
            // 数値に変換できるかチェック
            //////////////////////////////////////
            isNum = Int32.TryParse(str, out num);
            if (isNum == true)
            {
                ///////////////////////////////////////////////////////////
                // 文字列が数値の場合、最小・最大値の範囲内かチェック
                ///////////////////////////////////////////////////////////
                if (min <= num && num <= max)
                {
                    isWR = true;
                }
                else
                {
                    isWR = false;
                }
            }
            else
            {
                isWR = false;
            }

            return isWR;
        }


        /// <summary>
        /// Tags名の中に同じTagRef名が含まれるか
        /// </summary>
        /// <param name="tagRef">TagRef</param>
        /// <param name="tags">Tags</param>
        /// <returns>true:含まれる, false:含まれない</returns>
        private static bool IsTagRefInTags(TagRef tagRef, List<Tag> tags)
        {
            bool isInTags = false;

            foreach (Tag tag in tags)
            {
                if (tagRef.Name == tag.Name)
                {
                    isInTags = true;
                    break;
                }
            }

            return isInTags;
        }
        #endregion
    }
}
