﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using RTedge;
using System;
using System.Threading.Tasks;
using System.Diagnostics;           // Debug.WriteLine出力用
using System.IO;
using System.Windows.Forms;
using Timer = System.Windows.Forms.Timer;
using System.Runtime.InteropServices;

namespace EgBoot
{
    [StructLayout(LayoutKind.Sequential, Pack = 1)]//1バイトアライメントに設定
    /// <summary>
    /// EgTagSetDefaultValue データ構造
    /// </summary>
    public struct SETDEFAULTVALUEDESC
    {
        public byte byTarget;                       // 初期値に更新対象 0:ALL 0以外:Group番号対象
        public UInt16 woGroup;                      // グループ番号
    }

    public partial class MainForm : Form
    {
        edge_API EGAPI;                         //edge APIのインスタンス
        MnConvertDefFile CNV;                   //xml変換のインスタンス
        edge_API.RTCDMEMINFO m_LogMBoxInfo = new edge_API.RTCDMEMINFO(); //Log出力用メールボックス情報
        private  UInt32 Live;                   // サービス生存マーク

        public const UInt16 EDGE_VER_MAJER = 3;
        public const UInt16 EDGE_VER_MINER = 8;
        public const UInt16 EDGE_VER_REVISION = 1;

        System.Timers.Timer lastTagTrigEntry = new System.Timers.Timer(1000);       // タグトリガー最終登録を行う(動作間隔:1000ms)
        public UInt16 limitEntryDist;                                               // タグトリガー最終登録までのリミット
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MainForm()
        {

            InitializeComponent();          //画面のinitialize

            EGAPI = new edge_API();         //edge APIのインスタンス作成
            CNV = new MnConvertDefFile(ref EGAPI, ref m_LogMBoxInfo);   //xml変換のインスタンス作成

            //イニシャル処理
            Initialize();
        }

        /// <summary>
        /// 初期化処理
        /// </summary>
        private void Initialize()
        {
            try
            {
                //exeのパスからxmlに変換
                string xmlPath = Path.ChangeExtension(System.Reflection.Assembly.GetEntryAssembly().Location, ".xml");

                // xmlファイルを読み込み
                RTedgeXmlObj.RTedge xmlData = CNV.ReadXmlFile(xmlPath);
                // EgBoot固有の初期化処理
                int ret = CNV.InitEgBootXml(xmlData);
                if ( ret != 0)
                {
                    // NOTE: イニシャル処理でエラーが発生した場合、_PrintSysLog自体も失敗するため、メッセージボックスにてエラー表示します。
                    MessageBox.Show("Failed to initialize." + Environment.NewLine + "Please check the environment." + Environment.NewLine + "EgInit error: 0x" + ret.ToString("x8"), "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                //メッセージは自分自身なので送らない個別に実施
                edge_API.EDGECONFIG config = new edge_API.EDGECONFIG();
                config = EGAPI.EDGE_CONFIG_DEFAULT;         //初期値設定
                config.bReadXmlF = 0;                       //読み込みは行わない
                config.mInstMessFunc = UserMessageHandler;  //ユーザーメッセージハンドラ追加
                //イニシャル処理
                ret = EGAPI.EgInit(config);                    //イニシャライズ実行
                if (ret != 0)
                {
                    edge_FW._PrintSysLog("EgBoot", "ERR", "Failed to initialize. Please check the environment.");

                    // NOTE: イニシャル処理でエラーが発生した場合、_PrintSysLog自体も失敗するため、メッセージボックスにてエラー表示します。
                    MessageBox.Show("Failed to initialize." + Environment.NewLine + "Please check the environment." + Environment.NewLine + "EgInit error: 0x" + ret.ToString("x8"), "EgBoot error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    //EgBoot専用で登録するTAG
                    EgBootTagAdd();

                    //EgLog開始処理 (ログ欠損抑止のためEgLogを先行して起動させる)
                    CNV.StartEgLog(xmlData);

                    //xmlからオブジェクト作成(EgBoot専用)
                    CNV.ConvertEgBootXml2Obj(xmlData);

                    TagTrigTimerInit();        // タグトリガー最終登録用タイマー起動

                    // live timer
                    Timer drawtimer = new Timer();
                    drawtimer.Interval = 1000;                           // msec
                    drawtimer.Tick += (ss, ee) =>
                    {
                        EGAPI.EgTagWrite("SERVICE.EgBoot.Live", ++Live);
                        EGAPI.EgTagWrite("SERVICE.EgBoot.Run", (bool)true);
                    };
                    drawtimer.Enabled = true;
                }

                edge_FW._PrintSysLog("EgBoot", "DBG", "Initialize Finished.");
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// MainFormのロード
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            try
            {
                //何かあれば
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Coreで専用のTagを登録
        /// </summary>
        private void EgBootTagAdd()
        {
            try
            {
                //Tag追加 API
                // バージョン情報
                UInt16 Version = (UInt16)(EDGE_VER_MAJER * 100 + EDGE_VER_MINER * 10 + EDGE_VER_REVISION);
                EGAPI.EgTagCreate("EDGESYSTEM.EdgeVersion", (ushort)EGDEFINE.egTagDataType.UInt16, 1,"" ,"RT-edge version");
                EGAPI.EgTagWrite("EDGESYSTEM.EdgeVersion", Version);
                // ログ欠損カウント
                EGAPI.EgTagCreate("EDGESYSTEM.LogLostCnt", (ushort)EGDEFINE.egTagDataType.UInt32, 4, "", "LogMsg Lost Count");
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        
        /// <summary>
        /// ユーザーメッセージサービスストップ用処理
        /// </summary>
        /// <returns></returns>
        private void fnStop()
        {
            CloseProcess();              // 終了処理
            notifyIcon1.Visible = false; // アイコンをトレイから取り除く
            Application.Exit();
        }
        /// <summary>
        /// Form Close処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Closing(object sender, FormClosedEventArgs e)
        {
            CloseProcess();
        }

        /// <summary>
        /// xml読込処理(メッセージ受信時の非同期実行用)
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="paramArray"></param>
        /// <returns></returns>
        private async Task ConvertXml2ObjAsync(string senderName, byte[] paramArray)
        {
            await Task.Run(() =>
            {
                //xmlからオブジェクトに変換
                CNV.ConvertXml2Obj(senderName, paramArray);
                //タグトリガーの登録処理(タグ生成後に一斉実行)
                limitEntryDist = 60000; // タグトリガー最終登録リミットを初期化
                if(CNV.ListTagTriggers.Count != 0)
                    CNV.AttachServiceTagTrigger(false);
                //読み終わったら送信元へメッセージ送信
                EGAPI.EgMailboxSendSC(senderName, EGAPI.EGFW.ServiceName, (int)EGDEFINE.EGMSG.EM_READ_REQUEST_SERVICE_XML_END);
            });
        }
        /// <summary>
        /// 指定のタグの値をデフォルト値にセット
        /// </summary>
        /// <param name="senderName">送信元名称(返信メールボックス名に使用)</param>
        /// <param name="paramArray">更新対象グループ</param>
        private void ReloadTagDefaultValues(string senderName, byte[] paramArray)
        {
            int result = 0;
            int last_result = 0;
            uint InitGroup = 0;
            SETDEFAULTVALUEDESC desc = new SETDEFAULTVALUEDESC();
            object descObj = (object)desc;

            //パラメータから更新対象グループを取得する、指定が無い場合は全部が対象
            if (paramArray != null && paramArray.Length >= 3)
            {
                egCommonFunc.fnByToObj(paramArray, ref descObj);
                desc = (SETDEFAULTVALUEDESC)descObj;
            }

            //全てのタグを回す
            foreach (MnConvertDefFile.TagVal tv in this.CNV.TagVals)
            {
                //タグの値が無い場合は更新対象外
                if ((tv.Value == null) || (tv.Value.Length == 0)) {
                    continue;
                }
                //グループNo指定がある場合チェック 0または指定が無い場合は全て
                if(desc.byTarget != 0)
                {
                    if (tv.InitGroup == null || tv.InitGroup == "")//nullまたは指定が無い場合は対象外
                        continue;
                    //文字列からushort型に
                    InitGroup = CUtility.fStringToUshort(tv.InitGroup);
                    
                    //同じ条件であれば初期値に戻す対象とする。違う場合はコンテニュー
                    if (desc.woGroup != InitGroup)
                        continue;//違う場合は次のタグへ
                }
                // タイプに応じた値の変更
                object objValue = egCommonFunc.GetTagAnyDataFromString(tv.Value, (EGDEFINE.egTagDataType)CUtility.fStringToUshort(tv.Type));
                if (objValue == null) {
                    edge_FW._PrintSysLog(MnConvertDefFile.CORE_SERVICE_NAME, "ERR", "ReloadAllTagDefaultValues -> EgTagWrite Value is Invalid Data. Name : {0}, Value : {1}, Type : {2}", tv.Name, tv.Value, tv.Type);
                    continue;
                }

                // Type別書き込み処理
                switch( CUtility.fStringToUshort(tv.Type) ) {
                case (ushort)EGDEFINE.egTagDataType.String:                 // 文字列
                    result = EGAPI.EgTagWriteString(tv.Name, (string)objValue);
                    break;
                default:                                                    // それ以外
                    result = EGAPI.EgTagWrite(tv.Name, objValue);
                    break;
                }
                if(result != 0) {
                    last_result = result;
                    edge_FW._PrintSysLog(MnConvertDefFile.CORE_SERVICE_NAME, "ERR", "ReloadAllTagDefaultValues -> EgTagWrite Error. Name : {0}, state : {1}", tv.Name, result.ToString());
                }
            }
                         
            byte[] byParams = BitConverter.GetBytes(last_result);   // 最終エラー値をバイト配列に変換して返却

            //読み終わったら送信元へメッセージ送信
            EGAPI.EgMailboxSendSC(senderName, EGAPI.EGFW.ServiceName, (int)EGDEFINE.EGMSG.EM_TAG_DEFAULT_VALUE_SET_END, byParams);
        }

        /// <summary>
        /// ユーザーメッセージハンドラ
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="messNo"></param>
        /// <param name="paramArray"></param>
        /// <returns></returns>
        public int UserMessageHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                int rtn = 0;

                switch (messNo) //メッセージ番号毎に処理
                {
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:
                        {
                            //終了関数呼び出し
                            fnStop();
                        }
                        break;
                    case (int)EGDEFINE.EGMSG.EM_READ_REQUEST_SERVICE_XML: //message Readxml 他のサービスが起動時に送られてくる
                        {
                            // xmlからオブジェクトに変換
                            // NOTE: 別スレッドにて非同期実行。処理完了は待ちません。
                            _ = ConvertXml2ObjAsync(senderName, paramArray);
                        }
                        break;
                    case (int)EGDEFINE.EGMSG.EM_TAG_DEFAULT_VALUE_SET: // デフォルト値セット
                        {
                            // 全タグ値のデフォルト値リロード実施
                            ReloadTagDefaultValues(senderName, paramArray);
                        }
                        break;
                    default:
                        break;
                }
                return rtn;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }

        }

        private void TagTrigTimerInit()
        {
            limitEntryDist = 60000; // タグトリガー最終登録までの時間制限を60secに設定
            lastTagTrigEntry.Elapsed += (sender, e) =>
            {
                bool boWait = true;

                while (boWait)
                {
                    limitEntryDist -= 1000;
                    if (limitEntryDist == 0)
                        boWait = false;
                }
                if (CNV.ListTagTriggers.Count != 0)
                    CNV.AttachServiceTagTrigger(true);     // タグトリガー最終登録処理実施

                lastTagTrigEntry.Enabled = false;
            };

            lastTagTrigEntry.Start();
        }

        /// <summary>
        /// 常駐アイコン終了メニュー押下時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
		private void 終了ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CloseProcess();              // 終了処理
            notifyIcon1.Visible = false; // アイコンをトレイから取り除く
            Application.Exit();
        }
        /// <summary>
        /// 終了処理
        /// </summary>
        private void CloseProcess()
        {
            try
            {
                EGAPI.EgMailboxDelete(m_LogMBoxInfo);   //ログ用メールボックス削除
                EGAPI.EgFinalize();                         //後処理
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
    }
}
