﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.IO;
using System.Linq;
using System.Globalization;
using System.Diagnostics;           // Debug.WriteLine出力用

namespace EgBoot
{
    public class CUtility
    {
        /// <summary>
        /// 数値を小数点まで表示する
        /// </summary>
        /// <param name="nValue">変換元の数値(float)</param>
        /// <param name="nDigit">小数桁数</param>
        /// <returns></returns>
        public static string strToStringDigit(float fValue, int nDigit)
        {
            return fValue.ToString("F" + nDigit.ToString());
        }
        /// <summary>
        /// 数値を小数点まで表示する
        /// </summary>
        /// <param name="nValue">変換元の数値(double)</param>
        /// <param name="nDigit">小数桁数</param>
        /// <returns></returns>
        public static string strToStringDigit(double dValue, int nDigit)
        {
            return dValue.ToString("F" + nDigit.ToString());
        }


        /// <summary>
        /// 文字列値からfloat型の値を返す
        /// </summary>
        /// <param name="strValue">変換元の値</param>
        /// <returns></returns>
        public static float fStringToFloat(string strValue)
        {
            float fValue = 0;
            try
            {
                if (float.TryParse(strValue, out fValue) == false)
                {
                    fValue = 0;
                }
            }
            catch (Exception)
            {
                fValue = 0;
            }
            return fValue;
        }

        /// <summary>
        /// 文字列値からdouble型の値を返す
        /// </summary>
        /// <param name="strValue">変換元の値</param>
        /// <returns></returns>
        public static double dStringToDouble(string strValue)
        {
            double dValue = 0;
            try
            {
                if (double.TryParse(strValue, out dValue) == false)
                {
                    dValue = 0;
                }
            }
            catch (Exception)
            {
                dValue = 0;
            }
            return dValue;
        }


        /// <summary>
        /// 文字列値からint型の値を返す
        /// </summary>
        /// <param name="strValue">変換元の値</param>
        /// <returns></returns>
        public static int fStringToInt(string strValue)
        {
            int nValue = 0;
            try
            {
                if (int.TryParse(strValue, out nValue) == false)
                {
                    nValue = 0;
                }
            }
            catch (Exception)
            {
                nValue = 0;
            }
            return nValue;
        }

        /// <summary>
        /// 文字列(16進数値)からlong型の値を返す
        /// </summary>
        /// <param name="strValue">変換元の値</param>
        /// <returns></returns>
        public static long dHexStringToLong(string strValue)
        {
            long lValue = 0;
            try
            {
                if (long.TryParse(strValue, System.Globalization.NumberStyles.HexNumber, CultureInfo.InvariantCulture, out lValue) == false)
                {
                    lValue = 0;
                }
            }
            catch (Exception)
            {
                lValue = 0;
            }
            return lValue;
        }

        /// <summary>
        /// 文字列値からint型の値を返す
        /// </summary>
        /// <param name="strValue">変換元の値</param>
        /// <returns></returns>
        public static Int16 fStringToInt16(string strValue)
        {
            Int16 nValue = 0;
            try
            {
                if (Int16.TryParse(strValue, out nValue) == false)
                {
                    nValue = 0;
                }
            }
            catch (Exception)
            {
                nValue = 0;
            }
            return nValue;
        }

        /// <summary>
        /// 文字列からUshortへ変換
        /// </summary>
        /// <param name="strValue"></param>
        /// <returns></returns>
        public static ushort fStringToUshort(string strValue)
        {
            ushort nValue = 0;
            try
            {
                if (ushort.TryParse(strValue, out nValue) == false)
                {
                    nValue = 0;
                }
            }
            catch (Exception)
            {
                nValue = 0;
            }
            return nValue;
        }

        /// <summary>
        /// 文字列からuint32へ変換
        /// </summary>
        /// <param name="strValue"></param>
        /// <returns></returns>
        public static uint fStringToUint(string strValue)
        {
            uint nValue = 0;
            try
            {
                if (uint.TryParse(strValue, out nValue) == false)
                {
                    nValue = 0;
                }
            }
            catch (Exception)
            {
                nValue = 0;
            }
            return nValue;
        }

        /// <summary>
        /// 文字列からEnum(T)へ変換
        /// </summary>
        /// <param name="strValue">Enumﾒﾝﾊﾞ名・数値指定のどちらも可。文字列表記は大文字・小文字を区別しない。</param>
        /// <returns></returns>
        public static T fStringToEnum<T>(string strValue) where T : struct
        {
            T nValue = default(T);
            try
            {
                if (typeof(T).IsEnum)
                {
                    if (Enum.TryParse(strValue, true, out nValue) == false)
                    {
                        nValue = default(T);
                    }
                }
            }
            catch (Exception)
            {
                nValue = default(T);
            }
            return nValue;
        }

        /// <summary>
        /// 値の上下限をチェックして、範囲内の値で返す
        /// </summary>
        /// <typeparam name="T">範囲チェックする値の型</typeparam>
        /// <param name="tValue">範囲チェックする値</param>
        /// <param name="tMin">下限値</param>
        /// <param name="tMax">上限値</param>
        /// <returns>上下限チェック後の値</returns>
        public static T CheckValueRange<T>(T tValue, T tMin, T tMax)
            where T : IComparable
        {
            T tReturn;

            if (tMin.CompareTo(tValue) > 0) { tReturn = tMin; }
            else if (tMax.CompareTo(tValue) < 0) { tReturn = tMax; }
            else { tReturn = tValue; }

            return tReturn;
        }

        /// <summary>
        /// 入力値の範囲チェックを行った値を返す
        /// </summary>
        /// <param name="strValue">範囲チェックする値</param>
        /// <param name="dMin">下限値</param>
        /// <param name="dMax">上限値</param>
        /// <returns>上下限チェック後の値</returns>
        public static int CheckInputValue(int inValue, int inMin, int inMax)
        {
            return CheckValueRange<int>(inValue, inMin, inMax);
        }

        /// <summary>
        /// 入力値の範囲チェックを行った値を返す
        /// </summary>
        /// <param name="strValue">範囲チェックする値</param>
        /// <param name="dMin">下限値</param>
        /// <param name="dMax">上限値</param>
        /// <returns>上下限チェック後の値</returns>
        public static int CheckInputValue(string strValue, int inMin, int inMax)
        {
            return CheckValueRange<int>(fStringToInt(strValue), inMin, inMax);
        }

        /// <summary>
        /// 入力値の範囲チェックを行った値を返す(16進数用)
        /// strMin, strMaxは必ず16進数文字を指定すること
        /// </summary>
        /// <param name="strValue">範囲チェックする値</param>
        /// <param name="dMin">下限値</param>
        /// <param name="dMax">上限値</param>
        /// <returns>上下限チェック後の値</returns>
        public static long CheckInputHexValue(string strValue, string strMin, string strMax)
        {
            return CheckValueRange(dHexStringToLong(strValue), dHexStringToLong(strMin), dHexStringToLong(strMax));
        }



        /// <summary>
        /// 入力値の範囲チェックを行った値を返す
        /// </summary>
        /// <param name="strValue">範囲チェックする値</param>
        /// <param name="dMin">下限値</param>
        /// <param name="dMax">上限値</param>
        /// <returns>上下限チェック後の値</returns>
        public static double CheckInputValue(string strValue, double dMin, double dMax)
        {
            return CheckValueRange(dStringToDouble(strValue), dMin, dMax);
        }

        /// <summary>
        /// 入力値の範囲チェックを行い小数点桁数でフォーマットした文字列を返す
        /// </summary>
        /// <param name="strValue">範囲チェックする値</param>
        /// <param name="dMin">下限値</param>
        /// <param name="dMax">上限値</param>
        /// <param name="nDigit">小数点桁数</param>
        /// <returns>上下限チェック後の値</returns>
        public static string CheckInputValue(string strValue, double dMin, double dMax, int nDigit)
        {
            return strToStringDigit(CheckValueRange<double>(dStringToDouble(strValue), dMin, dMax), nDigit);
        }

        /// <summary>
        /// 入力値の範囲チェックを行い小数点桁数でフォーマットした文字列を返す
        /// </summary>
        /// <param name="strValue">範囲チェックする値</param>
        /// <param name="fMin">下限値</param>
        /// <param name="fMax">上限値</param>
        /// <param name="nDigit">小数点桁数</param>
        /// <returns>上下限チェック後の値</returns>
        public static string CheckInputValue(string strValue, float fMin, float fMax, int nDigit)
        {
            return strToStringDigit(CheckValueRange<float>(fStringToFloat(strValue), fMin, fMax), nDigit);
        }

        /// <summary>
        /// 文字列終端から'\0'を取り除く
        /// </summary>
        /// <param name="strValue">'\0'を取り除く文字列</param>
        /// <returns>'\0'を取り除いた文字列</returns>
        public static string TrimEnd(string strValue)
        {
            int nPos = -1;
            string strReturn = strValue;

            try
            {
                nPos = strValue.IndexOf('\0');
            }
            catch (Exception)
            {
            }
            if (nPos >= 0)
            {
                strReturn = strValue.Substring(0, nPos);
            }

            return strReturn;
        }

        /// <summary>
        /// 文字列からチェックデジット値を取得する
        /// </summary>
        /// <param name="strCode"></param>
        /// <returns></returns>
        public static string fnCreateCheckDigit(string strCode)
        {
            string strRtn = "";
            int inEven = 0;
            int inOdd = 0;
            int inPos = 0;
            int inSum;
            string strBuff;

            for (int i = 0; i < strCode.Length; i += 2)
            {
                // 偶数桁の数字を加算する
                inPos = strCode.Length - i - 1;
                if (inPos >= 0)
                {
                    strBuff = strCode.Substring(inPos, 1);
                    inEven += fStringToInt(strBuff);
                }
                // 奇数桁の数字を加算する
                inPos = strCode.Length - i - 2;
                if (inPos >= 0)
                {
                    strBuff = strCode.Substring(inPos, 1);
                    inOdd += fStringToInt(strBuff);
                }
            }
            inSum = inEven * 3 + inOdd;
            strRtn = ((10 - (inSum % 10)) % 10).ToString();

            return strRtn;
        }

        /// <summary>
        /// 値をSTRING型にして返す
        /// </summary>
        /// <param name="oValue"></param>
        /// <returns></returns>
        public static string fnGetString(object oValue)
        {
            string strRtn = "";
            try
            {
                if (oValue != null)
                {
                    strRtn = oValue.ToString();
                }
            }
            catch (Exception)
            {
            }

            return strRtn;
        }

        /// <summary>
        /// 値をINT型にして返す
        /// </summary>
        /// <param name="oValue"></param>
        /// <returns></returns>
        public static int fnGetInt(object oValue)
        {
            int inRtn = 0;
            try
            {
                int.TryParse(oValue.ToString(), out inRtn);
            }
            catch (Exception)
            {
            }

            return inRtn;
        }

        /// <summary>
        /// 値をShort型にして返す
        /// </summary>
        /// <param name="oValue"></param>
        /// <returns></returns>
        public static short fnGetShort(object oValue)
        {
            short shRtn = 0;
            try
            {
                short.TryParse(oValue.ToString(), out shRtn);
            }
            catch (Exception)
            {
            }

            return shRtn;
        }
        /// <summary>
        /// ファイルの移動
        /// </summary>
        /// <param name="stFromCertPath"></param>
        /// <param name="stToCertFolderPath"></param>
        /// <returns></returns>
        public static bool fnMoveFile(string stFromPath, string stToPath)
        {
            try
            {
                //上書きコピーして削除する
                //(moveの場合既にファイルがいたらエラーになるので)
                System.IO.File.Copy(stFromPath, stToPath, true);
                FileInfo file = new FileInfo(stFromPath);
                if ((file.Attributes & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
                {
                    file.Attributes = FileAttributes.Normal;
                }
                file.Delete();
                return true;
            }
            catch (Exception ep)
            {
                Debug.WriteLine(ep.Message);
                return false;
            }
        }
        /// <summary>
        /// ファイルコピー
        /// </summary>
        /// <param name="stFromPath"></param>
        /// <param name="stToPath"></param>
        /// <param name="bOverwrite"></param>
        /// <returns></returns>
        public static bool fnCopyFile(string stFromPath, string stToPath, bool bOverwrite)
        {
            try
            {
                System.IO.File.Copy(stFromPath, stToPath, bOverwrite);
                return true;
            }
            catch (Exception ep)
            {
                Debug.WriteLine(ep.Message);
                return false;
            }
        }
        /// <summary>
        /// ファイル名に使用できない文字列が含まれていないか調べる
        /// </summary>
        /// <param name="fileName">チェックするファイル名。</param>
        /// <returns>不正がないと判断された場合は、True。それ以外は、False。</returns>
        public static bool IsValidFileName(string fileName)
        {

            try
            {
                if (string.IsNullOrEmpty(fileName)) //NULL・空の場合はfalse
                {
                    return false;
                }

                char[] invalidPathChars = Path.GetInvalidPathChars();

                return invalidPathChars.Any(ch => fileName.Contains(ch));

            }
            catch (Exception ep)
            {
                Debug.WriteLine(ep.Message);
                return false;
            }
        }

        /// <summary>
        /// パス名に使用できない文字列が含まれていないか調べる
        /// </summary>
        /// <param name="fileName">チェックするパス名。</param>
        /// <returns>不正がないと判断された場合は、True。それ以外は、False。</returns>
        public static bool IsValidPath(string path)
        {

            try
            {
                if (string.IsNullOrEmpty(path)) //NULL・空の場合はfalse
                {
                    return false;
                }
                char[] invalidPathChars = Path.GetInvalidPathChars();

                return invalidPathChars.Any(ch => path.Contains(ch));

            }
            catch (Exception ep)
            {
                Debug.WriteLine(ep.Message);
                return false;
            }
        }
        /// <summary>
        /// ダブルクォーテーションを前後につける
        /// </summary>
        /// <param name="stOrtStr"></param>
        /// <returns></returns>
        public static string AddDQStr(string stOrtStr)
        {
            string stNewStr = "\"" + stOrtStr + "\"";
            return stNewStr;
        }
        /// <summary>
        /// メッセージの変換
        /// </summary>
        /// <param name="stFormatMess"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public static string ChgFormatMess(string stFormatMess, params object[] args)
        {
            string formatMess = stFormatMess;
            for (int i = 0; i < args.Length; ++i)
            {
                formatMess = formatMess.Replace("{" + i.ToString() + "}", args[i].ToString());
            }
            return formatMess;
        }

        /// <summary>
        /// ﾌｫﾙﾀﾞが存在しているかﾁｪｯｸし、なければﾌｫﾙﾀﾞを作成する
        /// </summary>
        /// <param name="srPath"></param>
        /// <returns></returns>
        public static bool gfMakeFolder(string srPath)
        {
            // フォルダが存在しているかチェックする
            if (System.IO.Directory.Exists(srPath))
            {                                                           // 既にフォルダが存在している場合
                return true;                                            // 成功を返して処理を終了する
            }

            try
            {
                System.IO.Directory.CreateDirectory(srPath);            // フォルダを生成する
            }
            catch (Exception)
            {                                                           // 例外異常が発生した場合
                return false;                                           // 失敗を返して処理を終了する
            }
            return true;
        }

        
    }
    /// <summary>
    /// csvの要素の前後に""をつける
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class CsvList<T> : System.Collections.Generic.List<string>
    {
        // Listt<T>のAddを隠蔽し再定義（orverrideできなため）
        public new void Add(string item)
        {
            string newStr = item.Replace("\"", "\"\"");
            // ダブルクォーテーションで文字列を囲む
            base.Add("\"" + newStr + "\"");
        }
    }

}
