﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using RTedge;
using System;
using System.Collections.Generic;
using System.Diagnostics;           // Debug.WriteLine出力用
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Reflection;

namespace EgLog
{
    public partial class OutPutLog : Form
    {
        edge_API EGAPI  = new edge_API();       //edge APIのインスタンス
        bool IsActionF  = false;                // ログ取得処理の実行判定フラグ
        int dispRowMax  = 100;                  // ログ表示上限数
        
        // xmlで指定したサービスコンテナ名 EgInit後設定
        public string ServiceRealName = "";
        // サービスコンテナ名デフォルト値 EgInit前で使用
        public string DefServiceName = "EgLog";
        /// <summary>
        /// サービスプロパティタグ名：[.Live]
        /// </summary>
        private string TAG_LIVE
        {
            get { return String.Format("SERVICE.{0}.Live", GetServiceName()); }
        }
        /// <summary>
        /// サービスプロパティタグ名：[.Run]
        /// </summary>
        private string TAG_RUN
        {
            get { return String.Format("SERVICE.{0}.Run", GetServiceName()); }
        }
        /// <summary>
        /// サービスプロパティタグ名：[.Error]
        /// </summary>
        private string TAG_ERROR
        {
            get { return String.Format("SERVICE.{0}.Error", GetServiceName()); }
        }

        
        /// <summary>
        /// 自己のサービス名を返す
        /// </summary>
        /// <returns></returns>
        public String GetServiceName()
        {
            if (ServiceRealName.Length > 0) //XMLで指定したサービス名 EgInit後に設定 設定がない場合はデフォルト値を返す
                return ServiceRealName;
            return DefServiceName; //"EgLog"
        }
        public OutPutLog(string[] args)
        {
            InitializeComponent();

            //イニシャル処理
            edge_API.EDGECONFIG config = new edge_API.EDGECONFIG();
            config = EGAPI.EDGE_CONFIG_DEFAULT;         //初期値設定
            config.mInstMessFunc = UserMessageHandler;  //ユーザーメッセージハンドラ追加
            int ret = EGAPI.EgInit(config);             //イニシャライズ実行
            if (ret != 0)
                edge_FW._PrintSysLog(DefServiceName, "ERR", "Failed to initialize. Please check the environment.");

            // サービス名の設定
            ServiceRealName = EGAPI.EGFW.ServiceRealName;

            // 起動引数の取得
            if (args.Count() > 0)
            {
                foreach (var item in args)
                {
                    string[] arr = item.Split('=');

                    // ログ表示上限数を取得
                    if (arr.Count() > 1 && arr[0].Trim() == "DispNumMax")
                    {
                        int.TryParse(arr[1].Trim(), out dispRowMax);
                        edge_FW._PrintSysLog(GetServiceName(), "DBG", "Maximum number of logs to display : [{0}]", dispRowMax.ToString());
                    }
                }
            }
        }

        /// <summary>
        /// ログ取得処理
        /// NOTE: drawtimerイベントにて、画面更新とファイル出力を非同期で実行します
        /// </summary>
        private async void GetLog()
        {
            Queue<List<string>> logMessList = new Queue<List<string>>();    // 画面表示更新用
            Queue<List<string>> fileoutput = new Queue<List<string>>();     // ログファイル出力用

            var logfile = new LogFile();

            // 画面更新・ファイル出力タイマー
            UInt32 Live = 0;
            Timer drawtimer = new Timer();

            drawtimer.Interval = 1000;                           // msec
            drawtimer.Tick += (ss, ee) =>
            {
                int nowCnt = logMessList.Count;
                if (nowCnt > 0)                             // データが溜まっていれば更新
                {
                    //
                    // 画面更新
                    //
                    Invoke((MethodInvoker)delegate
                    {
                        LogView.BeginUpdate();
                        try
                        {
                            for (int i = 0; i < nowCnt; i++)
                            {
                                List<string> item = logMessList.Dequeue();

                                var msg = new ListViewItem(item.ToArray());
                                LogView.Items.Add(msg);
                                LogView.Items[LogView.Items.Count - 1].Selected = true;
                                LogView.Items[LogView.Items.Count - 1].Focused = true;
                                if( item.Contains("ERR") )
                                    LogView.Items[LogView.Items.Count - 1].ForeColor = System.Drawing.Color.Red;
                                else if (item.Contains("DBG"))
                                    LogView.Items[LogView.Items.Count - 1].ForeColor = System.Drawing.Color.LightGray;

                                // 上限超過時は先頭削除
                                if (LogView.Items.Count > dispRowMax)
                                {
                                    LogView.Items.RemoveAt(0);
                                }
                            }
                            LogView.EnsureVisible(LogView.Items.Count - 1);

                            FilePathBox.Text = logfile.GetFilePath();       // ファイルパス更新
                        }
                        catch (Exception)
                        {
                            // retire to update
                        }
                        LogView.EndUpdate();
                    });
                    //
                    // ログファイル出力
                    //
                    try
                    {
                        while(true)
                        {
                            var msg = fileoutput.Dequeue();     // 未出力のメッセージをFIFOから取り出し
                            if (msg != null)
                            {
                                String record = "";
                                int col = 0;
                                foreach(var i in msg)           // 出力する一行を組み立てます
                                {
                                    if( col++ > 0)
                                        record += ",";
                                    record += i.ToString().TrimEnd('\r','\n');  // 不要な改行はカット
                                }
                                logfile.Out(record);   // ファイルに出力します
                            }
                        }
                    }
                    catch (Exception)
                    {
                        // 例外は無視します
                    }
                }
                try
                {
                    // Service Live indicator 
                    EGAPI.EgTagWrite(TAG_LIVE, ++Live);
                }
                catch (Exception)
                { }
            };
            drawtimer.Enabled = true;

            // メールボックスからログ取得
            await Task.Run(() =>
            {
                try
                {
                    if (IsActionF)
                    {
                        // Service Run indicator 
                        EGAPI.EgTagWrite(TAG_RUN, (bool)false);
                        return;
                    }
                    // Service Run indicator 
                    EGAPI.EgTagWrite(TAG_RUN, (bool)true);
                    // Service Error indicator 
                    EGAPI.EgTagWrite(TAG_ERROR, (bool)false);
                }
                catch (Exception) { }

                int rtn = 0;
                IsActionF = true;
                while (true)
                {
                    if (!IsActionF)
                        break;
                    //
                    // FIFOReadでデータを取得
                    //
                    while (true)
                    {
                        if (!IsActionF)
                            break;

                        byte[] data = new byte[2048];
                        rtn = EGAPI._EgMailboxReceive(EGDEFINE.EG_MBOX_NAME_LOG, ref data, 0);
                        if (data.Length > 0 && rtn == 0)
                        {
                            string stValue = System.Text.Encoding.GetEncoding("shift_jis").GetString(data);
                            stValue = stValue.TrimEnd('\0');
                            // \0以降を削除
                            string outStr = "";
                            int p = stValue.IndexOf('\0');
                            if (p < 0) p = stValue.Length;
                            outStr = stValue.Substring(0, p);
                            stValue = outStr.TrimEnd('\0');

                            // 読み込んだ一行をカンマ毎に分けて配列に格納する
                            string[] values = stValue.Split(',');

                            // 配列からリストに格納する
                            List<string> lists = new List<string>();
                            lists.AddRange(values);

                            // 項目分繰り返す
                            for (int i = 0; i < lists.Count; ++i)
                            {
                                //先頭のスペースを除去して、ダブルクォーテーションが入っていないか判定する
                                if (lists[i] != string.Empty && lists[i].TrimStart()[0] == '"')
                                {
                                    // もう一回ダブルクォーテーションが出てくるまで要素を結合
                                    while (true)
                                    {
                                        int size = lists[i].TrimEnd().Length - 1;
                                        if (lists[i].TrimEnd()[size] != '"')
                                        {
                                            lists[i] = lists[i] + "," + lists[i + 1];

                                            //結合したら要素を削除する
                                            lists.RemoveAt(i + 1);
                                        }
                                        else
                                            break;
                                    }
                                }
                            }
                            List<string> newList = new List<string>();
                            // 次にダブルクォテーションを消す
                            foreach ( var item in lists )
                            {
                                item.Replace(Environment.NewLine, "");          // 改行文字をとり除く
                                newList.Add(TrimDoubleQuotationMarks(item));
                            }

                            try
                            {
                                logMessList.Enqueue(newList);               // 表示リストを更新
                            }
                            catch { }
                            try
                            {
                                fileoutput.Enqueue(newList);                // ファイル出力候補に追加
                            }
                            catch { }
                        }
                        else
                        {
                            System.Threading.Thread.Sleep(500);         // any mailbox receive interval
                            break;
                        }
                    }
                }
            });
        }

        /// <summary>
        /// ダブルクォテーション削除
        /// </summary>
        /// <param name="target"></param>
        /// <returns></returns>
        public static string TrimDoubleQuotationMarks(string target)
        {
            return target.Trim(new char[] { '"' });
        }

        /// <summary>
        /// 終了関数
        /// </summary>
        private void fnStop()
        {
            Close();
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        private void CloseProcess()
        {
            try
            {
                EGAPI.EgFinalize();
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// ウィンドウ表示処理
        /// </summary>
        private void DispWindow()
        {
            this.WindowState = FormWindowState.Normal;
            this.Visible = true;
            this.ShowInTaskbar = true;
            this.Activate();
        }

        #region イベント

        /// <summary>
        /// ユーザーメッセージハンドラ
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="messNo"></param>
        /// <param name="paramArray"></param>
        /// <returns></returns>
        public int UserMessageHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                switch (messNo)
                {
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:
                        {
                            //終了関数呼び出し
                            fnStop();
                        }
                        break;
                    default:
                        return -1;
                }
                return 0;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        #region 未使用
        private void CloseButton_Click(object sender, EventArgs e)
        {
            // 本メソッドは、未使用（ボタンを非表示としている）
            Close();
        }

        private void DrawButton_Click(object sender, EventArgs e)
        {
            // 本メソッドは、未使用（ボタンを非表示としている）
            if (IsActionF == false)
                GetLog();
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            // 本メソッドは、未使用（ボタンを非表示としている）
            if (IsActionF == true)
                IsActionF = false;
        }
        #endregion

        private void Form_Load(object sender, EventArgs e)
        {
            //詳細表示にする
            LogView.View = View.Details;
            // 列を追加する（列に表示する文字列、幅、列に表示する文字の配置位置）
            LogView.Columns.Add("Date", 160, HorizontalAlignment.Left);
            LogView.Columns.Add("Writer", 80, HorizontalAlignment.Left);
            LogView.Columns.Add("Service", 80, HorizontalAlignment.Left);
            LogView.Columns.Add("Kind", 80, HorizontalAlignment.Left);
            LogView.Columns.Add("Message", 500, HorizontalAlignment.Left);

            if (IsActionF == false)
                GetLog();
        }

        private void Form_Closed(object sender, FormClosedEventArgs e)
        {
            IsActionF = false;
            System.Threading.Thread.Sleep(600);
            CloseProcess();
        }

        private void Form_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                // フォームが最小化の状態であればフォームを非表示にする
                this.WindowState = FormWindowState.Normal;
                this.Visible = false;
                this.ShowInTaskbar = false;
            }
        }

        /// <summary>
        /// 常駐アイコン ダブルクリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void notifyIcon1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            DispWindow();   // ウィンドウ表示
        }

        /// <summary>
        /// 常駐アイコン 表示メニュー押下時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void 表示ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DispWindow();   // ウィンドウ表示
        }

        /// <summary>
        /// 常駐アイコン 終了メニュー押下時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void 終了ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion
    }

    /// <summary>
    /// ログファイル出力処理クラス
    /// </summary>
    public class LogFile
    {
        public string OutDir;

        public LogFile()
        {
            OutDir = GetDefaultDir();
        }
        public string GetFilePath()
        {
            return Path.Combine(OutDir,GetFileName());
        }
        private string GetFileName()
        {
            return "EgLog" + GetToday().Date.Day + ".log";  // 日付でファイル名を作ります
        }
        public string GetDefaultDir()
        {
            return Directory.GetParent(Assembly.GetExecutingAssembly().Location).ToString();    // EgLog.exe位置がデフォルト出力先
        }
        private DateTime GetToday()
        {
            return DateTime.Today;
        }
        private bool CheckFile(string filepath)
        {
            try
            {
                DateTime filedate = File.GetCreationTime(filepath).Date;
                var diff = (GetToday() - filedate.Date).TotalDays;
                if (diff <= 1)          // ファイルタイムスタンプ差が一日以内なら削除しない
                    return false;
                File.Delete(filepath);  // 古いログを削除する
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
        public bool Out(string message)
        {
            try
            {
                string filepath = GetFilePath();          // ファイル出力先パスを組み立て
                CheckFile(filepath);                                            // ファイルが古かったら削除
                File.AppendAllText(filepath, message + Environment.NewLine);    // アペンドで書出し
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
    }

}
